@@ -33,6 +33,7 @@
 #include "pixmaps/portrait.xpm"
 #include "pixmaps/landscape.xpm"
 
+#include "preferences.h"
 #include "paper.h"
 
 enum {
@@ -59,8 +60,9 @@ dia_page_layout_get_type(void)
       sizeof(DiaPageLayoutClass),
       (GtkClassInitFunc) dia_page_layout_class_init,
       (GtkObjectInitFunc) dia_page_layout_init,
-      (GtkArgSetFunc) NULL,
-      (GtkArgGetFunc) NULL
+      NULL,
+      NULL,
+      (GtkClassInitFunc) NULL,
     };
     pl_type = gtk_type_unique(gtk_table_get_type(), &pl_info);
   }
@@ -78,11 +80,13 @@ dia_page_layout_class_init(DiaPageLayoutClass *class)
   pl_signals[CHANGED] =
     gtk_signal_new("changed",
 		   GTK_RUN_FIRST,
-		   object_class->type,
+		   GTK_CLASS_TYPE (object_class),
 		   GTK_SIGNAL_OFFSET(DiaPageLayoutClass, changed),
 		   gtk_signal_default_marshaller,
 		   GTK_TYPE_NONE, 0);
+#if 0 /* FIXME ?*/
   gtk_object_class_add_signals(object_class, pl_signals, LAST_SIGNAL);
+#endif
 
   object_class->destroy = dia_page_layout_destroy;
 }
@@ -129,8 +133,8 @@ dia_page_layout_init(DiaPageLayout *self)
        i++, paper_names = g_list_next(paper_names)) {
     menuitem = gtk_menu_item_new_with_label(paper_names->data);
     gtk_object_set_user_data(GTK_OBJECT(menuitem), GINT_TO_POINTER(i));
-    gtk_signal_connect(GTK_OBJECT(menuitem), "activate",
-		       GTK_SIGNAL_FUNC(paper_size_change), self);
+    g_signal_connect(GTK_OBJECT(menuitem), "activate",
+		     G_CALLBACK(paper_size_change), self);
     gtk_container_add(GTK_CONTAINER(menu), menuitem);
     gtk_widget_show(menuitem);
   }
@@ -300,33 +304,33 @@ dia_page_layout_init(DiaPageLayout *self)
   gtk_widget_show(self->darea);
 
   /* connect the signal handlers */
-  gtk_signal_connect_object(GTK_OBJECT(self->orient_portrait), "toggled",
-			    GTK_SIGNAL_FUNC(orient_changed), GTK_OBJECT(self));
+  g_signal_connect_swapped(GTK_OBJECT(self->orient_portrait), "toggled",
+			  G_CALLBACK(orient_changed), GTK_OBJECT(self));
 
-  gtk_signal_connect_object(GTK_OBJECT(self->tmargin), "changed",
-			    GTK_SIGNAL_FUNC(margin_changed), GTK_OBJECT(self));
-  gtk_signal_connect_object(GTK_OBJECT(self->bmargin), "changed",
-			    GTK_SIGNAL_FUNC(margin_changed), GTK_OBJECT(self));
-  gtk_signal_connect_object(GTK_OBJECT(self->lmargin), "changed",
-			    GTK_SIGNAL_FUNC(margin_changed), GTK_OBJECT(self));
-  gtk_signal_connect_object(GTK_OBJECT(self->rmargin), "changed",
-			    GTK_SIGNAL_FUNC(margin_changed), GTK_OBJECT(self));
+  g_signal_connect_swapped(GTK_OBJECT(self->tmargin), "changed",
+			   G_CALLBACK(margin_changed), GTK_OBJECT(self));
+  g_signal_connect_swapped(GTK_OBJECT(self->bmargin), "changed",
+			   G_CALLBACK(margin_changed), GTK_OBJECT(self));
+  g_signal_connect_swapped(GTK_OBJECT(self->lmargin), "changed",
+			   G_CALLBACK(margin_changed), GTK_OBJECT(self));
+  g_signal_connect_swapped(GTK_OBJECT(self->rmargin), "changed",
+			   G_CALLBACK(margin_changed), GTK_OBJECT(self));
 
-  gtk_signal_connect_object(GTK_OBJECT(self->fitto), "toggled",
-			    GTK_SIGNAL_FUNC(scalemode_changed),
+  g_signal_connect_swapped(GTK_OBJECT(self->fitto), "toggled",
+			   G_CALLBACK(scalemode_changed),
 			    GTK_OBJECT(self));
-  gtk_signal_connect_object(GTK_OBJECT(self->scaling), "changed",
-			    GTK_SIGNAL_FUNC(scale_changed), GTK_OBJECT(self));
-  gtk_signal_connect_object(GTK_OBJECT(self->fitw), "changed",
-			    GTK_SIGNAL_FUNC(scale_changed), GTK_OBJECT(self));
-  gtk_signal_connect_object(GTK_OBJECT(self->fith), "changed",
-			    GTK_SIGNAL_FUNC(scale_changed), GTK_OBJECT(self));
+  g_signal_connect_swapped(GTK_OBJECT(self->scaling), "changed",
+			   G_CALLBACK(scale_changed), GTK_OBJECT(self));
+  g_signal_connect_swapped(GTK_OBJECT(self->fitw), "changed",
+			   G_CALLBACK(scale_changed), GTK_OBJECT(self));
+  g_signal_connect_swapped(GTK_OBJECT(self->fith), "changed",
+			   G_CALLBACK(scale_changed), GTK_OBJECT(self));
 
-  gtk_signal_connect_object(GTK_OBJECT(self->darea), "size_allocate",
-			    GTK_SIGNAL_FUNC(darea_size_allocate),
+  g_signal_connect_swapped(GTK_OBJECT(self->darea), "size_allocate",
+			   G_CALLBACK(darea_size_allocate),
 			    GTK_OBJECT(self));
-  gtk_signal_connect_object(GTK_OBJECT(self->darea), "expose_event",
-			    GTK_SIGNAL_FUNC(darea_expose_event),
+  g_signal_connect_swapped(GTK_OBJECT(self->darea), "expose_event",
+			   G_CALLBACK(darea_expose_event),
 			    GTK_OBJECT(self));
 
   gdk_color_white(gtk_widget_get_colormap(GTK_WIDGET(self)), &self->white);
@@ -362,7 +366,7 @@ dia_page_layout_set_paper(DiaPageLayout *self, const gchar *paper)
 
   i = find_paper(paper);
   if (i == -1)
-    i = get_default_paper();
+    i = find_paper(prefs.new_diagram.papertype);
   gtk_option_menu_set_history(GTK_OPTION_MENU(self->paper_size), i);
   gtk_menu_item_activate(
 	GTK_MENU_ITEM(GTK_OPTION_MENU(self->paper_size)->menu_item));
@@ -500,7 +504,7 @@ dia_page_layout_get_paper_size(const gchar *paper,
 
   i = find_paper(paper);
   if (i == -1)
-    i = get_default_paper();
+    i = find_paper(prefs.new_diagram.papertype);
   if (width)
     *width = get_paper_pswidth(i);
   if (height)
@@ -516,7 +520,7 @@ dia_page_layout_get_default_margins(const gchar *paper,
 
   i = find_paper(paper);
   if (i == -1)
-    i = get_default_paper();
+    i = find_paper(prefs.new_diagram.papertype);
   if (tmargin)
     *tmargin = get_paper_tmargin(i);
   if (bmargin)
@@ -749,8 +753,10 @@ dia_page_layout_destroy(GtkObject *object)
 {
   DiaPageLayout *self = DIA_PAGE_LAYOUT(object);
 
-  if (self->gc)
+  if (self->gc) {
     gdk_gc_unref(self->gc);
+    self->gc = NULL;
+  }
 
   if (parent_class->destroy)
     (* parent_class->destroy)(object);
@@ -778,18 +784,18 @@ main(int argc, char **argv)
 
   win = gtk_window_new(GTK_WINDOW_TOPLEVEL);
   gtk_window_set_title(GTK_WINDOW(win), _("Page Setup"));
-  gtk_signal_connect(GTK_OBJECT(win), "destroy",
-		     GTK_SIGNAL_FUNC(gtk_main_quit), NULL);
+  g_signal_connect(GTK_OBJECT(win), "destroy",
+		   G_CALLBACK(gtk_main_quit), NULL);
 
   pl = dia_page_layout_new();
   gtk_container_set_border_width(GTK_CONTAINER(pl), 5);
   gtk_container_add(GTK_CONTAINER(win), pl);
   gtk_widget_show(pl);
 
-  gtk_signal_connect(GTK_OBJECT(pl), "changed",
-		     GTK_SIGNAL_FUNC(changed_signal), NULL);
-  gtk_signal_connect(GTK_OBJECT(pl), "fittopage",
-		     GTK_SIGNAL_FUNC(fittopage_signal), NULL);
+  g_signal_connect(GTK_OBJECT(pl), "changed",
+		   G_CALLBACK(changed_signal), NULL);
+  g_signal_connect(GTK_OBJECT(pl), "fittopage",
+		   G_CALLBACK(fittopage_signal), NULL);
 
   gtk_widget_show(win);
   gtk_main();
