@@ -29,10 +29,21 @@ typedef struct _DDisplay DDisplay;
 #include "menus.h"
 #include "diarenderer.h"
 
+/** Defines the pixels per cm, default is 20 pixels = 1 cm */
+/** This is close to, but not quite the same as, the physical display size
+ * in most cases */
+
 #define DDISPLAY_MAX_ZOOM 500.0
 #define DDISPLAY_NORMAL_ZOOM 20.0
 #define DDISPLAY_MIN_ZOOM 1.0
 
+/* The zoom amount should be uniform.  Pixels per cm should be defined by the
+ * renderer alone.  But that'd take a lot of fiddling in renderers. */
+/*
+#define DDISPLAY_MAX_ZOOM 100.0
+#define DDISPLAY_NORMAL_ZOOM 1.0
+#define DDISPLAY_MIN_ZOOM 0.01
+*/
 struct _DDisplay {
   Diagram *diagram;               /* pointer to the associated diagram */
 
@@ -56,6 +67,7 @@ struct _DDisplay {
     
 
   GtkWidget *zoom_status;         
+  GtkWidget *grid_status;
   GtkWidget *modified_status;
 
   GtkAccelGroup *accel_group;
@@ -80,6 +92,10 @@ struct _DDisplay {
   guint update_id;                /* idle handler ID for redraws       */
 
   GtkIMContext *im_context;
+
+  /* Preedit String */
+  gchar *preedit_string;
+  PangoAttrList *preedit_attrs;
 };
 
 extern GdkCursor *default_cursor;
@@ -111,7 +127,7 @@ void ddisplay_set_origo(DDisplay *ddisp,
 			coord x, coord y);
 void ddisplay_zoom(DDisplay *ddisp, Point *point,
 		   real zoom_factor);
-
+void ddisplay_set_snap_to_grid(DDisplay *ddisp, gboolean snap);
 void ddisplay_set_renderer(DDisplay *ddisp, int aa_renderer);
 void ddisplay_resize_canvas(DDisplay *ddisp,
 			    int width,
@@ -134,6 +150,8 @@ void ddisplay_scroll_up(DDisplay *ddisp);
 void ddisplay_scroll_down(DDisplay *ddisp);
 void ddisplay_scroll_left(DDisplay *ddisp);
 void ddisplay_scroll_right(DDisplay *ddisp);
+gboolean ddisplay_scroll_center_point(DDisplay *ddisp, Point *p);
+gboolean ddisplay_scroll_to_object(DDisplay *ddisp, Object *obj);
 
 void display_update_menu_state(DDisplay *ddisp);
 void ddisplay_update_statusbar(DDisplay *ddisp);
@@ -141,4 +159,6 @@ void ddisplay_do_update_menu_sensitivity (DDisplay *ddisp);
 
 void display_set_active(DDisplay *ddisp);
 
+void ddisplay_im_context_preedit_reset(DDisplay *ddisp, Focus *focus);
+
 #endif /* DDISPLAY_H */
