@@ -27,12 +27,14 @@
 #include <locale.h>
 #include <string.h> /* strlen */
 #include <signal.h>
+#include <errno.h>
 
 #include "intl.h"
 #include "message.h"
 #include "diagram.h"
 #include "diagramdata.h"
 #include "render_eps.h"
+#include "diapsrenderer.h"
 #include "paginate_psprint.h"
 #include "diapagelayout.h"
 
@@ -57,21 +59,24 @@ static dia_print_options last_print_options =
 };
 
 static void
-count_objs(Object *obj, Renderer *renderer, int active_layer, guint *nobjs)
+count_objs(Object *obj, DiaRenderer *renderer, int active_layer, guint *nobjs)
 {
   (*nobjs)++;
 }
 
 static guint
-print_page(DiagramData *data, RendererEPS *rend, Rectangle *bounds)
+print_page(DiagramData *data, DiaRenderer *diarend, Rectangle *bounds)
 {
+  DiaPsRenderer *rend = DIA_PS_RENDERER(diarend);
   guint nobjs = 0;
   gfloat tmargin = data->paper.tmargin, bmargin = data->paper.bmargin;
   gfloat lmargin = data->paper.lmargin;
   gfloat scale = data->paper.scaling;
 
+  rend->paper = data->paper.name;
+
   /* count the number of objects in this region */
-  data_render(data, (Renderer *)rend, bounds,
+  data_render(data, diarend, bounds,
               (ObjectRenderer) count_objs, &nobjs);
 
   if (nobjs == 0)
@@ -102,10 +107,12 @@ print_page(DiagramData *data, RendererEPS *rend, Rectangle *bounds)
   fprintf(rend->file, "%f %f l ", bounds->right, bounds->bottom);
   fprintf(rend->file, "%f %f l ", bounds->left, bounds->bottom);
   fprintf(rend->file, "%f %f l ", bounds->left, bounds->top);
-  fprintf(rend->file, "clip\n");
+  /* Tip from Dov Grobgeld: Clip does not destroy the path, so we should
+     do a newpath afterwards */
+  fprintf(rend->file, "clip n\n"); 
 
   /* render the region */
-  data_render(data, (Renderer *)rend, bounds, NULL, NULL);
+  data_render(data, diarend, bounds, NULL, NULL);
 
   /* restore print context */
   fprintf(rend->file, "gr\n");
@@ -119,7 +126,7 @@ print_page(DiagramData *data, RendererEPS *rend, Rectangle *bounds)
 void
 paginate_psprint(Diagram *dia, FILE *file)
 {
-  RendererEPS *rend;
+  DiaRenderer *rend;
   Rectangle *extents;
   gfloat width, height;
   gfloat x, y, initx, inity;
@@ -129,9 +136,11 @@ paginate_psprint(Diagram *dia, FILE *file)
   old_locale = setlocale(LC_NUMERIC, "C");
   rend = new_psprint_renderer(dia, file);
 
+#ifdef DIA_PS_RENDERER_DUAL_PASS
   /* Prepare the prolog (with fonts etc) */
-  data_render(dia->data, (Renderer *)rend, NULL, NULL, NULL);
+  data_render(dia->data, DIA_RENDERER(rend), NULL, NULL, NULL);
   eps_renderer_prolog_done(rend);
+#endif
 
   /* the usable area of the page */
   width = dia->data->paper.width;
@@ -160,7 +169,7 @@ paginate_psprint(Diagram *dia, FILE *file)
       nobjs += print_page(dia->data,rend, &page_bounds);
     }
 
-  destroy_eps_renderer(rend);
+  g_object_unref(rend);
 
   setlocale(LC_NUMERIC, old_locale);
 }
@@ -195,6 +204,7 @@ diagram_print_ps(Diagram *dia)
   GtkWidget *cmd, *ofile;
   gboolean cont = FALSE;
   DDisplay *ddisp;
+  gchar *printcmd = NULL;
 
   FILE *file;
   gboolean is_pipe;
@@ -211,10 +221,10 @@ diagram_print_ps(Diagram *dia)
 
   /* create the dialog */
   dialog = gtk_dialog_new();
-  gtk_signal_connect(GTK_OBJECT(dialog), "delete_event",
-		     GTK_SIGNAL_FUNC(gtk_main_quit), NULL);
-  gtk_signal_connect(GTK_OBJECT(dialog), "delete_event",
-		     GTK_SIGNAL_FUNC(gtk_true), NULL);
+  g_signal_connect(GTK_OBJECT(dialog), "delete_event",
+		   G_CALLBACK(gtk_main_quit), NULL);
+  g_signal_connect(GTK_OBJECT(dialog), "delete_event",
+		   G_CALLBACK(gtk_true), NULL);
   vbox = GTK_DIALOG(dialog)->vbox;
 
   frame = gtk_frame_new(_("Select Printer"));
@@ -238,8 +248,8 @@ diagram_print_ps(Diagram *dia)
   gtk_table_attach(GTK_TABLE(table), cmd, 1,2, 0,1,
 		   GTK_FILL|GTK_EXPAND, GTK_FILL|GTK_EXPAND, 0, 0);
   gtk_widget_show(cmd);
-  gtk_signal_connect(GTK_OBJECT(iscmd), "toggled",
-		     GTK_SIGNAL_FUNC(change_entry_state), cmd);
+  g_signal_connect(GTK_OBJECT(iscmd), "toggled",
+		   G_CALLBACK(change_entry_state), cmd);
 
   isofile = gtk_radio_button_new_with_label(GTK_RADIO_BUTTON(iscmd)->group,
 					    _("File"));
@@ -252,22 +262,22 @@ diagram_print_ps(Diagram *dia)
   gtk_table_attach(GTK_TABLE(table), ofile, 1,2, 1,2,
 		   GTK_FILL|GTK_EXPAND, GTK_FILL|GTK_EXPAND, 0, 0);
   gtk_widget_show(ofile);
-  gtk_signal_connect(GTK_OBJECT(isofile), "toggled",
-		     GTK_SIGNAL_FUNC(change_entry_state), ofile);
+  g_signal_connect(GTK_OBJECT(isofile), "toggled",
+		   G_CALLBACK(change_entry_state), ofile);
 
   box = GTK_DIALOG(dialog)->action_area;
 
   button = gtk_button_new_with_label(_("OK"));
-  gtk_signal_connect(GTK_OBJECT(button), "clicked", 
-		     GTK_SIGNAL_FUNC(ok_pressed), &cont);
+  g_signal_connect(GTK_OBJECT(button), "clicked", 
+		   G_CALLBACK(ok_pressed), &cont);
   GTK_WIDGET_SET_FLAGS(button, GTK_CAN_DEFAULT);
   gtk_box_pack_start(GTK_BOX(box), button, TRUE, TRUE, 0);
   gtk_widget_grab_default(button);
   gtk_widget_show(button);
 
   button = gtk_button_new_with_label(_("Cancel"));
-  gtk_signal_connect(GTK_OBJECT(button), "clicked", 
-		     GTK_SIGNAL_FUNC(gtk_main_quit), NULL);
+  g_signal_connect(GTK_OBJECT(button), "clicked", 
+		   G_CALLBACK(gtk_main_quit), NULL);
   GTK_WIDGET_SET_FLAGS(button, GTK_CAN_DEFAULT);
   gtk_box_pack_start(GTK_BOX(box), button, TRUE, TRUE, 0);
   gtk_widget_show(button);
@@ -278,9 +288,21 @@ diagram_print_ps(Diagram *dia)
 		     last_print_options.command 
 		     ? last_print_options.command : win32_printer_default ());
 #else
-  gtk_entry_set_text(GTK_ENTRY(cmd), 
-		     last_print_options.command 
-		     ? last_print_options.command : "lpr");
+  if (last_print_options.command) {
+    gtk_entry_set_text(GTK_ENTRY(cmd), last_print_options.command);
+  } else {
+    gchar *printer = g_getenv("PRINTER");
+
+    if (printer) {
+      printcmd = g_strdup_printf("lpr -P%s", printer);
+    } else {
+      printcmd = g_strdup("lpr");
+    }
+    
+    gtk_entry_set_text(GTK_ENTRY(cmd), printcmd);
+    g_free(printcmd);
+    printcmd = NULL;
+  }
 #endif
   /* Ought to use filename+extension here */
   gtk_entry_set_text(GTK_ENTRY(ofile), 
@@ -299,14 +321,15 @@ diagram_print_ps(Diagram *dia)
   }
 
   if (gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(iscmd))) {
+    printcmd = g_strdup(gtk_entry_get_text(GTK_ENTRY(cmd)));
 #ifdef G_OS_WIN32
-    file = win32_printer_open (gtk_entry_get_text(GTK_ENTRY(cmd)));
+    file = win32_printer_open (printcmd);
 #else
-    file = popen(gtk_entry_get_text(GTK_ENTRY(cmd)), "w");
+    file = popen(printcmd, "w");
 #endif
     is_pipe = TRUE;
   } else {
-    gchar *filename = gtk_entry_get_text(GTK_ENTRY(ofile));
+    const gchar *filename = gtk_entry_get_text(GTK_ENTRY(ofile));
     if (!g_path_is_absolute(filename)) {
       char *diagram_dir;
       char *full_filename;
@@ -325,17 +348,19 @@ diagram_print_ps(Diagram *dia)
   /* Store dialog values */
   g_free( last_print_options.command );
   g_free( last_print_options.output );
-  last_print_options.command = g_strdup( gtk_entry_get_text(GTK_ENTRY(cmd)) );
+  if (printcmd)
+    last_print_options.command = g_strdup( printcmd );
   last_print_options.output = g_strdup( gtk_entry_get_text(GTK_ENTRY(ofile)) );
   last_print_options.printer = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(iscmd));
   
   if (!file) {
-    if (is_pipe)
-      message_warning(_("Could not run command '%s'"),
-		      gtk_entry_get_text(GTK_ENTRY(cmd)));
-    else
-      message_warning(_("Could not open '%s' for writing"),
-		      gtk_entry_get_text(GTK_ENTRY(ofile)));
+    if (is_pipe) {
+      message_warning(_("Could not run command '%s': %s"), printcmd, strerror(errno));
+      g_free(printcmd);
+    } else
+      message_warning(_("Could not open '%s' for writing: %s"),
+		      gtk_entry_get_text(GTK_ENTRY(ofile)), strerror(errno));
+    gtk_widget_destroy(dialog);
     return;
   }
 
@@ -349,14 +374,21 @@ diagram_print_ps(Diagram *dia)
 
   paginate_psprint(dia, file);
   gtk_widget_destroy(dialog);
-  if (is_pipe)
-    pclose(file);
-  else
+  if (is_pipe) {
+    int exitval = pclose(file);
+    if (exitval != 0) {
+      message_error(_("Printing error: command '%s' returned %d\n"),
+		    printcmd, exitval);
+    }
+  } else
     fclose(file);
 
 #ifndef G_OS_WIN32
   sigaction(SIGPIPE, &old_action, NULL);
 #endif
   if (sigpipe_received)
-    message_error(_("Error occured while printing"));
+    message_error(_("Printing error: command '%s' caused sigpipe."),
+		  printcmd);
+
+  if (is_pipe) g_free(printcmd);
 }
