@@ -23,6 +23,11 @@
 #include "dia_dirs.h"
 #ifdef G_OS_WIN32
 #include <windows.h>
+#include <direct.h>
+#define mkdir(s,a) _mkdir(s)
+#else
+#include <sys/stat.h>
+#include <sys/types.h>
 #endif
 
 gchar*
@@ -94,3 +99,45 @@ dia_config_filename(const gchar *subfile)
   return g_strconcat(homedir, G_DIR_SEPARATOR_S ".dia" G_DIR_SEPARATOR_S,
 		     subfile, NULL);
 }
+
+/** Ensure that the directory part of `filename' exists.
+ * Returns TRUE if the directory existed or has been created.
+ */
+gboolean
+dia_config_ensure_dir(const gchar *filename)
+{
+  gchar *dir = g_path_get_dirname(filename);
+  gboolean exists;
+  if (dir == NULL) return FALSE;
+  if (strcmp(dir, ".")) {
+    if (g_file_test(dir, G_FILE_TEST_IS_DIR)) {
+      exists = TRUE;
+    } else {
+      if (dia_config_ensure_dir(dir)) {
+	exists = (mkdir(dir, 0755) == 0);
+      } else {
+	exists = FALSE;
+      }
+    }
+  } else {
+    exists = FALSE;
+  }
+  g_free(dir);
+  return exists;
+}
+
+/** Return an absolute filename from an absolute or relative filename.
+ * The value returned is newly allocated. 
+ */
+gchar *
+dia_get_absolute_filename (const gchar *filename)
+{
+  gchar *current_dir;
+  gchar *fullname;
+  if (filename == NULL) return NULL;
+  if (g_path_is_absolute(filename)) return g_strdup(filename);
+  current_dir = g_get_current_dir();
+  fullname = g_build_filename(current_dir, filename, NULL);
+  g_free(current_dir);
+  return fullname;
+}
