@@ -55,7 +55,6 @@ begin_render(DiaRenderer *self)
   DiaSvgRenderer *renderer = DIA_SVG_RENDERER (self);
 
   renderer->linewidth = 0;
-  renderer->fontsize = 1.0;
   renderer->linecap = "butt";
   renderer->linejoin = "miter";
   renderer->linestyle = NULL;
@@ -186,8 +185,6 @@ set_dashlength(DiaRenderer *self, real length)
 static void
 set_fillstyle(DiaRenderer *self, FillStyle mode)
 {
-  DiaSvgRenderer *renderer = DIA_SVG_RENDERER (self);
-
   switch(mode) {
   case FILLSTYLE_SOLID:
     break;
@@ -196,17 +193,6 @@ set_fillstyle(DiaRenderer *self, FillStyle mode)
   }
 }
 
-static void
-set_font(DiaRenderer *self, DiaFont *font, real height)
-{
-  DiaSvgRenderer *renderer = DIA_SVG_RENDERER (self);
-
-  renderer->fontsize = height;
-  if (renderer->font) 
-    dia_font_unref(renderer->font);  
-  renderer->font = dia_font_ref(font);
-}
-
 /* the return value of this function should not be saved anywhere */
 static const gchar *
 get_draw_style(DiaSvgRenderer *renderer,
@@ -220,7 +206,7 @@ get_draw_style(DiaSvgRenderer *renderer,
 
   /* TODO(CHECK): the shape-export didn't have 'fill: none' here */
   old_locale = setlocale(LC_NUMERIC, "C");
-  g_string_sprintf(str, "fill: none; stroke-width: %g", renderer->linewidth);
+  g_string_sprintf(str, "fill: none; fill-opacity:0; stroke-width: %g", renderer->linewidth);
   setlocale(LC_NUMERIC, old_locale);
   if (strcmp(renderer->linecap, "butt"))
     g_string_sprintfa(str, "; stroke-linecap: %s", renderer->linecap);
@@ -314,7 +300,7 @@ draw_polygon(DiaRenderer *self,
   GString *str;
   char *old_locale;
 
-  node = xmlNewChild(renderer->root, renderer->svg_name_space, "polyline", NULL);
+  node = xmlNewChild(renderer->root, renderer->svg_name_space, "polygon", NULL);
   
   xmlSetProp(node, "style", get_draw_style(renderer, line_colour));
 
@@ -338,7 +324,7 @@ fill_polygon(DiaRenderer *self,
   GString *str;
   char *old_locale;
 
-  node = xmlNewChild(renderer->root, renderer->svg_name_space, "polyline", NULL);
+  node = xmlNewChild(renderer->root, renderer->svg_name_space, "polygon", NULL);
   
   xmlSetProp(node, "style", get_fill_style(renderer, colour));
 
@@ -670,29 +656,32 @@ draw_string(DiaRenderer *self,
   style = (char*)get_fill_style(renderer, colour);
   /* return value must not be freed */
   renderer->linewidth = saved_width;
+  /* This is going to break for non-LTR texts, as SVG thinks 'start' is
+   * 'right' for those.
+   */
   switch (alignment) {
   case ALIGN_LEFT:
-    style = g_strconcat(style, "; text-align: left", NULL);
+    style = g_strconcat(style, "; text-anchor:start", NULL);
     break;
   case ALIGN_CENTER:
-    style = g_strconcat(style, "; text-align: center", NULL);
+    style = g_strconcat(style, "; text-anchor:middle", NULL);
     break;
   case ALIGN_RIGHT:
-    style = g_strconcat(style, "; text-align: right", NULL);
+    style = g_strconcat(style, "; text-anchor:end", NULL);
     break;
   }
   old_locale = setlocale(LC_NUMERIC, "C");
-  tmp = g_strdup_printf("%s; font-size: %g", style, renderer->fontsize);
+  tmp = g_strdup_printf("%s; font-size: %g", style, self->font_height);
   setlocale(LC_NUMERIC, old_locale);
   g_free (style);
   style = tmp;
 
-  if (renderer->font) {
+  if (self->font) {
      tmp = g_strdup_printf("%s; font-family: %s; font-style: %s; "
                            "font-weight: %s",style,
-                           dia_font_get_family(renderer->font),
-                           dia_font_get_slant_string(renderer->font),
-                           dia_font_get_weight_string(renderer->font));
+                           dia_font_get_family(self->font),
+                           dia_font_get_slant_string(self->font),
+                           dia_font_get_weight_string(self->font));
      g_free(style);
      style = tmp;
   }
@@ -740,9 +729,6 @@ draw_image(DiaRenderer *self,
 static void
 dia_svg_renderer_init (GTypeInstance   *instance, gpointer g_class)
 {
-  DiaSvgRenderer *renderer = DIA_SVG_RENDERER (instance);
-
-  
 }
 
 static gpointer parent_class = NULL;
@@ -751,11 +737,6 @@ static gpointer parent_class = NULL;
 static void
 dia_svg_renderer_finalize (GObject *object)
 {
-  DiaSvgRenderer *renderer = DIA_SVG_RENDERER (object);
-
-  if (renderer->font) 
-    dia_font_unref(renderer->font);  
-
   G_OBJECT_CLASS (parent_class)->finalize (object);
 }
 
