@@ -24,7 +24,6 @@
 
 #include "object.h"
 #include "message.h"
-#include "sheet.h"
 
 #include "dummy_dep.h"
 
@@ -63,6 +62,7 @@ object_destroy(Object *obj)
 /* After this copying you have to fix up:
    handles
    connections
+   children/parents
 */
 void
 object_copy(Object *from, Object *to)
@@ -81,13 +81,167 @@ object_copy(Object *from, Object *to)
   to->num_connections = from->num_connections;
   if (to->connections != NULL) g_free(to->connections);
   if (to->num_connections>0)
-    to->connections = g_malloc(sizeof(ConnectionPoint *) * to->num_connections);
+    to->connections = g_malloc0(sizeof(ConnectionPoint *) * to->num_connections);
   else
     to->connections = NULL;
 
   to->ops = from->ops;
+
+  to->can_parent = from->can_parent;
+  to->parent = from->parent;;
+  to->children = g_list_copy(from->children);
 }
 
+static guint
+pointer_hash(gpointer some_pointer)
+{
+  return (guint) some_pointer;
+}
+
+GList *
+object_copy_list(GList *list_orig)
+{
+  GList *list_copy;
+  GList *list;
+  Object *obj;
+  Object *obj_copy;
+  GHashTable *hash_table;
+  int i;
+
+  hash_table = g_hash_table_new((GHashFunc) pointer_hash, NULL);
+
+  list = list_orig;
+  list_copy = NULL;
+  while (list != NULL) {
+    obj = (Object *)list->data;
+
+    obj_copy = obj->ops->copy(obj);
+
+    g_hash_table_insert(hash_table, obj, obj_copy);
+    
+    list_copy = g_list_append(list_copy, obj_copy);
+    
+    list = g_list_next(list);
+  }
+
+  /* Rebuild the connections and parent/child references between the
+  objects in the list: */
+  list = list_orig;
+  while (list != NULL) {
+    obj = (Object *)list->data;
+    obj_copy = g_hash_table_lookup(hash_table, obj);
+    
+    if (obj_copy->parent)
+      obj_copy->parent = g_hash_table_lookup(hash_table, obj_copy->parent);
+
+    if (obj_copy->can_parent && obj_copy->children)
+    {
+      GList *child_list = obj_copy->children;
+      while(child_list)
+      {
+        Object *child_obj = (Object *) child_list->data;
+        child_list->data = g_hash_table_lookup(hash_table, child_obj);
+	child_list = g_list_next(child_list);
+      }
+    }
+
+    for (i=0;i<obj->num_handles;i++) {
+      ConnectionPoint *con_point;
+      con_point = obj->handles[i]->connected_to;
+      
+      if ( con_point != NULL ) {
+	Object *other_obj;
+	Object *other_obj_copy;
+	int con_point_nr;
+	
+	other_obj = con_point->object;
+	other_obj_copy = g_hash_table_lookup(hash_table, other_obj);
+
+	if (other_obj_copy == NULL)
+	  break; /* other object was not on list. */
+	
+	con_point_nr=0;
+	while (other_obj->connections[con_point_nr] != con_point) {
+	  con_point_nr++;
+	}
+
+	object_connect(obj_copy, obj_copy->handles[i],
+		       other_obj_copy->connections[con_point_nr]);
+      }
+    }
+    
+    list = g_list_next(list);
+  }
+  
+  g_hash_table_destroy(hash_table);
+  
+  return list_copy;
+}
+
+ObjectChange*
+object_list_move_delta_r(GList *objects, Point *delta, gboolean affected)
+{
+  GList *list;
+  Object *obj;
+  Point pos;
+  ObjectChange *objchange = NULL;
+
+  if (delta->x == 0 && delta->y == 0)
+       return NULL;
+
+  list = objects;
+  while (list != NULL) {
+    obj = (Object *) list->data;
+    
+    pos = obj->position;
+    point_add(&pos, delta);
+
+    if (obj->parent && affected)
+    {
+      Rectangle *p_ext = parent_handle_extents(obj->parent);
+      Rectangle *c_ext = parent_handle_extents(obj);
+      Point new_delta = parent_move_child_delta(p_ext, c_ext, delta);
+      point_add(&pos, &new_delta);
+      point_add(delta, &new_delta);
+
+      g_free(p_ext);
+      g_free(c_ext);
+    }
+    objchange = obj->ops->move(obj, &pos);
+
+    if (obj->can_parent && obj->children)
+      objchange = object_list_move_delta_r(obj->children, delta, FALSE);
+
+    list = g_list_next(list);
+  }
+  return objchange;
+}
+
+extern ObjectChange*
+object_list_move_delta(GList *objects, Point *delta)
+{
+  GList *list;
+  Object *obj;
+  GList *process;
+  ObjectChange *objchange = NULL;
+
+  objects = parent_list_affected_hierarchy(objects);
+  list = objects;
+  /* The recursive function object_list_move_delta cannot process the toplevel
+     (in selection) objects so we have to have this extra loop */
+  while (list != NULL)
+  {
+    obj = (Object *) list->data;
+
+    process = NULL;
+    process = g_list_append(process, obj);
+    objchange = object_list_move_delta_r(process, delta, (obj->parent != NULL) );
+    g_list_free(process);
+
+    list = g_list_next(list);
+  }
+  return objchange;
+}
 
 void
 destroy_object_list(GList *list_to_be_destroyed)
@@ -311,6 +465,27 @@ void object_load(Object *obj, ObjectNode obj_node)
     data_rectangle( attribute_first_data(attr), &obj->bounding_box );
 }
 
+Layer *dia_object_get_parent_layer(Object *obj) {
+  return obj->parent_layer;
+}
+
+/** Returns true if `obj' is currently selected.
+ * Note that this takes time proportional to the number of selected
+ * objects, so don't use it frivolously.
+ */
+gboolean
+dia_object_is_selected (const Object *obj)
+{
+  Layer *layer = obj->parent_layer;
+  DiagramData *diagram = layer->parent_diagram;
+
+  GList *selected = diagram->selected;
+  for (; selected != NULL; selected = g_list_next(selected)) {
+    if (selected->data == obj) return TRUE;
+  }
+  return FALSE;
+}
+
 /****** Object register: **********/
 
 static guint hash(gpointer key)
@@ -353,98 +528,10 @@ object_register_type(ObjectType *type)
   g_hash_table_insert(object_type_table, type->name, type);
 }
 
-static GSList *object_types_nosheet_list = NULL;
-
-static void
-object_type_detect_nosheet_callback(gpointer key, gpointer value,
-                                    gpointer user_data)
+void
+object_registry_foreach (GHFunc func, gpointer user_data)
 {
-  GSList *sheets_list;
-  gboolean found = FALSE;
-  gchar *sheet_name;
-  guint i;
-  static gchar *ignore[] =          /* Ignore ObjectType's with these   */
-  {                                 /* 'name's.  Mostly used for object */
-      "Group",                      /* types that are obsolete and thus */
-                                    /* are not found on any sheet.      */
-      "Contact - if",
-      "Contact - ifnot",
-      "Contact - lamp",
-      "Contact - relay",
-      "GRAFCET - Vector",
-      "UML - Objet",
-      NULL
-  };
-
-  for (i = 0; ignore[i]; i++)
-    if (!strcmp(((ObjectType *)(value))->name, ignore[i]))
-      return;
-
-  /* We also ignore ObjectType's that begin with 'Standard -' */
-
-  sheet_name = object_type_get_sheet_name((ObjectType *)value);
-  if (!strcmp(sheet_name, "Standard"))
-  {
-    g_free(sheet_name);
-    return;
-  }
-  g_free(sheet_name);
-  
-  for (sheets_list = get_sheets_list(); sheets_list && found == FALSE;
-       sheets_list = g_slist_next(sheets_list))
-  {
-    GSList *objects_list = ((Sheet *)(sheets_list)->data)->objects;
-
-    for ( ; objects_list; objects_list = g_slist_next(objects_list))
-    {
-      if (!strcmp(key, ((SheetObject *)(objects_list)->data)->object_type))
-      {
-        found = TRUE;
-        break;
-      }
-    }
-  }
-  
-  if (found == FALSE)
-    object_types_nosheet_list = g_slist_append(object_types_nosheet_list,
-                                               g_memdup(value,
-                                                        sizeof(ObjectType)));
-}
-
-GSList * 
-object_types_detect_nosheet(void)
-{
-  g_slist_foreach(object_types_nosheet_list, (GFunc)g_free, NULL);
-  g_slist_free(object_types_nosheet_list);
-  g_hash_table_foreach(object_type_table,
-                       (GHFunc)object_type_detect_nosheet_callback, NULL);
-  return object_types_nosheet_list;
-}
-
-gchar *
-object_type_get_sheet_name(ObjectType *type)
-{
-  gchar *pos;
-  gchar *sheet_name;
-
-  sheet_name = g_strdup(type->name);
-  pos = strchr(sheet_name, '-');
-  if (pos)
-    return g_strchomp(g_strdelimit(sheet_name, "-", '\0'));
-  else
-    return sheet_name;
-}
-
-gchar *
-object_type_strip_sheet_from_name(ObjectType *type)
-{
-  gchar *pos;
-
-  pos = strchr(type->name, '-');
-  if (pos)
-    return pos + 2;
-  else
-    return type->name;
+  g_hash_table_foreach (object_type_table, func, user_data);
 }
 
 ObjectType *
@@ -499,7 +586,6 @@ Object *object_copy_using_properties(Object *obj)
   Handle *handle1,*handle2;
   Object *newobj = obj->type->ops->create(&startpoint,NULL,
                                           &handle1,&handle2);
-  object_copy_props(newobj,obj);
+  object_copy_props(newobj,obj,FALSE);
   return newobj;
 }
-
