@@ -28,6 +28,7 @@
 
 #include <string.h>
 
+#undef GTK_DISABLE_DEPRECATED /* gtk_signal_connect, ... */
 #include <gtk/gtk.h>
 #define WIDGET GtkWidget
 #include "widgets.h"
@@ -38,10 +39,10 @@
 const gchar *prop_dialogdata_key = "object-props:dialogdata";
 
 static void prop_dialog_signal_destroy(GtkWidget *dialog_widget);
-static void prop_dialog_fill(PropDialog *dialog, Object *obj);
+static void prop_dialog_fill(PropDialog *dialog, Object *obj, gboolean is_default);
 
 PropDialog *
-prop_dialog_new(Object *obj) 
+prop_dialog_new(Object *obj, gboolean is_default) 
 {
   PropDialog *dialog = g_new0(PropDialog,1);
   dialog->props = NULL;
@@ -57,7 +58,7 @@ prop_dialog_new(Object *obj)
   gtk_signal_connect(GTK_OBJECT(dialog->widget), "destroy",
                      GTK_SIGNAL_FUNC(prop_dialog_signal_destroy), NULL);
 
-  prop_dialog_fill(dialog,obj);
+  prop_dialog_fill(dialog,obj,is_default);
 
   return dialog;
 }
@@ -99,6 +100,17 @@ prop_dialog_add_raw(PropDialog *dialog, GtkWidget *widget)
   gtk_container_add(GTK_CONTAINER(dialog->lastcont),widget);
 }
 
+void 
+prop_dialog_add_raw_with_flags(PropDialog *dialog, GtkWidget *widget,
+			       gboolean expand, gboolean fill)
+{
+  g_return_if_fail(GTK_IS_BOX(dialog->lastcont));
+
+  dialog->curtable = NULL;
+  if (!widget) return;
+  gtk_box_pack_start(GTK_BOX(dialog->lastcont),widget, expand, fill, 0);
+}
+
 static void
 prop_dialog_make_curtable(PropDialog *dialog) 
 {
@@ -126,7 +138,7 @@ prop_dialog_add_widget(PropDialog *dialog, GtkWidget *label, GtkWidget *widget)
   gtk_table_attach(GTK_TABLE(dialog->curtable),widget, 
                    1,2,
                    dialog->currow, dialog->currow+1,
-                   GTK_FILL, GTK_FILL|GTK_EXPAND,
+                   GTK_FILL|GTK_EXPAND, GTK_FILL|GTK_EXPAND,
                    0,0);
   gtk_widget_show(label);
   gtk_widget_show(widget);
@@ -256,12 +268,13 @@ prop_dialog_add_property(PropDialog *dialog, Property *prop)
 }
 
 static void 
-prop_dialog_add_properties(PropDialog *dialog, GPtrArray *props)     
+prop_dialog_add_properties(PropDialog *dialog, GPtrArray *props)
 {
   guint i;
 
   for (i = 0; i < props->len; i++) {
-    prop_dialog_add_property(dialog,g_ptr_array_index(props,i));
+    Property *prop = (Property*)g_ptr_array_index(props,i);
+    prop_dialog_add_property(dialog, prop);
   }
 }
 
@@ -276,9 +289,9 @@ prop_get_data_from_widgets(PropDialog *dialog)
     
   }
 }
-
+extern gboolean pdtpp_is_visible_no_standard(const PropDescription *pdesc);
 static void 
-prop_dialog_fill(PropDialog *dialog, Object *obj) {
+prop_dialog_fill(PropDialog *dialog, Object *obj, gboolean is_default) {
   const PropDescription *pdesc;
   GPtrArray *props;
 
@@ -289,7 +302,11 @@ prop_dialog_fill(PropDialog *dialog, Object *obj) {
   pdesc = object_get_prop_descriptions(obj);
   if (!pdesc) return;
 
-  props = prop_list_from_descs(pdesc,pdtpp_is_visible);
+  if (is_default)
+      props = prop_list_from_descs(pdesc,pdtpp_is_visible_no_standard);
+  else
+      props = prop_list_from_descs(pdesc,pdtpp_is_visible);
+
   if (!props) return;
     
   dialog->props = props;
