@@ -32,21 +32,15 @@
 #endif
 #endif 
 
-typedef struct _PropDescription PropDescription;
-typedef struct _Property Property;
-typedef struct _PropEventData PropEventData;
-typedef struct _PropDialog PropDialog;
-typedef struct _PropEventHandlerChain PropEventHandlerChain;
-typedef struct _PropWidgetAssoc PropWidgetAssoc;
 
 #ifdef HAVE_STDDEF_H
 #include <stddef.h>
 #endif
 
 #include <glib.h>
+#include "diatypes.h"
 
 #include "geometry.h"
-#include "render.h"
 #include "arrows.h"
 #include "color.h"
 #include "font.h"
@@ -54,7 +48,6 @@ typedef struct _PropWidgetAssoc PropWidgetAssoc;
 #include "dia_xml.h"
 #include "intl.h"
 #include "textattr.h"
-#include "charconv.h"
 
 
 typedef gboolean (*PropDescToPropPredicate)(const PropDescription *pdesc);
@@ -124,8 +117,8 @@ typedef void (*PropertyType_GetFromOffset)(const Property *prop,
                                          void *base, guint offset, guint offset2);
 typedef void (*PropertyType_SetFromOffset)(Property *prop,
                                          void *base, guint offset, guint offset2);
+typedef int (*PropertyType_GetDataSize)(Property *prop);
 
-typedef struct _PropertyOps PropertyOps; 
 
 struct _PropertyOps {
   PropertyType_New  new_prop;
@@ -141,6 +134,7 @@ struct _PropertyOps {
   
   PropertyType_GetFromOffset get_from_offset;
   PropertyType_SetFromOffset set_from_offset;
+  PropertyType_GetDataSize get_data_size;
 };
 
 typedef const gchar *PropertyType;
@@ -164,6 +158,7 @@ typedef const gchar *PropertyType;
 #define PROP_TYPE_STRING "string"            /* StringProperty */
 #define PROP_TYPE_FILE "file"                /* StringProperty */
 #define PROP_TYPE_TEXT "text" /* can't be visible */ /* TextProperty */
+#define PROP_TYPE_STRINGLIST "stringlist" /* can't be visible */ /* StringListProperty */
 
 /* Geometric types : */
 #define PROP_TYPE_REAL "real"                /* RealProperty */
@@ -237,15 +232,17 @@ struct _PropDescription {
 #define PROP_FLAG_DONT_MERGE 0x0004 /* in case group properties are edited */
 #define PROP_FLAG_NO_DEFAULTS 0x0008 /* don't edit this in defaults dlg. */
 #define PROP_FLAG_LOAD_ONLY 0x0010 /* for loading old formats */
+#define PROP_FLAG_STANDARD 0x0020 /* One of the default toolbox props */
+#define PROP_FLAG_MULTIVALUE 0x0040 /* Multiple values for prop in group */
+#define PROP_FLAG_WIDGET_ONLY 0x0080 /* only cosmetic property, no data */
+#define PROP_FLAG_OPTIONAL 0x0100 /* don't complain if it does not exist */
 
 #define PROP_DESC_END { NULL, 0, 0, NULL, NULL, NULL, 0 }
 
 /* extra data pointers for various property types */
-typedef struct _PropNumData PropNumData;
 struct _PropNumData {
   gfloat min, max, step;
 };
-typedef struct _PropEnumData PropEnumData;
 struct _PropEnumData {
   const gchar *name;
   guint enumv;
@@ -254,21 +251,18 @@ struct _PropEnumData {
 typedef gpointer (*NewRecordFunc)(void);
 typedef void (*FreeRecordFunc)(gpointer rec);
 
-typedef struct _PropDescCommonArrayExtra PropDescCommonArrayExtra;
 struct _PropDescCommonArrayExtra { /* don't use this directly. 
                                       Use one of below */
   PropDescription *record;
   const gchar *composite_type; /* can be NULL. */ 
 };
 
-typedef struct _PropDescDArrayExtra PropDescDArrayExtra;
 struct _PropDescDArrayExtra {
   PropDescCommonArrayExtra common; /* must be first */
   NewRecordFunc newrec;        
   FreeRecordFunc freerec; 
 };
 
-typedef struct _PropDescSArrayExtra PropDescSArrayExtra;
 struct  _PropDescSArrayExtra {
   PropDescCommonArrayExtra common; /* must be first */
   guint element_size;  /* sizeof(record) */
@@ -306,6 +300,7 @@ struct _Property {
 #define PXP_SAVED            0x00000040
 #define PXP_GFO              0x00000080
 #define PXP_SFO              0x00000100
+#define PXP_NOTSET           0x00000200
 #define PXP_SHAMELESS        0xFFFFFFFF
 
 /* ***************************************************************** */
@@ -356,8 +351,10 @@ GPtrArray *prop_list_from_single(Property *prop);
 /* Some predicates: */
 gboolean pdtpp_true(const PropDescription *pdesc); /* always true */
 gboolean pdtpp_is_visible(const PropDescription *pdesc); 
+gboolean pdtpp_is_visible_no_standard(const PropDescription *pdesc); 
 gboolean pdtpp_is_not_visible(const PropDescription *pdesc); 
 gboolean pdtpp_do_save(const PropDescription *pdesc); 
+gboolean pdtpp_do_save_no_standard(const PropDescription *pdesc);
 gboolean pdtpp_do_load(const PropDescription *pdesc);
 gboolean pdtpp_do_not_save(const PropDescription *pdesc); 
 /* all but DONT_MERGE and NO_DEFAULTS: */
@@ -387,7 +384,6 @@ Property *make_new_prop(const char *name, PropertyType type, guint flags);
 #ifndef offsetof
 #define offsetof(type, member) ( (int) & ((type*)0) -> member )
 #endif
-typedef struct _PropOffset PropOffset;
 struct _PropOffset {
   const gchar *name;
   PropertyType type;
@@ -418,8 +414,10 @@ gboolean object_set_props_from_offsets(Object *obj, PropOffset *offsets,
 ObjectChange *object_apply_props(Object *obj, GPtrArray *props);
 
 /* standard properties dialogs that can be used for objects that
- * implement describe_props, get_props and set_props */
-WIDGET *object_create_props_dialog     (Object *obj);
+ * implement describe_props, get_props and set_props.
+ * If is_default is set, this is a default dialog, not an object dialog.
+ */
+WIDGET *object_create_props_dialog     (Object *obj, gboolean is_default);
 ObjectChange *object_apply_props_from_dialog (Object *obj, WIDGET *dialog);
 
 /* create a property from the object's property descriptors. To be freed with
@@ -435,7 +433,8 @@ void          object_save_props(Object *obj, ObjectNode obj_node);
 
 /* standard way to copy the properties of an object into another (of the
    same type) */
-void          object_copy_props(Object *dest, Object *src);
+void          object_copy_props(Object *dest, const Object *src,
+                                gboolean is_default);
 
 /* Return a reference to objects property with 'name' or NULL */
 Property     *object_get_prop_by_name (Object *obj, const char* name);
@@ -475,27 +474,39 @@ extern PropEnumData prop_std_text_align_data[];
 #endif
 
 #define PROP_STD_LINE_WIDTH \
-  { "line_width", PROP_TYPE_REAL, PROP_FLAG_VISIBLE, \
+  { "line_width", PROP_TYPE_REAL, PROP_FLAG_VISIBLE|PROP_FLAG_STANDARD, \
+    N_("Line width"), NULL, &prop_std_line_width_data }
+#define PROP_STD_LINE_WIDTH_OPTIONAL \
+  { "line_width", PROP_TYPE_REAL, PROP_FLAG_VISIBLE|PROP_FLAG_STANDARD|PROP_FLAG_OPTIONAL, \
     N_("Line width"), NULL, &prop_std_line_width_data }
 #define PROP_STD_LINE_COLOUR \
-  { "line_colour", PROP_TYPE_COLOUR, PROP_FLAG_VISIBLE, \
+  { "line_colour", PROP_TYPE_COLOUR, PROP_FLAG_VISIBLE|PROP_FLAG_STANDARD, \
+    N_("Line colour"), NULL, NULL }
+#define PROP_STD_LINE_COLOUR_OPTIONAL \
+  { "line_colour", PROP_TYPE_COLOUR, PROP_FLAG_VISIBLE|PROP_FLAG_STANDARD|PROP_FLAG_OPTIONAL, \
     N_("Line colour"), NULL, NULL }
 #define PROP_STD_LINE_STYLE \
-  { "line_style", PROP_TYPE_LINESTYLE, PROP_FLAG_VISIBLE, \
+  { "line_style", PROP_TYPE_LINESTYLE, PROP_FLAG_VISIBLE|PROP_FLAG_STANDARD, \
+    N_("Line style"), NULL, NULL }
+#define PROP_STD_LINE_STYLE_OPTIONAL \
+  { "line_style", PROP_TYPE_LINESTYLE, PROP_FLAG_VISIBLE|PROP_FLAG_STANDARD|PROP_FLAG_OPTIONAL, \
     N_("Line style"), NULL, NULL }
 
 #define PROP_STD_FILL_COLOUR \
-  { "fill_colour", PROP_TYPE_COLOUR, PROP_FLAG_VISIBLE, \
+  { "fill_colour", PROP_TYPE_COLOUR, PROP_FLAG_VISIBLE|PROP_FLAG_STANDARD, \
+    N_("Fill colour"), NULL, NULL }
+#define PROP_STD_FILL_COLOUR_OPTIONAL \
+  { "fill_colour", PROP_TYPE_COLOUR, PROP_FLAG_VISIBLE|PROP_FLAG_STANDARD|PROP_FLAG_OPTIONAL, \
     N_("Fill colour"), NULL, NULL }
 #define PROP_STD_SHOW_BACKGROUND \
   { "show_background", PROP_TYPE_BOOL, PROP_FLAG_VISIBLE, \
     N_("Draw background"), NULL, NULL }
 
 #define PROP_STD_START_ARROW \
-  { "start_arrow", PROP_TYPE_ARROW, PROP_FLAG_VISIBLE, \
+  { "start_arrow", PROP_TYPE_ARROW, PROP_FLAG_VISIBLE|PROP_FLAG_STANDARD, \
     N_("Start arrow"), NULL, NULL }
 #define PROP_STD_END_ARROW \
-  { "end_arrow", PROP_TYPE_ARROW, PROP_FLAG_VISIBLE, \
+  { "end_arrow", PROP_TYPE_ARROW, PROP_FLAG_VISIBLE|PROP_FLAG_STANDARD, \
     N_("End arrow"), NULL, NULL }
 
 #define PROP_STD_TEXT \
@@ -514,13 +525,16 @@ extern PropEnumData prop_std_text_align_data[];
   { "text_height", PROP_TYPE_REAL, PROP_FLAG_VISIBLE|PROP_FLAG_DONT_SAVE, \
     N_("Font size"), NULL, &prop_std_text_height_data }
 #define PROP_STD_TEXT_COLOUR \
-  { "text_colour", PROP_TYPE_COLOUR, PROP_FLAG_VISIBLE|PROP_FLAG_DONT_SAVE, \
+  { "text_colour", PROP_TYPE_COLOUR, PROP_FLAG_VISIBLE|PROP_FLAG_DONT_SAVE|PROP_FLAG_STANDARD, \
+    N_("Text colour"), NULL, NULL }
+#define PROP_STD_TEXT_COLOUR_OPTIONAL \
+  { "text_colour", PROP_TYPE_COLOUR, PROP_FLAG_VISIBLE|PROP_FLAG_DONT_SAVE|PROP_FLAG_STANDARD|PROP_FLAG_OPTIONAL, \
     N_("Text colour"), NULL, NULL }
 
 /* Convenience macros */
 #define PROP_NOTEBOOK_BEGIN(name) \
   { "nbook_" name, PROP_TYPE_NOTEBOOK_BEGIN, \
-              PROP_FLAG_VISIBLE|PROP_FLAG_DONT_SAVE, NULL, NULL}
+              PROP_FLAG_VISIBLE|PROP_FLAG_DONT_SAVE|PROP_FLAG_WIDGET_ONLY, NULL, NULL}
 #define PROP_STD_NOTEBOOK_BEGIN PROP_NOTEBOOK_BEGIN("std")
 #define PROP_OFFSET_NOTEBOOK_BEGIN(name) \
   { "nbook_" name, PROP_TYPE_NOTEBOOK_BEGIN, 0}
@@ -528,7 +542,7 @@ extern PropEnumData prop_std_text_align_data[];
 
 #define PROP_NOTEBOOK_END(name) \
   { "nbook_" name "_end", PROP_TYPE_NOTEBOOK_END, \
-      PROP_FLAG_VISIBLE|PROP_FLAG_DONT_SAVE, NULL, NULL}
+      PROP_FLAG_VISIBLE|PROP_FLAG_DONT_SAVE|PROP_FLAG_WIDGET_ONLY, NULL, NULL}
 #define PROP_STD_NOTEBOOK_END PROP_NOTEBOOK_END("std")
 #define PROP_OFFSET_NOTEBOOK_END(name) \
   { "nbook_" name "_end", PROP_TYPE_NOTEBOOK_END, 0}
@@ -536,13 +550,13 @@ extern PropEnumData prop_std_text_align_data[];
 
 #define PROP_NOTEBOOK_PAGE(name,flags,descr) \
   { "nbook_page_" name, PROP_TYPE_NOTEBOOK_PAGE, \
- PROP_FLAG_VISIBLE|PROP_FLAG_DONT_SAVE|flags,descr,NULL}
+ PROP_FLAG_VISIBLE|PROP_FLAG_DONT_SAVE|PROP_FLAG_WIDGET_ONLY|flags,descr,NULL}
 #define PROP_OFFSET_NOTEBOOK_PAGE(name) \
   { "nbook_page_" name , PROP_TYPE_NOTEBOOK_PAGE, 0}
 
 #define PROP_MULTICOL_BEGIN(name) \
   { "mcol_" name, PROP_TYPE_MULTICOL_BEGIN, \
-              PROP_FLAG_VISIBLE|PROP_FLAG_DONT_SAVE, NULL, NULL}
+              PROP_FLAG_VISIBLE|PROP_FLAG_DONT_SAVE|PROP_FLAG_WIDGET_ONLY, NULL, NULL}
 #define PROP_STD_MULTICOL_BEGIN PROP_MULTICOL_BEGIN("std")
 #define PROP_OFFSET_MULTICOL_BEGIN(name) \
   { "mcol_" name, PROP_TYPE_NOTEBOOK_BEGIN, 0}
@@ -550,7 +564,7 @@ extern PropEnumData prop_std_text_align_data[];
 
 #define PROP_MULTICOL_END(name) \
   { "mcol_" name "_end", PROP_TYPE_MULTICOL_END, \
-      PROP_FLAG_VISIBLE|PROP_FLAG_DONT_SAVE, NULL, NULL}
+      PROP_FLAG_VISIBLE|PROP_FLAG_DONT_SAVE|PROP_FLAG_WIDGET_ONLY, NULL, NULL}
 #define PROP_STD_MULTICOL_END PROP_MULTICOL_END("std")
 #define PROP_OFFSET_MULTICOL_END(name) \
   { "mcol_" name "_end", PROP_TYPE_NOTEBOOK_END, 0}
@@ -558,21 +572,21 @@ extern PropEnumData prop_std_text_align_data[];
 
 #define PROP_MULTICOL_COLUMN(name) \
   { "mcol_col_" name, PROP_TYPE_MULTICOL_COLUMN, \
- PROP_FLAG_VISIBLE|PROP_FLAG_DONT_SAVE,NULL,NULL}
+ PROP_FLAG_VISIBLE|PROP_FLAG_DONT_SAVE|PROP_FLAG_WIDGET_ONLY,NULL,NULL}
 #define PROP_OFFSET_MULTICOL_COLUMN(name) \
   { "mcol_col_" name, PROP_TYPE_MULTICOL_COLUMN, 0}
 
 #define PROP_FRAME_BEGIN(name,flags,descr) \
   { "frame_" name, PROP_TYPE_FRAME_BEGIN, \
-              PROP_FLAG_VISIBLE|PROP_FLAG_DONT_SAVE|flags, descr, NULL}
+              PROP_FLAG_VISIBLE|PROP_FLAG_DONT_SAVE|PROP_FLAG_WIDGET_ONLY|flags, descr, NULL}
 #define PROP_OFFSET_FRAME_BEGIN(name) \
   { "frame_" name, PROP_TYPE_FRAME_BEGIN, 0}
 
 #define PROP_FRAME_END(name,flags) \
   { "frame_" name "_end", PROP_TYPE_FRAME_END, \
-      PROP_FLAG_VISIBLE|PROP_FLAG_DONT_SAVE|flags, NULL, NULL}
+      PROP_FLAG_VISIBLE|PROP_FLAG_DONT_SAVE|PROP_FLAG_WIDGET_ONLY|flags, NULL, NULL}
 #define PROP_OFFSET_FRAME_END(name) \
-  { "frame_" name "_end", PROP_TYPE_NOTEBOOK_END, 0}
+  { "frame_" name "_end", PROP_TYPE_FRAME_END, 0}
 
 
 
