@@ -43,10 +43,15 @@ gboolean pdtpp_from_object(const PropDescription *pdesc)
 { return TRUE; }
 gboolean pdtpp_is_visible(const PropDescription *pdesc) 
 { return (pdesc->flags & PROP_FLAG_VISIBLE) != 0; } 
+gboolean pdtpp_is_visible_no_standard(const PropDescription *pdesc) 
+{ return (pdesc->flags & PROP_FLAG_VISIBLE) != 0 &&
+         (pdesc->flags & PROP_FLAG_STANDARD) == 0; } 
 gboolean pdtpp_is_not_visible(const PropDescription *pdesc) 
 { return (pdesc->flags & PROP_FLAG_VISIBLE) == 0; } 
 gboolean pdtpp_do_save(const PropDescription *pdesc)
 { return (pdesc->flags & (PROP_FLAG_DONT_SAVE|PROP_FLAG_LOAD_ONLY)) == 0; } 
+gboolean pdtpp_do_save_no_standard(const PropDescription *pdesc)
+{ return (pdesc->flags & (PROP_FLAG_DONT_SAVE|PROP_FLAG_LOAD_ONLY|PROP_FLAG_STANDARD)) == 0; } 
 gboolean pdtpp_do_load(const PropDescription *pdesc)
 { return (((pdesc->flags & PROP_FLAG_DONT_SAVE) == 0) ||
             ((pdesc->flags & PROP_FLAG_LOAD_ONLY) != 0)); } 
@@ -145,6 +150,10 @@ prop_list_load(GPtrArray *props, DataNode data)
     Property *prop = g_ptr_array_index(props,i);
     AttributeNode attr = object_find_attribute(data, prop->name);
     DataNode data = attr ? attribute_first_data(attr) : NULL;
+    if (!attr && prop->descr->flags & PROP_FLAG_OPTIONAL) {
+      prop->experience |= PXP_NOTSET;
+      continue;
+    }
     if ((!attr) || (!data)) {
       g_warning("No attribute %s (%p) or no data(%p) in this attribute",
                 prop->name,attr,data);
