@@ -26,7 +26,6 @@
 #endif
 
 #include <assert.h>
-#include <gtk/gtk.h>
 #include <math.h>
 #include <string.h>
 #include <stdlib.h>
@@ -34,7 +33,7 @@
 #include "intl.h"
 #include "object.h"
 #include "connection.h"
-#include "render.h"
+#include "diarenderer.h"
 #include "handle.h"
 #include "arrows.h"
 #include "properties.h"
@@ -60,12 +59,13 @@ typedef struct _Annotation {
 
 #define HANDLE_MOVE_TEXT (HANDLE_CUSTOM1)
 
-static void annotation_move_handle(Annotation *annotation, Handle *handle,
-				   Point *to, HandleMoveReason reason, ModifierKeys modifiers);
-static void annotation_move(Annotation *annotation, Point *to);
+static ObjectChange* annotation_move_handle(Annotation *annotation, Handle *handle,
+					    Point *to, ConnectionPoint *cp,
+					    HandleMoveReason reason, ModifierKeys modifiers);
+static ObjectChange* annotation_move(Annotation *annotation, Point *to);
 static void annotation_select(Annotation *annotation, Point *clicked_point,
-			      Renderer *interactive_renderer);
-static void annotation_draw(Annotation *annotation, Renderer *renderer);
+			      DiaRenderer *interactive_renderer);
+static void annotation_draw(Annotation *annotation, DiaRenderer *renderer);
 static Object *annotation_create(Point *startpoint,
 				 void *user_data,
 				 Handle **handle1,
@@ -114,9 +114,8 @@ static ObjectOps annotation_ops = {
   (SetPropsFunc)        annotation_set_props
 };
 
-#ifndef _MSC_VER
-#define TEMPORARY_EVENT_TEST
-#endif
+#undef TEMPORARY_EVENT_TEST
+
 #ifdef TEMPORARY_EVENT_TEST
 static gboolean 
 handle_btn1(Annotation *annotation, Property *prop) {
@@ -125,9 +124,9 @@ handle_btn1(Annotation *annotation, Property *prop) {
   col = annotation->attrs.color;
   /* g_message("in handle_btn1 for object %p col=%.2f:%.2f:%.2f",
      annotation,col.red,col.green,col.blue); */
-  col.red = (1.0*random())/RAND_MAX;
-  col.green = (1.0*random())/RAND_MAX;
-  col.blue = (1.0*random())/RAND_MAX;
+  col.red = g_random_double();
+  col.green = g_random_double();
+  col.blue = g_random_double();
   annotation->attrs.color = col;
   text_set_attributes(annotation->text,&annotation->attrs);
   /* g_message("end of handle_btn1 for object %p col=%.2f:%.2f:%.2f",
@@ -204,7 +203,7 @@ annotation_distance_from(Annotation *annotation, Point *point)
 
 static void
 annotation_select(Annotation *annotation, Point *clicked_point,
-	    Renderer *interactive_renderer)
+	    DiaRenderer *interactive_renderer)
 {
   text_set_cursor(annotation->text, clicked_point, interactive_renderer);
   text_grab_focus(annotation->text, &annotation->connection.object);
@@ -212,9 +211,10 @@ annotation_select(Annotation *annotation, Point *clicked_point,
   connection_update_handles(&annotation->connection);
 }
 
-static void
+static ObjectChange*
 annotation_move_handle(Annotation *annotation, Handle *handle,
-		 Point *to, HandleMoveReason reason, ModifierKeys modifiers)
+		       Point *to, ConnectionPoint *cp,
+		       HandleMoveReason reason, ModifierKeys modifiers)
 {
   Point p1, p2;
   Point *endpoints;
@@ -230,24 +230,26 @@ annotation_move_handle(Annotation *annotation, Handle *handle,
     endpoints = &(conn->endpoints[0]); 
     if (handle->id == HANDLE_MOVE_STARTPOINT) {
       p1 = endpoints[0];
-      connection_move_handle(conn, handle->id, to, reason);
+      connection_move_handle(conn, handle->id, to, cp, reason, modifiers);
       p2 = endpoints[0];
       point_sub(&p2, &p1);
       point_add(&annotation->text->position, &p2);
       point_add(&p2,&(endpoints[1]));
-      connection_move_handle(conn, HANDLE_MOVE_ENDPOINT, &p2, reason);   
+      connection_move_handle(conn, HANDLE_MOVE_ENDPOINT, &p2, NULL, reason, 0);
     } else {      
       p1 = endpoints[1];
-      connection_move_handle(conn, handle->id, to, reason);
+      connection_move_handle(conn, handle->id, to, cp, reason, modifiers);
       p2 = endpoints[1];
       point_sub(&p2, &p1);
       point_add(&annotation->text->position, &p2);
     }
   }
   annotation_update_data(annotation);
+
+  return NULL;
 }
 
-static void
+static ObjectChange*
 annotation_move(Annotation *annotation, Point *to)
 {
   Point start_to_end;
@@ -266,24 +268,27 @@ annotation_move(Annotation *annotation, Point *to)
   point_add(&annotation->text->position, &delta);
   
   annotation_update_data(annotation);
+
+  return NULL;
 }
 
 static void
-annotation_draw(Annotation *annotation, Renderer *renderer)
+annotation_draw(Annotation *annotation, DiaRenderer *renderer)
 {
   Point *endpoints;
   Point vect,rvect,v1,v2;
   Point pts[4];
   real vlen;
+  DiaRendererClass *renderer_ops = DIA_RENDERER_GET_CLASS (renderer);
 
   assert(annotation != NULL);
   assert(renderer != NULL);
 
   endpoints = &annotation->connection.endpoints[0];
   
-  renderer->ops->set_linewidth(renderer, ANNOTATION_LINE_WIDTH);
-  renderer->ops->set_linestyle(renderer, LINESTYLE_SOLID);
-  renderer->ops->set_linecaps(renderer, LINECAPS_BUTT);
+  renderer_ops->set_linewidth(renderer, ANNOTATION_LINE_WIDTH);
+  renderer_ops->set_linestyle(renderer, LINESTYLE_SOLID);
+  renderer_ops->set_linecaps(renderer, LINECAPS_BUTT);
 
   vect = annotation->connection.endpoints[1];
   point_sub(&vect,&annotation->connection.endpoints[0]);
@@ -310,7 +315,7 @@ annotation_draw(Annotation *annotation, Renderer *renderer)
     point_add(&pts[1],&v1);
     point_sub(&pts[2],&v1);
     pts[3] = annotation->connection.endpoints[1];
-    renderer->ops->draw_polyline(renderer,
+    renderer_ops->draw_polyline(renderer,
 			     pts, sizeof(pts) / sizeof(pts[0]),
 			     &color_black);
   }
@@ -329,6 +334,7 @@ annotation_create(Point *startpoint,
   Object *obj; 
   Point offs;
   Point defaultlen = { 1.0, 1.0 };
+  DiaFont* font;
 
   annotation = g_malloc0(sizeof(Annotation));
 
@@ -345,21 +351,13 @@ annotation_create(Point *startpoint,
   
   connection_init(conn, 3, 0);
 
-#if 0
-  annotation->text = new_text("", annotation_defaults.text_font,
-			      annotation_defaults.text_fontheight,
-			      &conn->endpoints[1],
-			      &annotation_defaults.text_color,
-			      ALIGN_CENTER);
-#else
-  /* choose default font name for your locale. see also font_data structure
-     in lib/font.c. */
-  annotation->text = new_text("", font_getfont (_("Helvetica")),
-			      ANNOTATION_FONTHEIGHT,
-			      &conn->endpoints[1],
-			      &color_black,
-			      ALIGN_CENTER);
-#endif
+  font = dia_font_new_from_style(DIA_FONT_SANS,ANNOTATION_FONTHEIGHT);
+  annotation->text = new_text("", font,
+                              ANNOTATION_FONTHEIGHT,
+                              &conn->endpoints[1],
+                              &color_black,
+                              ALIGN_CENTER);
+  dia_font_unref(font);
 
   offs.x = .3 * ANNOTATION_FONTHEIGHT;
   if (conn->endpoints[1].y < conn->endpoints[0].y)
@@ -391,7 +389,7 @@ annotation_destroy(Annotation *annotation)
 {
   connection_destroy(&annotation->connection);
 
-  g_free(annotation->text);
+  text_destroy(annotation->text);
 }
 
 static void
