@@ -27,7 +27,7 @@
 #include "intl.h"
 #include "object.h"
 #include "element.h"
-#include "render.h"
+#include "diarenderer.h"
 #include "attributes.h"
 #include "text.h"
 #include "properties.h"
@@ -40,13 +40,16 @@ typedef struct _Component Component;
 struct _Component {
   Element element;
 
-  ConnectionPoint connections[8];
+  ConnectionPoint connections[10];
 
-  utfchar *stereotype;
+  char *stereotype;
   Text *text;
 
-  utfchar *st_stereotype;
+  char *st_stereotype;
   TextAttributes attrs;
+
+  Color line_color;
+  Color fill_color;
 };
 
 #define COMPONENT_BORDERWIDTH 0.1
@@ -57,11 +60,12 @@ struct _Component {
 
 static real component_distance_from(Component *cmp, Point *point);
 static void component_select(Component *cmp, Point *clicked_point,
-				Renderer *interactive_renderer);
-static void component_move_handle(Component *cmp, Handle *handle,
-				     Point *to, HandleMoveReason reason, ModifierKeys modifiers);
-static void component_move(Component *cmp, Point *to);
-static void component_draw(Component *cmp, Renderer *renderer);
+				DiaRenderer *interactive_renderer);
+static ObjectChange* component_move_handle(Component *cmp, Handle *handle,
+					   Point *to, ConnectionPoint *cp,
+					   HandleMoveReason reason, ModifierKeys modifiers);
+static ObjectChange* component_move(Component *cmp, Point *to);
+static void component_draw(Component *cmp, DiaRenderer *renderer);
 static Object *component_create(Point *startpoint,
 				   void *user_data,
 				   Handle **handle1,
@@ -112,6 +116,8 @@ static ObjectOps component_ops = {
 
 static PropDescription component_props[] = {
   ELEMENT_COMMON_PROPERTIES,
+  PROP_STD_LINE_COLOUR_OPTIONAL, 
+  PROP_STD_FILL_COLOUR_OPTIONAL, 
   { "stereotype", PROP_TYPE_STRING, PROP_FLAG_VISIBLE,
   N_("Stereotype"), NULL, NULL },
   PROP_STD_TEXT_FONT,
@@ -124,11 +130,16 @@ static PropDescription component_props[] = {
 static PropDescription *
 component_describe_props(Component *component)
 {
+  if (component_props[0].quark == 0) {
+    prop_desc_list_calculate_quarks(component_props);
+  }
   return component_props;
 }
 
 static PropOffset component_offsets[] = {
   ELEMENT_COMMON_PROPERTIES_OFFSETS,
+  {"line_colour",PROP_TYPE_COLOUR,offsetof(Component,line_color)},
+  {"fill_colour",PROP_TYPE_COLOUR,offsetof(Component,fill_color)},
   {"stereotype", PROP_TYPE_STRING, offsetof(Component , stereotype) },
   {"text",PROP_TYPE_TEXT,offsetof(Component,text)},
   {"text_font",PROP_TYPE_FONT,offsetof(Component,attrs.font)},
@@ -166,35 +177,41 @@ component_distance_from(Component *cmp, Point *point)
 
 static void
 component_select(Component *cmp, Point *clicked_point,
-	       Renderer *interactive_renderer)
+	       DiaRenderer *interactive_renderer)
 {
   text_set_cursor(cmp->text, clicked_point, interactive_renderer);
   text_grab_focus(cmp->text, &cmp->element.object);
   element_update_handles(&cmp->element);
 }
 
-static void
+static ObjectChange*
 component_move_handle(Component *cmp, Handle *handle,
-			 Point *to, HandleMoveReason reason, ModifierKeys modifiers)
+		      Point *to, ConnectionPoint *cp,
+		      HandleMoveReason reason, ModifierKeys modifiers)
 {
   assert(cmp!=NULL);
   assert(handle!=NULL);
   assert(to!=NULL);
 
   assert(handle->id < 8);
+
+  return NULL;
 }
 
-static void
+static ObjectChange*
 component_move(Component *cmp, Point *to)
 {
   cmp->element.corner = *to;
 
   component_update_data(cmp);
+
+  return NULL;
 }
 
 static void
-component_draw(Component *cmp, Renderer *renderer)
+component_draw(Component *cmp, DiaRenderer *renderer)
 {
+  DiaRendererClass *renderer_ops = DIA_RENDERER_GET_CLASS (renderer);
   Element *elem;
   real x, y, w, h;
   Point p1, p2;
@@ -209,49 +226,49 @@ component_draw(Component *cmp, Renderer *renderer)
   w = elem->width;
   h = elem->height;
   
-  renderer->ops->set_fillstyle(renderer, FILLSTYLE_SOLID);
-  renderer->ops->set_linewidth(renderer, COMPONENT_BORDERWIDTH);
-  renderer->ops->set_linestyle(renderer, LINESTYLE_SOLID);
+  renderer_ops->set_fillstyle(renderer, FILLSTYLE_SOLID);
+  renderer_ops->set_linewidth(renderer, COMPONENT_BORDERWIDTH);
+  renderer_ops->set_linestyle(renderer, LINESTYLE_SOLID);
 
   p1.x = x + COMPONENT_CWIDTH/2; p1.y = y;
   p2.x = x+w; p2.y = y+h;
 
-  renderer->ops->fill_rect(renderer, 
+  renderer_ops->fill_rect(renderer, 
 			   &p1, &p2,
-			   &color_white);
-  renderer->ops->draw_rect(renderer, 
+			   &cmp->fill_color);
+  renderer_ops->draw_rect(renderer, 
 			   &p1, &p2,
-			   &color_black);
+			   &cmp->line_color);
 
   p1.x= x; p1.y = y +(h - 3*COMPONENT_CHEIGHT)/2.0;
   p2.x = x+COMPONENT_CWIDTH; p2.y = p1.y + COMPONENT_CHEIGHT;
 
-  renderer->ops->fill_rect(renderer, 
+  renderer_ops->fill_rect(renderer, 
 			   &p1, &p2,
-			   &color_white);
+			   &cmp->fill_color);
   
-  renderer->ops->draw_rect(renderer, 
+  renderer_ops->draw_rect(renderer, 
 			   &p1, &p2,
-			   &color_black);
+			   &cmp->line_color);
   
   p1.y = p2.y + COMPONENT_CHEIGHT;
   p2.y = p1.y + COMPONENT_CHEIGHT;
 
-  renderer->ops->fill_rect(renderer, 
+  renderer_ops->fill_rect(renderer, 
 			   &p1, &p2,
-			   &color_white);
+			   &cmp->fill_color);
   
-  renderer->ops->draw_rect(renderer, 
+  renderer_ops->draw_rect(renderer, 
 			   &p1, &p2,
-			   &color_black);
+			   &cmp->line_color);
 
   if (cmp->st_stereotype != NULL &&
       cmp->st_stereotype[0] != '\0') {
     p1 = cmp->text->position;
     p1.y -= cmp->text->height;
-    renderer->ops->set_font(renderer, cmp->text->font, cmp->text->height);
-    renderer->ops->draw_string(renderer, cmp->st_stereotype, &p1, 
-			       ALIGN_LEFT, &color_black);
+    renderer_ops->set_font(renderer, cmp->text->font, cmp->text->height);
+    renderer_ops->draw_string(renderer, cmp->st_stereotype, &p1, 
+			       ALIGN_LEFT, &cmp->attrs.color);
   }
 
   text_draw(cmp->text, renderer);
@@ -263,6 +280,7 @@ component_update_data(Component *cmp)
   Element *elem = &cmp->element;
   Object *obj = &elem->object;
   Point p;
+  real cw2, ch;
 
   cmp->stereotype = remove_stereotype_from_string(cmp->stereotype);
   if (!cmp->st_stereotype) {
@@ -291,28 +309,55 @@ component_update_data(Component *cmp)
     DiaFont *font;
     font = cmp->text->font;
     elem->height += cmp->text->height;
-    elem->width = MAX(elem->width, font_string_width(cmp->st_stereotype,
+    elem->width = MAX(elem->width, dia_font_string_width(cmp->st_stereotype,
 						     font, cmp->text->height) +
 		      2*COMPONENT_MARGIN_X + COMPONENT_CWIDTH);
   }
 
+  cw2 = COMPONENT_CWIDTH/2;
+  ch = COMPONENT_CHEIGHT;
   /* Update connections: */
-  cmp->connections[0].pos = elem->corner;
-  cmp->connections[1].pos.x = elem->corner.x + elem->width / 2.0;
-  cmp->connections[1].pos.y = elem->corner.y;
-  cmp->connections[2].pos.x = elem->corner.x + elem->width;
-  cmp->connections[2].pos.y = elem->corner.y;
-  cmp->connections[3].pos.x = elem->corner.x;
-  cmp->connections[3].pos.y = elem->corner.y + elem->height / 2.0;
-  cmp->connections[4].pos.x = elem->corner.x + elem->width;
-  cmp->connections[4].pos.y = elem->corner.y + elem->height / 2.0;
-  cmp->connections[5].pos.x = elem->corner.x;
-  cmp->connections[5].pos.y = elem->corner.y + elem->height;
-  cmp->connections[6].pos.x = elem->corner.x + elem->width / 2.0;
-  cmp->connections[6].pos.y = elem->corner.y + elem->height;
-  cmp->connections[7].pos.x = elem->corner.x + elem->width;
-  cmp->connections[7].pos.y = elem->corner.y + elem->height;
-  
+  connpoint_update(&cmp->connections[0],
+		   elem->corner.x + cw2,
+		   elem->corner.y,
+		   DIR_NORTH|DIR_WEST);
+  connpoint_update(&cmp->connections[1],
+		   elem->corner.x + cw2 + (elem->width - cw2) / 2,
+		   elem->corner.y,
+		   DIR_NORTH);
+  connpoint_update(&cmp->connections[2],
+		   elem->corner.x + elem->width,
+		   elem->corner.y,
+		   DIR_NORTH|DIR_EAST);
+  connpoint_update(&cmp->connections[3],
+		   elem->corner.x + cw2,
+		   elem->corner.y + elem->height / 2.0,
+		   DIR_WEST);
+  connpoint_update(&cmp->connections[4],
+		   elem->corner.x + elem->width,
+		   elem->corner.y + elem->height / 2.0,
+		   DIR_EAST);
+  connpoint_update(&cmp->connections[5],
+		   elem->corner.x + cw2,
+		   elem->corner.y + elem->height,
+		   DIR_SOUTH|DIR_WEST);
+  connpoint_update(&cmp->connections[6],
+		   elem->corner.x + cw2 + (elem->width - cw2)/2,
+		   elem->corner.y + elem->height,
+		   DIR_SOUTH);
+  connpoint_update(&cmp->connections[7],
+		   elem->corner.x + elem->width,
+		   elem->corner.y + elem->height,
+		   DIR_SOUTH|DIR_EAST);
+  connpoint_update(&cmp->connections[8],
+		   elem->corner.x,
+		   elem->corner.y + elem->height / 2.0 - ch,
+		   DIR_WEST);
+  connpoint_update(&cmp->connections[9],
+		   elem->corner.x,
+		   elem->corner.y + elem->height / 2.0 + ch,
+		   DIR_WEST);
+
   element_update_boundingbox(elem);
 
   obj->position = elem->corner;
@@ -341,11 +386,13 @@ component_create(Point *startpoint,
 
   obj->ops = &component_ops;
 
-  elem->corner = *startpoint;
+  obj->can_parent = TRUE;
 
-  /* choose default font name for your locale. see also font_data structure
-     in lib/font.c. */
-  font = font_getfont (_("Helvetica"));
+  elem->corner = *startpoint;
+  cmp->line_color = attributes_get_foreground();
+  cmp->fill_color = attributes_get_background();
+
+  font = dia_font_new_from_style (DIA_FONT_SANS, 0.8);
   p = *startpoint;
   p.x += COMPONENT_CWIDTH + COMPONENT_MARGIN_X;
   p.y += 2*COMPONENT_CHEIGHT;
@@ -353,9 +400,11 @@ component_create(Point *startpoint,
   cmp->text = new_text("", font, 0.8, &p, &color_black, ALIGN_LEFT);
   text_get_attributes(cmp->text,&cmp->attrs);
 
-  element_init(elem, 8, 8);
+  dia_font_unref(font);
   
-  for (i=0;i<8;i++) {
+  element_init(elem, 8, 10);
+  
+  for (i=0;i<10;i++) {
     obj->connections[i] = &cmp->connections[i];
     cmp->connections[i].object = obj;
     cmp->connections[i].connected = NULL;
