@@ -27,9 +27,10 @@
 #include "intl.h"
 #include "object.h"
 #include "connection.h"
-#include "render.h"
+#include "diarenderer.h"
 #include "handle.h"
 #include "properties.h"
+#include "attributes.h"
 
 #include "pixmaps/implements.xpm"
 
@@ -45,7 +46,10 @@ struct _Implements {
 
   Point circle_center; /* Calculated from diameter*/
 
-  utfchar *text;
+  Color text_color;
+  Color line_color;
+
+  gchar *text;
   Point text_pos;
   real text_width;
 
@@ -60,12 +64,13 @@ struct _Implements {
 
 static DiaFont *implements_font = NULL;
 
-static void implements_move_handle(Implements *implements, Handle *handle,
-				   Point *to, HandleMoveReason reason, ModifierKeys modifiers);
-static void implements_move(Implements *implements, Point *to);
+static ObjectChange* implements_move_handle(Implements *implements, Handle *handle,
+					    Point *to, ConnectionPoint *cp,
+					    HandleMoveReason reason, ModifierKeys modifiers);
+static ObjectChange* implements_move(Implements *implements, Point *to);
 static void implements_select(Implements *implements, Point *clicked_point,
-			      Renderer *interactive_renderer);
-static void implements_draw(Implements *implements, Renderer *renderer);
+			      DiaRenderer *interactive_renderer);
+static void implements_draw(Implements *implements, DiaRenderer *renderer);
 static Object *implements_create(Point *startpoint,
 				 void *user_data,
 				 Handle **handle1,
@@ -117,6 +122,8 @@ static ObjectOps implements_ops = {
 
 static PropDescription implements_props[] = {
   CONNECTION_COMMON_PROPERTIES,
+  PROP_STD_LINE_COLOUR_OPTIONAL, 
+  PROP_STD_TEXT_COLOUR_OPTIONAL,
   { "text", PROP_TYPE_STRING, PROP_FLAG_VISIBLE,
     N_("Interface:"), NULL, NULL },
   { "text_pos", PROP_TYPE_POINT, 0, NULL, NULL, NULL },
@@ -134,6 +141,8 @@ implements_describe_props(Implements *implements)
 
 static PropOffset implements_offsets[] = {
   CONNECTION_COMMON_PROPERTIES_OFFSETS,
+  { "line_colour",PROP_TYPE_COLOUR,offsetof(Implements, line_color) },
+  { "text_colour",PROP_TYPE_COLOUR,offsetof(Implements, text_color) },
   { "text", PROP_TYPE_STRING, offsetof(Implements, text) },
   { "text_pos", PROP_TYPE_POINT, offsetof(Implements, text_pos) },
   { "diameter", PROP_TYPE_REAL, offsetof(Implements, circle_diameter) },
@@ -174,14 +183,15 @@ implements_distance_from(Implements *implements, Point *point)
 
 static void
 implements_select(Implements *implements, Point *clicked_point,
-	    Renderer *interactive_renderer)
+	    DiaRenderer *interactive_renderer)
 {
   connection_update_handles(&implements->connection);
 }
 
-static void
+static ObjectChange*
 implements_move_handle(Implements *implements, Handle *handle,
-		 Point *to, HandleMoveReason reason, ModifierKeys modifiers)
+		       Point *to, ConnectionPoint *cp,
+		       HandleMoveReason reason, ModifierKeys modifiers)
 {
   Point v1, v2;
   
@@ -202,15 +212,18 @@ implements_move_handle(Implements *implements, Handle *handle,
       implements->circle_diameter = 0.03;
   } else {
     v1 = implements->connection.endpoints[1];
-    connection_move_handle(&implements->connection, handle->id, to, reason);
+    connection_move_handle(&implements->connection, handle->id, to, cp, 
+			   reason, modifiers);
     point_sub(&v1, &implements->connection.endpoints[1]);
     point_sub(&implements->text_pos, &v1);
   }
 
   implements_update_data(implements);
+
+  return NULL;
 }
 
-static void
+static ObjectChange*
 implements_move(Implements *implements, Point *to)
 {
   Point start_to_end;
@@ -229,11 +242,14 @@ implements_move(Implements *implements, Point *to)
   point_add(&implements->text_pos, &delta);
   
   implements_update_data(implements);
+
+  return NULL;
 }
 
 static void
-implements_draw(Implements *implements, Renderer *renderer)
+implements_draw(Implements *implements, DiaRenderer *renderer)
 {
+  DiaRendererClass *renderer_ops = DIA_RENDERER_GET_CLASS (renderer);
   Point *endpoints;
   
   assert(implements != NULL);
@@ -241,30 +257,30 @@ implements_draw(Implements *implements, Renderer *renderer)
 
   endpoints = &implements->connection.endpoints[0];
   
-  renderer->ops->set_linewidth(renderer, IMPLEMENTS_WIDTH);
-  renderer->ops->set_linestyle(renderer, LINESTYLE_SOLID);
-  renderer->ops->set_linecaps(renderer, LINECAPS_BUTT);
+  renderer_ops->set_linewidth(renderer, IMPLEMENTS_WIDTH);
+  renderer_ops->set_linestyle(renderer, LINESTYLE_SOLID);
+  renderer_ops->set_linecaps(renderer, LINECAPS_BUTT);
 
-  renderer->ops->draw_line(renderer,
+  renderer_ops->draw_line(renderer,
 			   &endpoints[0], &endpoints[1],
-			   &color_black);
+			   &implements->line_color);
 
-  renderer->ops->fill_ellipse(renderer, &implements->circle_center,
+  renderer_ops->fill_ellipse(renderer, &implements->circle_center,
 			      implements->circle_diameter,
 			      implements->circle_diameter,
 			      &color_white);
-  renderer->ops->draw_ellipse(renderer, &implements->circle_center,
+  renderer_ops->draw_ellipse(renderer, &implements->circle_center,
 			      implements->circle_diameter,
 			      implements->circle_diameter,
-			      &color_black);
+			      &implements->line_color);
 
 
-  renderer->ops->set_font(renderer, implements_font, IMPLEMENTS_FONTHEIGHT);
+  renderer_ops->set_font(renderer, implements_font, IMPLEMENTS_FONTHEIGHT);
   if (implements->text)
-    renderer->ops->draw_string(renderer,
+    renderer_ops->draw_string(renderer,
 			       implements->text,
 			       &implements->text_pos, ALIGN_LEFT,
-			       &color_black);
+			       &implements->text_color);
 }
 
 static Object *
@@ -279,9 +295,8 @@ implements_create(Point *startpoint,
   Point defaultlen = { 1.0, 1.0 };
 
   if (implements_font == NULL) {
-	  /* choose default font name for your locale. see also font_data structure
-	     in lib/font.c. if "Courier" works for you, it would be better.  */
-	  implements_font = font_getfont (_("Courier"));
+    implements_font = 
+      dia_font_new_from_style(DIA_FONT_MONOSPACE, IMPLEMENTS_FONTHEIGHT);
   }
   
   implements = g_malloc0(sizeof(Implements));
@@ -298,6 +313,8 @@ implements_create(Point *startpoint,
   
   connection_init(conn, 4, 0);
 
+  implements->line_color = attributes_get_foreground();
+  implements->text_color = color_black;
   implements->text = NULL;
   implements->text_width = 0.0;
   implements->text_pos = conn->endpoints[1];
@@ -344,9 +361,9 @@ implements_update_data(Implements *implements)
 
   implements->text_width = 0.0;
   if (implements->text)
-    implements->text_width = font_string_width(implements->text,
-					       implements_font,
-					       IMPLEMENTS_FONTHEIGHT);
+    implements->text_width = dia_font_string_width(implements->text,
+                                                   implements_font,
+                                                   IMPLEMENTS_FONTHEIGHT);
 
   obj->position = conn->endpoints[0];
 
@@ -381,7 +398,11 @@ implements_update_data(Implements *implements)
   /* Add boundingbox for text: */
   rect.left = implements->text_pos.x;
   rect.right = rect.left + implements->text_width;
-  rect.top = implements->text_pos.y - font_ascent(implements_font, IMPLEMENTS_FONTHEIGHT);
+  rect.top = implements->text_pos.y;
+  if (implements->text)
+    rect.top -= dia_font_ascent(implements->text,
+				implements_font,
+				IMPLEMENTS_FONTHEIGHT);
   rect.bottom = rect.top + IMPLEMENTS_FONTHEIGHT;
   rectangle_union(&obj->bounding_box, &rect);
 }
