@@ -27,7 +27,7 @@
 #include "intl.h"
 #include "object.h"
 #include "element.h"
-#include "render.h"
+#include "diarenderer.h"
 #include "attributes.h"
 #include "text.h"
 #include "properties.h"
@@ -42,6 +42,9 @@ struct _Note {
 
   Text *text;
   TextAttributes attrs;
+
+  Color line_color;
+  Color fill_color;
 };
 
 #define NOTE_BORDERWIDTH 0.1
@@ -52,11 +55,12 @@ struct _Note {
 
 static real note_distance_from(Note *note, Point *point);
 static void note_select(Note *note, Point *clicked_point,
-			Renderer *interactive_renderer);
-static void note_move_handle(Note *note, Handle *handle,
-			     Point *to, HandleMoveReason reason, ModifierKeys modifiers);
-static void note_move(Note *note, Point *to);
-static void note_draw(Note *note, Renderer *renderer);
+			DiaRenderer *interactive_renderer);
+static ObjectChange* note_move_handle(Note *note, Handle *handle,
+				      Point *to, ConnectionPoint *cp,
+				      HandleMoveReason reason, ModifierKeys modifiers);
+static ObjectChange* note_move(Note *note, Point *to);
+static void note_draw(Note *note, DiaRenderer *renderer);
 static Object *note_create(Point *startpoint,
 			   void *user_data,
 			   Handle **handle1,
@@ -107,9 +111,11 @@ static ObjectOps note_ops = {
 
 static PropDescription note_props[] = {
   ELEMENT_COMMON_PROPERTIES,
+  PROP_STD_LINE_COLOUR_OPTIONAL, 
+  PROP_STD_FILL_COLOUR_OPTIONAL, 
   PROP_STD_TEXT_FONT,
   PROP_STD_TEXT_HEIGHT,
-  PROP_STD_TEXT_COLOUR,
+  PROP_STD_TEXT_COLOUR_OPTIONAL,
   { "text", PROP_TYPE_TEXT, 0, N_("Text"), NULL, NULL },   
   PROP_DESC_END
 };
@@ -117,11 +123,16 @@ static PropDescription note_props[] = {
 static PropDescription *
 note_describe_props(Note *note)
 {
+  if (note_props[0].quark == 0) {
+    prop_desc_list_calculate_quarks(note_props);
+  }
   return note_props;
 }
 
 static PropOffset note_offsets[] = {
   ELEMENT_COMMON_PROPERTIES_OFFSETS,
+  {"line_colour",PROP_TYPE_COLOUR,offsetof(Note,line_color)},
+  {"fill_colour",PROP_TYPE_COLOUR,offsetof(Note,fill_color)},
   {"text",PROP_TYPE_TEXT,offsetof(Note,text)},
   {"text_font",PROP_TYPE_FONT,offsetof(Note,attrs.font)},
   {"text_height",PROP_TYPE_REAL,offsetof(Note,attrs.height)},
@@ -156,35 +167,41 @@ note_distance_from(Note *note, Point *point)
 
 static void
 note_select(Note *note, Point *clicked_point,
-	       Renderer *interactive_renderer)
+	       DiaRenderer *interactive_renderer)
 {
   text_set_cursor(note->text, clicked_point, interactive_renderer);
   text_grab_focus(note->text, &note->element.object);
   element_update_handles(&note->element);
 }
 
-static void
+static ObjectChange*
 note_move_handle(Note *note, Handle *handle,
-		 Point *to, HandleMoveReason reason, ModifierKeys modifiers)
+		 Point *to, ConnectionPoint *cp,
+		 HandleMoveReason reason, ModifierKeys modifiers)
 {
   assert(note!=NULL);
   assert(handle!=NULL);
   assert(to!=NULL);
 
   assert(handle->id < 8);
+
+  return NULL;
 }
 
-static void
+static ObjectChange*
 note_move(Note *note, Point *to)
 {
   note->element.corner = *to;
 
   note_update_data(note);
+
+  return NULL;
 }
 
 static void
-note_draw(Note *note, Renderer *renderer)
+note_draw(Note *note, DiaRenderer *renderer)
 {
+  DiaRendererClass *renderer_ops = DIA_RENDERER_GET_CLASS (renderer);
   Element *elem;
   real x, y, w, h;
   Point poly[5];
@@ -199,9 +216,9 @@ note_draw(Note *note, Renderer *renderer)
   w = elem->width;
   h = elem->height;
   
-  renderer->ops->set_fillstyle(renderer, FILLSTYLE_SOLID);
-  renderer->ops->set_linewidth(renderer, NOTE_BORDERWIDTH);
-  renderer->ops->set_linestyle(renderer, LINESTYLE_SOLID);
+  renderer_ops->set_fillstyle(renderer, FILLSTYLE_SOLID);
+  renderer_ops->set_linewidth(renderer, NOTE_BORDERWIDTH);
+  renderer_ops->set_linestyle(renderer, LINESTYLE_SOLID);
 
   poly[0].x = x;
   poly[0].y = y;
@@ -214,22 +231,22 @@ note_draw(Note *note, Renderer *renderer)
   poly[4].x = x;
   poly[4].y = y+h;
 
-  renderer->ops->fill_polygon(renderer, 
+  renderer_ops->fill_polygon(renderer, 
 			      poly, 5,
-			      &color_white);
-  renderer->ops->draw_polygon(renderer, 
+			      &note->fill_color);
+  renderer_ops->draw_polygon(renderer, 
 			      poly, 5,
-			      &color_black);
+			      &note->line_color);
 
   poly[0] = poly[1];
   poly[1].x = x + w - NOTE_CORNER;
   poly[1].y = y + NOTE_CORNER;
   poly[2] = poly[2];
  
-  renderer->ops->set_linewidth(renderer, NOTE_CORNERWIDTH);
-  renderer->ops->draw_polyline(renderer, 
+  renderer_ops->set_linewidth(renderer, NOTE_CORNERWIDTH);
+  renderer_ops->draw_polyline(renderer, 
 			   poly, 3,
-			   &color_black);
+			   &note->line_color);
 
   text_draw(note->text, renderer);
 }
@@ -254,20 +271,28 @@ note_update_data(Note *note)
 
   /* Update connections: */
   note->connections[0].pos = elem->corner;
+  note->connections[0].directions = DIR_NORTH|DIR_WEST;
   note->connections[1].pos.x = elem->corner.x + elem->width / 2.0;
   note->connections[1].pos.y = elem->corner.y;
+  note->connections[0].directions = DIR_NORTH;
   note->connections[2].pos.x = elem->corner.x + elem->width;
   note->connections[2].pos.y = elem->corner.y;
+  note->connections[2].directions = DIR_NORTH|DIR_EAST;
   note->connections[3].pos.x = elem->corner.x;
   note->connections[3].pos.y = elem->corner.y + elem->height / 2.0;
+  note->connections[3].directions = DIR_WEST;
   note->connections[4].pos.x = elem->corner.x + elem->width;
   note->connections[4].pos.y = elem->corner.y + elem->height / 2.0;
+  note->connections[4].directions = DIR_EAST;
   note->connections[5].pos.x = elem->corner.x;
   note->connections[5].pos.y = elem->corner.y + elem->height;
+  note->connections[5].directions = DIR_SOUTH|DIR_WEST;
   note->connections[6].pos.x = elem->corner.x + elem->width / 2.0;
   note->connections[6].pos.y = elem->corner.y + elem->height;
+  note->connections[6].directions = DIR_SOUTH;
   note->connections[7].pos.x = elem->corner.x + elem->width;
   note->connections[7].pos.y = elem->corner.y + elem->height;
+  note->connections[7].directions = DIR_SOUTH|DIR_EAST;
   
   element_update_boundingbox(elem);
 
@@ -298,14 +323,17 @@ note_create(Point *startpoint,
 
   elem->corner = *startpoint;
 
-  /* choose default font name for your locale. see also font_data structure
-     in lib/font.c. if "Courier" works for you, it would be better.  */
-  font = font_getfont (_("Courier"));
+  note->line_color = attributes_get_foreground();
+  note->fill_color = attributes_get_background();
+
+  font = dia_font_new_from_style (DIA_FONT_MONOSPACE, 0.8);
   p = *startpoint;
   p.x += NOTE_BORDERWIDTH/2.0 + NOTE_MARGIN_X;
-  p.y += NOTE_BORDERWIDTH/2.0 + NOTE_CORNER + font_ascent(font, 0.8);
+  p.y += NOTE_BORDERWIDTH/2.0 + NOTE_CORNER + dia_font_ascent("A",font, 0.8);
   
   note->text = new_text("", font, 0.8, &p, &color_black, ALIGN_LEFT);
+  dia_font_unref(font);
+
   text_get_attributes(note->text,&note->attrs);
   
   element_init(elem, 8, 8);
