@@ -50,6 +50,10 @@ struct _Objet {
   char *exstate;  /* used for explicit state */
   Text *attributes;
 
+  Color text_color;
+  Color line_color;
+  Color fill_color;
+
   Point ex_pos, st_pos;
   int is_active;
   int show_attributes;
@@ -71,9 +75,10 @@ struct _Objet {
 static real objet_distance_from(Objet *ob, Point *point);
 static void objet_select(Objet *ob, Point *clicked_point,
 			 DiaRenderer *interactive_renderer);
-static void objet_move_handle(Objet *ob, Handle *handle,
-			      Point *to, HandleMoveReason reason, ModifierKeys modifiers);
-static void objet_move(Objet *ob, Point *to);
+static ObjectChange* objet_move_handle(Objet *ob, Handle *handle,
+				       Point *to, ConnectionPoint *cp,
+				       HandleMoveReason reason, ModifierKeys modifiers);
+static ObjectChange* objet_move(Objet *ob, Point *to);
 static void objet_draw(Objet *ob, DiaRenderer *renderer);
 static Object *objet_create(Point *startpoint,
 			    void *user_data,
@@ -134,10 +139,9 @@ static ObjectOps objet_ops = {
 
 static PropDescription objet_props[] = {
   ELEMENT_COMMON_PROPERTIES,
-  /*PROP_STD_TEXT_ALIGNMENT,
-  PROP_STD_TEXT_FONT,
-  PROP_STD_TEXT_HEIGHT,
-  PROP_STD_TEXT_COLOUR,*/
+  PROP_STD_TEXT_COLOUR_OPTIONAL,
+  PROP_STD_LINE_COLOUR_OPTIONAL, 
+  PROP_STD_FILL_COLOUR_OPTIONAL, 
   { "text", PROP_TYPE_TEXT, 0, N_("Text"), NULL, NULL },
   { "stereotype", PROP_TYPE_STRING, PROP_FLAG_VISIBLE,
     N_("Stereotype"), NULL, NULL },
@@ -152,17 +156,23 @@ static PropDescription objet_props[] = {
     N_("Show attributes"),NULL,NULL}, 
   { "multiple", PROP_TYPE_BOOL, PROP_FLAG_VISIBLE,
     N_("Multiple instance"),NULL,NULL}, 
-  { NULL, 0, 0, NULL, NULL, NULL, 0}
+  PROP_DESC_END
 };
 
 static PropDescription *
 objet_describe_props(Objet *ob)
 {
+  if (objet_props[0].quark == 0) {
+    prop_desc_list_calculate_quarks(objet_props);
+  }
   return objet_props;
 }
 
 static PropOffset objet_offsets[] = {
   ELEMENT_COMMON_PROPERTIES_OFFSETS,
+  { "text_colour",PROP_TYPE_COLOUR,offsetof(Objet, text_color) },
+  { "line_colour",PROP_TYPE_COLOUR,offsetof(Objet, line_color) },
+  { "fill_colour",PROP_TYPE_COLOUR,offsetof(Objet, fill_color) },
   { "name", PROP_TYPE_STRING, offsetof(Objet, exstate) },
   { "stereotype", PROP_TYPE_STRING, offsetof(Objet, stereotype) },
   { "text", PROP_TYPE_TEXT, offsetof(Objet, text) },
@@ -212,22 +222,27 @@ objet_select(Objet *ob, Point *clicked_point,
   element_update_handles(&ob->element);
 }
 
-static void
+static ObjectChange*
 objet_move_handle(Objet *ob, Handle *handle,
-			 Point *to, HandleMoveReason reason, ModifierKeys modifiers)
+		  Point *to, ConnectionPoint *cp,
+		  HandleMoveReason reason, ModifierKeys modifiers)
 {
   assert(ob!=NULL);
   assert(handle!=NULL);
   assert(to!=NULL);
 
   assert(handle->id < 8);
+
+  return NULL;
 }
 
-static void
+static ObjectChange*
 objet_move(Objet *ob, Point *to)
 {
   ob->element.corner = *to;
   objet_update_data(ob);
+
+  return NULL;
 }
 
 static void
@@ -264,10 +279,10 @@ objet_draw(Objet *ob, DiaRenderer *renderer)
     p2.y -= OBJET_MARGIN_M;
     renderer_ops->fill_rect(renderer, 
 			     &p1, &p2,
-			     &color_white); 
+			     &ob->fill_color);
     renderer_ops->draw_rect(renderer, 
 			     &p1, &p2,
-			     &color_black);
+			     &ob->line_color);
     p1.x -= OBJET_MARGIN_M;
     p1.y += OBJET_MARGIN_M;
     p2.x -= OBJET_MARGIN_M;
@@ -277,10 +292,10 @@ objet_draw(Objet *ob, DiaRenderer *renderer)
     
   renderer_ops->fill_rect(renderer, 
 			   &p1, &p2,
-			   &color_white);
+			   &ob->fill_color);
   renderer_ops->draw_rect(renderer, 
 			   &p1, &p2,
-			   &color_black);
+			   &ob->line_color);
 
   
   text_draw(ob->text, renderer);
@@ -289,14 +304,14 @@ objet_draw(Objet *ob, DiaRenderer *renderer)
       renderer_ops->draw_string(renderer,
 				 ob->st_stereotype,
 				 &ob->st_pos, ALIGN_CENTER,
-				 &color_black);
+				 &ob->text_color);
   }
 
   if ((ob->exstate != NULL) && (ob->exstate[0] != '\0')) {
       renderer_ops->draw_string(renderer,
 				 ob->exstate,
 				 &ob->ex_pos, ALIGN_CENTER,
-				 &color_black);
+				 &ob->text_color);
   }
 
   /* Is there a better way to underline? */
@@ -312,7 +327,7 @@ objet_draw(Objet *ob, DiaRenderer *renderer)
     p2.x = p1.x + ob->text->row_width[i];
     renderer_ops->draw_line(renderer,
 			     &p1, &p2,
-			     &color_black);
+			     &ob->text_color);
     p1.y = p2.y += ob->text->height;
   }
 
@@ -323,7 +338,7 @@ objet_draw(Objet *ob, DiaRenderer *renderer)
       renderer_ops->set_linewidth(renderer, bw);
       renderer_ops->draw_line(renderer,
 			       &p1, &p2,
-			       &color_black);
+			       &ob->line_color);
 
       text_draw(ob->attributes, renderer);
   }
@@ -414,7 +429,16 @@ objet_update_data(Objet *ob)
   ob->connections[7].pos.x = elem->corner.x + elem->width;
   ob->connections[7].pos.y = elem->corner.y + elem->height;
   
-  element_update_boundingbox(elem);
+  ob->connections[0].directions = DIR_NORTH|DIR_WEST;
+  ob->connections[1].directions = DIR_NORTH;
+  ob->connections[2].directions = DIR_NORTH|DIR_EAST;
+  ob->connections[3].directions = DIR_WEST;
+  ob->connections[4].directions = DIR_EAST;
+  ob->connections[5].directions = DIR_SOUTH|DIR_WEST;
+  ob->connections[6].directions = DIR_SOUTH;
+  ob->connections[7].directions = DIR_SOUTH|DIR_EAST;
+                                                                                                
+                                                                                                  element_update_boundingbox(elem);
   obj->position = elem->corner;
   element_update_handles(elem);
 }
@@ -442,6 +466,10 @@ objet_create(Point *startpoint,
 
   elem->corner = *startpoint;
 
+  ob->text_color = color_black;
+  ob->line_color = attributes_get_foreground();
+  ob->fill_color = attributes_get_background();
+
   font = dia_font_new_from_style(DIA_FONT_SANS, OBJET_FONTHEIGHT);
   
   ob->show_attributes = FALSE;
