@@ -27,7 +27,7 @@
 #include "intl.h"
 #include "object.h"
 #include "element.h"
-#include "render.h"
+#include "diarenderer.h"
 #include "attributes.h"
 #include "text.h"
 #include "properties.h"
@@ -42,11 +42,14 @@ struct _SmallPackage {
 
   ConnectionPoint connections[8];
 
-  utfchar *stereotype;
+  char *stereotype;
   Text *text;
   
-  utfchar *st_stereotype;
+  char *st_stereotype;
   TextAttributes attrs;
+
+  Color line_color;
+  Color fill_color;
 };
 
 #define SMALLPACKAGE_BORDERWIDTH 0.1
@@ -57,11 +60,12 @@ struct _SmallPackage {
 
 static real smallpackage_distance_from(SmallPackage *pkg, Point *point);
 static void smallpackage_select(SmallPackage *pkg, Point *clicked_point,
-				Renderer *interactive_renderer);
-static void smallpackage_move_handle(SmallPackage *pkg, Handle *handle,
-				     Point *to, HandleMoveReason reason, ModifierKeys modifiers);
-static void smallpackage_move(SmallPackage *pkg, Point *to);
-static void smallpackage_draw(SmallPackage *pkg, Renderer *renderer);
+				DiaRenderer *interactive_renderer);
+static ObjectChange* smallpackage_move_handle(SmallPackage *pkg, Handle *handle,
+					      Point *to, ConnectionPoint *cp,
+					      HandleMoveReason reason, ModifierKeys modifiers);
+static ObjectChange* smallpackage_move(SmallPackage *pkg, Point *to);
+static void smallpackage_draw(SmallPackage *pkg, DiaRenderer *renderer);
 static Object *smallpackage_create(Point *startpoint,
 				   void *user_data,
 				   Handle **handle1,
@@ -112,6 +116,8 @@ static ObjectOps smallpackage_ops = {
 
 static PropDescription smallpackage_props[] = {
   ELEMENT_COMMON_PROPERTIES,
+  PROP_STD_LINE_COLOUR_OPTIONAL, 
+  PROP_STD_FILL_COLOUR_OPTIONAL, 
   { "stereotype", PROP_TYPE_STRING, PROP_FLAG_VISIBLE,
   N_("Stereotype"), NULL, NULL },
   PROP_STD_TEXT_FONT,
@@ -124,11 +130,16 @@ static PropDescription smallpackage_props[] = {
 static PropDescription *
 smallpackage_describe_props(SmallPackage *smallpackage)
 {
+  if (smallpackage_props[0].quark == 0) {
+    prop_desc_list_calculate_quarks(smallpackage_props);
+  }
   return smallpackage_props;
 }
 
 static PropOffset smallpackage_offsets[] = {
   ELEMENT_COMMON_PROPERTIES_OFFSETS,
+  {"line_colour", PROP_TYPE_COLOUR, offsetof(SmallPackage, line_color) },
+  {"fill_colour", PROP_TYPE_COLOUR, offsetof(SmallPackage, fill_color) },
   {"stereotype", PROP_TYPE_STRING, offsetof(SmallPackage , stereotype) },
   {"text",PROP_TYPE_TEXT,offsetof(SmallPackage,text)},
   {"text_font",PROP_TYPE_FONT,offsetof(SmallPackage,attrs.font)},
@@ -168,25 +179,28 @@ smallpackage_distance_from(SmallPackage *pkg, Point *point)
 
 static void
 smallpackage_select(SmallPackage *pkg, Point *clicked_point,
-	       Renderer *interactive_renderer)
+	       DiaRenderer *interactive_renderer)
 {
   text_set_cursor(pkg->text, clicked_point, interactive_renderer);
   text_grab_focus(pkg->text, &pkg->element.object);
   element_update_handles(&pkg->element);
 }
 
-static void
+static ObjectChange*
 smallpackage_move_handle(SmallPackage *pkg, Handle *handle,
-			 Point *to, HandleMoveReason reason, ModifierKeys modifiers)
+			 Point *to, ConnectionPoint *cp,
+			 HandleMoveReason reason, ModifierKeys modifiers)
 {
   assert(pkg!=NULL);
   assert(handle!=NULL);
   assert(to!=NULL);
 
   assert(handle->id < 8);
+
+  return NULL;
 }
 
-static void
+static ObjectChange*
 smallpackage_move(SmallPackage *pkg, Point *to)
 {
   Point p;
@@ -199,11 +213,14 @@ smallpackage_move(SmallPackage *pkg, Point *to)
   text_set_position(pkg->text, &p);
   
   smallpackage_update_data(pkg);
+
+  return NULL;
 }
 
 static void
-smallpackage_draw(SmallPackage *pkg, Renderer *renderer)
+smallpackage_draw(SmallPackage *pkg, DiaRenderer *renderer)
 {
+  DiaRendererClass *renderer_ops = DIA_RENDERER_GET_CLASS (renderer);
   Element *elem;
   real x, y, w, h;
   Point p1, p2;
@@ -218,39 +235,39 @@ smallpackage_draw(SmallPackage *pkg, Renderer *renderer)
   w = elem->width;
   h = elem->height;
   
-  renderer->ops->set_fillstyle(renderer, FILLSTYLE_SOLID);
-  renderer->ops->set_linewidth(renderer, SMALLPACKAGE_BORDERWIDTH);
-  renderer->ops->set_linestyle(renderer, LINESTYLE_SOLID);
+  renderer_ops->set_fillstyle(renderer, FILLSTYLE_SOLID);
+  renderer_ops->set_linewidth(renderer, SMALLPACKAGE_BORDERWIDTH);
+  renderer_ops->set_linestyle(renderer, LINESTYLE_SOLID);
 
 
   p1.x = x; p1.y = y;
   p2.x = x+w; p2.y = y+h;
 
-  renderer->ops->fill_rect(renderer, 
+  renderer_ops->fill_rect(renderer, 
 			   &p1, &p2,
-			   &color_white);
-  renderer->ops->draw_rect(renderer, 
+			   &pkg->fill_color);
+  renderer_ops->draw_rect(renderer, 
 			   &p1, &p2,
-			   &color_black);
+			   &pkg->line_color);
 
   p1.x= x; p1.y = y-SMALLPACKAGE_TOPHEIGHT;
   p2.x = x+SMALLPACKAGE_TOPWIDTH; p2.y = y;
 
-  renderer->ops->fill_rect(renderer, 
+  renderer_ops->fill_rect(renderer, 
 			   &p1, &p2,
-			   &color_white);
+			   &pkg->fill_color);
   
-  renderer->ops->draw_rect(renderer, 
+  renderer_ops->draw_rect(renderer, 
 			   &p1, &p2,
-			   &color_black);
+			   &pkg->line_color);
 
   text_draw(pkg->text, renderer);
 
   if ((pkg->st_stereotype != NULL) && (pkg->st_stereotype[0] != '\0')) {
     p1 = pkg->text->position;
     p1.y -= pkg->text->height;
-    renderer->ops->draw_string(renderer, pkg->st_stereotype, &p1, 
-			       ALIGN_LEFT, &color_black);
+    renderer_ops->draw_string(renderer, pkg->st_stereotype, &p1, 
+			       ALIGN_LEFT, &pkg->attrs.color);
   }
 }
 
@@ -280,9 +297,10 @@ smallpackage_update_data(SmallPackage *pkg)
   if ((pkg->stereotype != NULL) && (pkg->stereotype[0] != '\0')) {
     font = pkg->text->font;
     elem->height += pkg->text->height;
-    elem->width = MAX(elem->width, font_string_width(pkg->st_stereotype,
-						     font, pkg->text->height)+
-		      2*SMALLPACKAGE_MARGIN_X);
+    elem->width = MAX(elem->width,
+                      dia_font_string_width(pkg->st_stereotype,
+                                            font, pkg->text->height)+
+                      2*SMALLPACKAGE_MARGIN_X);
     p.y += pkg->text->height;
   }
 
@@ -305,6 +323,15 @@ smallpackage_update_data(SmallPackage *pkg)
   pkg->connections[7].pos.x = elem->corner.x + elem->width;
   pkg->connections[7].pos.y = elem->corner.y + elem->height;
   
+  pkg->connections[0].directions = DIR_NORTH|DIR_WEST;
+  pkg->connections[1].directions = DIR_NORTH;
+  pkg->connections[2].directions = DIR_NORTH|DIR_EAST;
+  pkg->connections[3].directions = DIR_WEST;
+  pkg->connections[4].directions = DIR_EAST;
+  pkg->connections[5].directions = DIR_SOUTH|DIR_WEST;
+  pkg->connections[6].directions = DIR_SOUTH;
+  pkg->connections[7].directions = DIR_SOUTH|DIR_EAST;
+                                                                                          
   element_update_boundingbox(elem);
   /* fix boundingbox for top rectangle: */
   obj->bounding_box.top -= SMALLPACKAGE_TOPHEIGHT;
@@ -337,14 +364,13 @@ smallpackage_create(Point *startpoint,
 
   elem->corner = *startpoint;
 
-  /* choose default font name for your locale. see also font_data structure
-     in lib/font.c. if "Courier" works for you, it would be better.  */
-  font = font_getfont (_("Courier"));
+  font = dia_font_new_from_style(DIA_FONT_MONOSPACE, 0.8);
   p = *startpoint;
   p.x += SMALLPACKAGE_MARGIN_X;
-  p.y += SMALLPACKAGE_MARGIN_Y + font_ascent(font, 0.8);
+  p.y += SMALLPACKAGE_MARGIN_Y+ dia_font_ascent("A",font, 0.8);
   
   pkg->text = new_text("", font, 0.8, &p, &color_black, ALIGN_LEFT);
+  dia_font_unref(font);
   text_get_attributes(pkg->text,&pkg->attrs);
   
   element_init(elem, 8, 8);
@@ -356,6 +382,9 @@ smallpackage_create(Point *startpoint,
   }
   elem->extra_spacing.border_trans = SMALLPACKAGE_BORDERWIDTH/2.0;
 
+  pkg->line_color = attributes_get_foreground();
+  pkg->fill_color = attributes_get_background();
+
   pkg->stereotype = NULL;
   pkg->st_stereotype = NULL;
   smallpackage_update_data(pkg);
@@ -373,7 +402,7 @@ static void
 smallpackage_destroy(SmallPackage *pkg)
 {
   text_destroy(pkg->text);
-
+  
   g_free(pkg->stereotype);
   g_free(pkg->st_stereotype);
 
