@@ -27,7 +27,6 @@
 #endif
 
 #include <assert.h>
-#include <gtk/gtk.h>
 #include <math.h>
 #include <string.h>
 #include <glib.h>
@@ -36,7 +35,7 @@
 #include "object.h"
 #include "element.h"
 #include "connectionpoint.h"
-#include "render.h"
+#include "diarenderer.h"
 #include "attributes.h"
 #include "text.h"
 #include "widgets.h"
@@ -85,12 +84,13 @@ typedef struct _Chronoline {
 
 static real chronoline_distance_from(Chronoline *chronoline, Point *point);
 static void chronoline_select(Chronoline *chronoline, Point *clicked_point,
-		       Renderer *interactive_renderer);
-static void chronoline_move_handle(Chronoline *chronoline, Handle *handle,
-			    Point *to, HandleMoveReason reason, 
+		       DiaRenderer *interactive_renderer);
+static ObjectChange* chronoline_move_handle(Chronoline *chronoline, Handle *handle,
+					    Point *to, ConnectionPoint *cp,
+					    HandleMoveReason reason, 
 			    ModifierKeys modifiers);
-static void chronoline_move(Chronoline *chronoline, Point *to);
-static void chronoline_draw(Chronoline *chronoline, Renderer *renderer);
+static ObjectChange* chronoline_move(Chronoline *chronoline, Point *to);
+static void chronoline_draw(Chronoline *chronoline, DiaRenderer *renderer);
 static void chronoline_update_data(Chronoline *chronoline);
 static Object *chronoline_create(Point *startpoint,
 			  void *user_data,
@@ -184,7 +184,7 @@ static PropDescription chronoline_props[] = {
   { "font_size", PROP_TYPE_REAL, PROP_FLAG_VISIBLE,
     N_("Font size"), NULL, &prop_std_text_height_data },
   { "font_color", PROP_TYPE_COLOUR, PROP_FLAG_VISIBLE,
-    N_("Text colour"), NULL, NULL },
+    N_("Text color"), NULL, NULL },
 
   PROP_STD_NOTEBOOK_END,
   {NULL}
@@ -252,37 +252,44 @@ chronoline_distance_from(Chronoline *chronoline, Point *point)
 
 static void
 chronoline_select(Chronoline *chronoline, Point *clicked_point,
-	   Renderer *interactive_renderer)
+	   DiaRenderer *interactive_renderer)
 {
   element_update_handles(&chronoline->element);
 }
 
-static void
+static ObjectChange*
 chronoline_move_handle(Chronoline *chronoline, Handle *handle,
-		Point *to, HandleMoveReason reason, ModifierKeys modifiers)
+		       Point *to, ConnectionPoint *cp,
+		       HandleMoveReason reason, ModifierKeys modifiers)
 {
   g_assert(chronoline!=NULL);
   g_assert(handle!=NULL);
   g_assert(to!=NULL);
 
-  element_move_handle(&chronoline->element, handle->id, to, reason);
+  element_move_handle(&chronoline->element, handle->id, to, cp, 
+		      reason, modifiers);
   chronoline_update_data(chronoline);
+
+  return NULL;
 }
 
-static void
+static ObjectChange*
 chronoline_move(Chronoline *chronoline, Point *to)
 {
   chronoline->element.corner = *to;
   chronoline_update_data(chronoline);
+
+  return NULL;
 }
 
 static void 
 cld_onebit(Chronoline *chronoline,
-	   Renderer *renderer,
+	   DiaRenderer *renderer,
 	   real x1,CLEventType s1,
 	   real x2,CLEventType s2,
 	   gboolean fill)
 {
+  DiaRendererClass *renderer_ops = DIA_RENDERER_GET_CLASS (renderer);
   Point pts[4];
   real y_down = chronoline->y_down;
   real y_up = chronoline->y_up;
@@ -296,14 +303,14 @@ cld_onebit(Chronoline *chronoline,
 
   if (fill) {
     if ((s1 == CLE_UNKNOWN) || (s2 == CLE_UNKNOWN)) {
-      renderer->ops->fill_polygon(renderer,pts,sizeof(pts)/sizeof(pts[0]),
+      renderer_ops->fill_polygon(renderer,pts,sizeof(pts)/sizeof(pts[0]),
 				  &chronoline->datagray);
     } else {
-      renderer->ops->fill_polygon(renderer,pts,sizeof(pts)/sizeof(pts[0]),
+      renderer_ops->fill_polygon(renderer,pts,sizeof(pts)/sizeof(pts[0]),
 				  &color_white);
     }    
   } else {
-    renderer->ops->draw_line(renderer,&pts[1],&pts[2],
+    renderer_ops->draw_line(renderer,&pts[1],&pts[2],
 			     &chronoline->data_color);
   }
 }
@@ -311,11 +318,12 @@ cld_onebit(Chronoline *chronoline,
 
 static void 
 cld_multibit(Chronoline *chronoline,
-	     Renderer *renderer,
+	     DiaRenderer *renderer,
 	     real x1,CLEventType s1,
 	     real x2,CLEventType s2,
 	     gboolean fill)
 {
+  DiaRendererClass *renderer_ops = DIA_RENDERER_GET_CLASS (renderer);
   Point pts[4];
   real ymid = .5 * (chronoline->y_down + chronoline->y_up);
   real y_down = chronoline->y_down;
@@ -339,16 +347,16 @@ cld_multibit(Chronoline *chronoline,
 
   if (fill) { 
     if ((s1 == CLE_UNKNOWN) || (s2 == CLE_UNKNOWN)) {
-      renderer->ops->fill_polygon(renderer,pts,sizeof(pts)/sizeof(pts[0]),
+      renderer_ops->fill_polygon(renderer,pts,sizeof(pts)/sizeof(pts[0]),
 				  &chronoline->datagray);
     } else {
-      renderer->ops->fill_polygon(renderer,pts,sizeof(pts)/sizeof(pts[0]),
+      renderer_ops->fill_polygon(renderer,pts,sizeof(pts)/sizeof(pts[0]),
 				  &color_white);
     }    
   } else {
-    renderer->ops->draw_line(renderer,&pts[1],&pts[2],
+    renderer_ops->draw_line(renderer,&pts[1],&pts[2],
 			     &chronoline->data_color);
-    renderer->ops->draw_line(renderer,&pts[0],&pts[3],
+    renderer_ops->draw_line(renderer,&pts[0],&pts[3],
 			     &chronoline->data_color);
   }
 }
@@ -356,9 +364,10 @@ cld_multibit(Chronoline *chronoline,
 
 
 inline static void
-chronoline_draw_really(Chronoline *chronoline, Renderer *renderer, 
+chronoline_draw_really(Chronoline *chronoline, DiaRenderer *renderer, 
 		       gboolean fill)
 {
+  DiaRendererClass *renderer_ops = DIA_RENDERER_GET_CLASS (renderer);
   Element *elem = &chronoline->element;
   real oldx,newx;
 
@@ -373,9 +382,9 @@ chronoline_draw_really(Chronoline *chronoline, Renderer *renderer,
   oldx = elem->corner.x;
 
   lst = chronoline->evtlist;
-  renderer->ops->set_linejoin(renderer, LINEJOIN_MITER);
-  renderer->ops->set_linestyle(renderer, LINESTYLE_SOLID);
-  renderer->ops->set_linewidth(renderer,chronoline->data_lwidth);
+  renderer_ops->set_linejoin(renderer, LINEJOIN_MITER);
+  renderer_ops->set_linestyle(renderer, LINESTYLE_SOLID);
+  renderer_ops->set_linewidth(renderer,chronoline->data_lwidth);
 
   while (lst) {
     evt = (CLEvent *)lst->data;
@@ -414,8 +423,9 @@ chronoline_draw_really(Chronoline *chronoline, Renderer *renderer,
 }
 
 static void
-chronoline_draw(Chronoline *chronoline, Renderer *renderer)
+chronoline_draw(Chronoline *chronoline, DiaRenderer *renderer)
 {
+  DiaRendererClass *renderer_ops = DIA_RENDERER_GET_CLASS (renderer);
   Element *elem;
   Point lr_corner;
   Point p1,p2,p3;
@@ -425,17 +435,17 @@ chronoline_draw(Chronoline *chronoline, Renderer *renderer)
 
   elem = &chronoline->element;
 
-  renderer->ops->set_linejoin(renderer, LINEJOIN_MITER);
-  renderer->ops->set_linestyle(renderer, LINESTYLE_DOTTED);
-  renderer->ops->set_linewidth(renderer, chronoline->main_lwidth);
+  renderer_ops->set_linejoin(renderer, LINEJOIN_MITER);
+  renderer_ops->set_linestyle(renderer, LINESTYLE_DOTTED);
+  renderer_ops->set_linewidth(renderer, chronoline->main_lwidth);
   p1.x = elem->corner.x + elem->width;
   p1.y = elem->corner.y;
-  renderer->ops->draw_line(renderer,&elem->corner,&p1,&chronoline->gray);
+  renderer_ops->draw_line(renderer,&elem->corner,&p1,&chronoline->gray);
 
   chronoline_draw_really(chronoline,renderer,TRUE);
   chronoline_draw_really(chronoline,renderer,FALSE);
 
-  renderer->ops->set_linestyle(renderer, LINESTYLE_SOLID);
+  renderer_ops->set_linestyle(renderer, LINESTYLE_SOLID);
 
   lr_corner.x = elem->corner.x + elem->width;
   lr_corner.y = elem->corner.y + elem->height;
@@ -444,20 +454,20 @@ chronoline_draw(Chronoline *chronoline, Renderer *renderer)
   p2.x = lr_corner.x;
   p1.y = p2.y = chronoline->y_down;
 
-  renderer->ops->set_linewidth(renderer, chronoline->main_lwidth);
-  renderer->ops->draw_line(renderer,&p1,&p2,&chronoline->color);
+  renderer_ops->set_linewidth(renderer, chronoline->main_lwidth);
+  renderer_ops->draw_line(renderer,&p1,&p2,&chronoline->color);
   p2.x = p1.x = elem->corner.x;
   p1.y = chronoline->y_down;
   p2.y = chronoline->y_up;
-  renderer->ops->draw_line(renderer,&p1,&p2,&chronoline->color);
+  renderer_ops->draw_line(renderer,&p1,&p2,&chronoline->color);
 
-  renderer->ops->set_font(renderer, chronoline->font, 
-			  chronoline->font_size);
+  renderer_ops->set_font(renderer, chronoline->font, 
+                          chronoline->font_size);
   p3.y = lr_corner.y - chronoline->font_size 
-    + font_ascent(chronoline->font, 
-		  chronoline->font_size);
+      + dia_font_ascent(chronoline->name,
+                        chronoline->font,chronoline->font_size);
   p3.x = p1.x - chronoline->main_lwidth;
-  renderer->ops->draw_string(renderer,chronoline->name,&p3,ALIGN_RIGHT,
+  renderer_ops->draw_string(renderer,chronoline->name,&p3,ALIGN_RIGHT,
 			     &chronoline->color);
 }
 
@@ -485,9 +495,9 @@ chronoline_update_data(Chronoline *chronoline)
   grayify(&chronoline->datagray,&chronoline->data_color);
   grayify(&chronoline->gray,&chronoline->color);
 
-  chronoline->labelwidth = font_string_width(chronoline->name,
-					     chronoline->font,
-					     chronoline->font_size);
+  chronoline->labelwidth = dia_font_string_width(chronoline->name,
+                                                 chronoline->font,
+                                                 chronoline->font_size);
 
   chronoline->y_up = elem->corner.y;
   chronoline->y_down = elem->corner.y + elem->height;
@@ -564,9 +574,11 @@ chronoline_update_data(Chronoline *chronoline)
       cp->pos.x = evt->x;
       if (chronoline->multibit) {
 	cp->pos.y = .5 * (chronoline->y_down + chronoline->y_up);
+	cp->directions = DIR_ALL;
       } else {
 	cp->pos.y = (evt->type==CLE_OFF?
 		     chronoline->y_down:chronoline->y_up);
+	cp->directions = (evt->type==CLE_OFF?DIR_SOUTH:DIR_NORTH);
       }
       i++;
       conn_elem = g_slist_next(conn_elem);
@@ -606,9 +618,8 @@ chronoline_create(Point *startpoint,
 
   chronoline->name = g_strdup("");
   chronoline->events = g_strdup("");
-  /* choose default font name for your locale. see also font_data structure
-     in lib/font.c. */
-  chronoline->font = font_getfont (_("Helvetica"));
+
+  chronoline->font = dia_font_new_from_style (DIA_FONT_SANS,1.0);
   chronoline->font_size = 1.0;
   chronoline->font_color = color_black;
   chronoline->start_time = 0.0;
@@ -634,6 +645,9 @@ chronoline_create(Point *startpoint,
 static void 
 chronoline_destroy(Chronoline *chronoline)
 {
+  g_free(chronoline->name);
+  g_free(chronoline->events);
+  dia_font_unref(chronoline->font);
   connpointline_destroy(chronoline->snap);
   destroy_clevent_list(chronoline->evtlist);
   element_destroy(&chronoline->element);
