@@ -19,19 +19,20 @@
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  */
 
+/* DO NOT USE THIS OBJECT AS A BASIS FOR A NEW OBJECT. */
+
 #ifdef HAVE_CONFIG_H
 #include <config.h>
 #endif
 
 #include <assert.h>
-#include <gtk/gtk.h>
 #include <math.h>
 
 #include "intl.h"
 #include "object.h"
 #include "element.h"
 #include "connectionpoint.h"
-#include "render.h"
+#include "diarenderer.h"
 #include "attributes.h"
 #include "text.h"
 #include "widgets.h"
@@ -51,7 +52,6 @@ typedef enum {
 } AnchorShape;
 
 typedef struct _Box Box;
-typedef struct _BoxDefaultsDialog BoxDefaultsDialog;
 
 struct _Box {
   Element element;
@@ -71,39 +71,22 @@ struct _Box {
 };
 
 typedef struct _BoxProperties {
-  Color *fg_color;
-  Color *bg_color;
   gboolean show_background;
-  real border_width;
   real corner_radius;
-
   real padding;
-  Color *font_color;
 } BoxProperties;
 
-struct _BoxDefaultsDialog {
-  GtkWidget *vbox;
-
-  GtkToggleButton *show_background;
-  GtkSpinButton *corner_radius;
-
-  GtkSpinButton *padding;
-  DiaFontSelector *font;
-  GtkSpinButton *font_size;
-};
-
-
-static BoxDefaultsDialog *box_defaults_dialog;
 static BoxProperties default_properties;
 
 static real box_distance_from(Box *box, Point *point);
 static void box_select(Box *box, Point *clicked_point,
-		       Renderer *interactive_renderer);
-static void box_move_handle(Box *box, Handle *handle,
-			    Point *to, HandleMoveReason reason, 
+		       DiaRenderer *interactive_renderer);
+static ObjectChange* box_move_handle(Box *box, Handle *handle,
+				     Point *to, ConnectionPoint *cp,
+				     HandleMoveReason reason, 
 			    ModifierKeys modifiers);
-static void box_move(Box *box, Point *to);
-static void box_draw(Box *box, Renderer *renderer);
+static ObjectChange* box_move(Box *box, Point *to);
+static void box_draw(Box *box, DiaRenderer *renderer);
 static void box_update_data(Box *box, AnchorShape horix, AnchorShape vert);
 static Object *box_create(Point *startpoint,
 			  void *user_data,
@@ -116,16 +99,14 @@ static void box_set_props(Box *box, GPtrArray *props);
 
 static void box_save(Box *box, ObjectNode obj_node, const char *filename);
 static Object *box_load(ObjectNode obj_node, int version, const char *filename);
-static GtkWidget *box_get_defaults(void);
-static void box_apply_defaults(void);
 
 static ObjectTypeOps box_type_ops =
 {
   (CreateFunc) box_create,
   (LoadFunc)   box_load,
   (SaveFunc)   box_save,
-  (GetDefaultsFunc)   box_get_defaults,
-  (ApplyDefaultsFunc) box_apply_defaults
+  (GetDefaultsFunc)   NULL,
+  (ApplyDefaultsFunc) NULL
 };
 
 ObjectType fc_box_type =
@@ -170,6 +151,7 @@ static PropDescription box_props[] = {
   PROP_STD_TEXT_FONT,
   PROP_STD_TEXT_HEIGHT,
   PROP_STD_TEXT_COLOUR,
+  PROP_STD_TEXT_ALIGNMENT,
   PROP_STD_SAVED_TEXT,
   
   { NULL, 0, 0, NULL, NULL, NULL, 0}
@@ -197,6 +179,7 @@ static PropOffset box_offsets[] = {
   {"text_font",PROP_TYPE_FONT,offsetof(Box,attrs.font)},
   {"text_height",PROP_TYPE_REAL,offsetof(Box,attrs.height)},
   {"text_colour",PROP_TYPE_COLOUR,offsetof(Box,attrs.color)},
+  {"text_alignment",PROP_TYPE_ENUM,offsetof(Box,attrs.alignment)},
   { NULL, 0, 0 },
 };
 
@@ -217,18 +200,6 @@ box_set_props(Box *box, GPtrArray *props)
   box_update_data(box, ANCHOR_MIDDLE, ANCHOR_MIDDLE);
 }
 
-static void
-box_apply_defaults()
-{
-  default_properties.corner_radius = gtk_spin_button_get_value_as_float(box_defaults_dialog->corner_radius);
-  default_properties.show_background = gtk_toggle_button_get_active(box_defaults_dialog->show_background);
-
-  default_properties.padding = gtk_spin_button_get_value_as_float(box_defaults_dialog->padding);
-  attributes_set_default_font(
-	dia_font_selector_get_font(box_defaults_dialog->font),
-	gtk_spin_button_get_value_as_float(box_defaults_dialog->font_size));
-}
-
 static void
 init_default_values() {
   static int defaults_initialized = 0;
@@ -240,112 +211,6 @@ init_default_values() {
   }
 }
 
-static GtkWidget *
-box_get_defaults()
-{
-  GtkWidget *vbox;
-  GtkWidget *hbox;
-  GtkWidget *label;
-  GtkWidget *checkbox;
-  GtkWidget *corner_radius;
-  GtkWidget *padding;
-  GtkWidget *fontsel;
-  GtkWidget *font_size;
-  GtkAdjustment *adj;
-  DiaFont *font;
-  real font_height;
-
-  if (box_defaults_dialog == NULL) {
-  
-    init_default_values();
-
-    box_defaults_dialog = g_new(BoxDefaultsDialog, 1);
-
-    vbox = gtk_vbox_new(FALSE, 5);
-    box_defaults_dialog->vbox = vbox;
-
-    gtk_object_ref(GTK_OBJECT(vbox));
-    gtk_object_sink(GTK_OBJECT(vbox));
-
-    hbox = gtk_hbox_new(FALSE, 5);
-    checkbox = gtk_check_button_new_with_label(_("Draw background"));
-    box_defaults_dialog->show_background = GTK_TOGGLE_BUTTON( checkbox );
-    gtk_widget_show(checkbox);
-    gtk_widget_show(hbox);
-    gtk_box_pack_start (GTK_BOX (hbox), checkbox, TRUE, TRUE, 0);
-    gtk_box_pack_start (GTK_BOX (vbox), hbox, TRUE, TRUE, 0);
-
-    hbox = gtk_hbox_new(FALSE, 5);
-    label = gtk_label_new(_("Corner rounding:"));
-    gtk_box_pack_start (GTK_BOX (hbox), label, FALSE, TRUE, 0);
-    gtk_widget_show (label);
-    adj = (GtkAdjustment *) gtk_adjustment_new(0.1, 0.0, 10.0, 0.1, 1.0, 1.0);
-    corner_radius = gtk_spin_button_new(adj, 1.0, 2);
-    gtk_spin_button_set_wrap(GTK_SPIN_BUTTON(corner_radius), TRUE);
-    gtk_spin_button_set_numeric(GTK_SPIN_BUTTON(corner_radius), TRUE);
-    box_defaults_dialog->corner_radius = GTK_SPIN_BUTTON(corner_radius);
-    gtk_box_pack_start(GTK_BOX (hbox), corner_radius, TRUE, TRUE, 0);
-    gtk_widget_show (corner_radius);
-    gtk_widget_show(hbox);
-    gtk_box_pack_start (GTK_BOX(vbox), hbox, TRUE, TRUE, 0);
-
-    hbox = gtk_hbox_new(FALSE, 5);
-    label = gtk_label_new(_("Text padding:"));
-    gtk_box_pack_start (GTK_BOX (hbox), label, FALSE, TRUE, 0);
-    gtk_widget_show (label);
-    adj = (GtkAdjustment *) gtk_adjustment_new(0.1, 0.0, 10.0, 0.1, 1.0, 1.0);
-    padding = gtk_spin_button_new(adj, 1.0, 2);
-    gtk_spin_button_set_wrap(GTK_SPIN_BUTTON(padding), TRUE);
-    gtk_spin_button_set_numeric(GTK_SPIN_BUTTON(padding), TRUE);
-    box_defaults_dialog->padding = GTK_SPIN_BUTTON(padding);
-    gtk_box_pack_start(GTK_BOX (hbox), padding, TRUE, TRUE, 0);
-    gtk_widget_show (padding);
-    gtk_widget_show(hbox);
-    gtk_box_pack_start (GTK_BOX(vbox), hbox, TRUE, TRUE, 0);
-
-    hbox = gtk_hbox_new(FALSE, 5);
-    label = gtk_label_new(_("Font:"));
-    gtk_box_pack_start (GTK_BOX (hbox), label, FALSE, TRUE, 0);
-    gtk_widget_show (label);
-    fontsel = dia_font_selector_new();
-    box_defaults_dialog->font = DIAFONTSELECTOR(fontsel);
-    gtk_box_pack_start (GTK_BOX (hbox), fontsel, TRUE, TRUE, 0);
-    gtk_widget_show (fontsel);
-    gtk_widget_show(hbox);
-    gtk_box_pack_start (GTK_BOX(vbox), hbox, TRUE, TRUE, 0);
-
-    hbox = gtk_hbox_new(FALSE, 5);
-    label = gtk_label_new(_("Font size:"));
-    gtk_box_pack_start (GTK_BOX (hbox), label, FALSE, TRUE, 0);
-    gtk_widget_show (label);
-    adj = (GtkAdjustment *) gtk_adjustment_new(0.1, 0.1, 10.0, 0.1, 1.0, 1.0);
-    font_size = gtk_spin_button_new(adj, 1.0, 2);
-    gtk_spin_button_set_wrap(GTK_SPIN_BUTTON(font_size), TRUE);
-    gtk_spin_button_set_numeric(GTK_SPIN_BUTTON(font_size), TRUE);
-    box_defaults_dialog->font_size = GTK_SPIN_BUTTON(font_size);
-    gtk_box_pack_start(GTK_BOX (hbox), font_size, TRUE, TRUE, 0);
-    gtk_widget_show (font_size);
-    gtk_widget_show(hbox);
-    gtk_box_pack_start (GTK_BOX(vbox), hbox, TRUE, TRUE, 0);
-
-    gtk_widget_show (vbox);
-    gtk_widget_show (vbox);
-  }
-
-  gtk_toggle_button_set_active(box_defaults_dialog->show_background, 
-			       default_properties.show_background);
-  gtk_spin_button_set_value(box_defaults_dialog->corner_radius, 
-			    default_properties.corner_radius);
-
-  gtk_spin_button_set_value(box_defaults_dialog->padding,
-			    default_properties.padding);
-  attributes_get_default_font(&font, &font_height);
-  dia_font_selector_set_font(box_defaults_dialog->font, font);
-  gtk_spin_button_set_value(box_defaults_dialog->font_size, font_height);
-
-  return box_defaults_dialog->vbox;
-}
-
 static real
 box_distance_from(Box *box, Point *point)
 {
@@ -361,7 +226,7 @@ box_distance_from(Box *box, Point *point)
 
 static void
 box_select(Box *box, Point *clicked_point,
-	   Renderer *interactive_renderer)
+	   DiaRenderer *interactive_renderer)
 {
   real radius;
 
@@ -388,9 +253,10 @@ box_select(Box *box, Point *clicked_point,
   }
 }
 
-static void
+static ObjectChange*
 box_move_handle(Box *box, Handle *handle,
-		Point *to, HandleMoveReason reason, ModifierKeys modifiers)
+		Point *to, ConnectionPoint *cp,
+		HandleMoveReason reason, ModifierKeys modifiers)
 {
   AnchorShape horiz = ANCHOR_MIDDLE, vert = ANCHOR_MIDDLE;
 
@@ -398,7 +264,7 @@ box_move_handle(Box *box, Handle *handle,
   assert(handle!=NULL);
   assert(to!=NULL);
 
-  element_move_handle(&box->element, handle->id, to, reason);
+  element_move_handle(&box->element, handle->id, to, cp, reason, modifiers);
 
   switch (handle->id) {
   case HANDLE_RESIZE_NW:
@@ -421,33 +287,37 @@ box_move_handle(Box *box, Handle *handle,
     break;
   }
   box_update_data(box, horiz, vert);
+
+  return NULL;
 }
 
-static void
+static ObjectChange*
 box_move(Box *box, Point *to)
 {
   box->element.corner = *to;
   
   box_update_data(box, ANCHOR_MIDDLE, ANCHOR_MIDDLE);
+
+  return NULL;
 }
 
 static void
-box_draw(Box *box, Renderer *renderer)
+box_draw(Box *box, DiaRenderer *renderer)
 {
+  DiaRendererClass *renderer_ops = DIA_RENDERER_GET_CLASS (renderer);
   Point lr_corner;
   Element *elem;
   real radius;
   
   assert(box != NULL);
   assert(renderer != NULL);
-
   elem = &box->element;
 
   lr_corner.x = elem->corner.x + elem->width;
   lr_corner.y = elem->corner.y + elem->height;
 
   if (box->show_background) {
-  renderer->ops->set_fillstyle(renderer, FILLSTYLE_SOLID);
+  renderer_ops->set_fillstyle(renderer, FILLSTYLE_SOLID);
   
   /* Problem:  How do we make the fill with rounded corners? */
   if (box->corner_radius > 0) {
@@ -460,14 +330,14 @@ box_draw(Box *box, Renderer *renderer)
     end.x = lr_corner.x-radius;
     start.y = elem->corner.y;
     end.y = lr_corner.y;
-    renderer->ops->fill_rect(renderer, &start, &end, &box->inner_color);
+    renderer_ops->fill_rect(renderer, &start, &end, &box->inner_color);
 
     center.y = elem->corner.y+radius;
-    renderer->ops->fill_arc(renderer, &center, 
+    renderer_ops->fill_arc(renderer, &center, 
 			    2.0*radius, 2.0*radius,
 			    90.0, 180.0, &box->inner_color);
     center.x = end.x;
-    renderer->ops->fill_arc(renderer, &center, 
+    renderer_ops->fill_arc(renderer, &center, 
 			    2.0*radius, 2.0*radius,
 			    0.0, 90.0, &box->inner_color);
 
@@ -475,29 +345,29 @@ box_draw(Box *box, Renderer *renderer)
     start.y = elem->corner.y+radius;
     end.x = lr_corner.x;
     end.y = center.y = lr_corner.y-radius;
-    renderer->ops->fill_rect(renderer, &start, &end, &box->inner_color);
+    renderer_ops->fill_rect(renderer, &start, &end, &box->inner_color);
 
     center.y = lr_corner.y-radius;
     center.x = elem->corner.x+radius;
-    renderer->ops->fill_arc(renderer, &center, 
+    renderer_ops->fill_arc(renderer, &center, 
 			    2.0*radius, 2.0*radius,
 			    180.0, 270.0, &box->inner_color);
     center.x = lr_corner.x-radius;
-    renderer->ops->fill_arc(renderer, &center, 
+    renderer_ops->fill_arc(renderer, &center, 
 			    2.0*radius, 2.0*radius,
 			    270.0, 360.0, &box->inner_color);
   } else {
-  renderer->ops->fill_rect(renderer, 
+  renderer_ops->fill_rect(renderer, 
 			   &elem->corner,
 			   &lr_corner, 
 			   &box->inner_color);
   }
   }
 
-  renderer->ops->set_linewidth(renderer, box->border_width);
-  renderer->ops->set_linestyle(renderer, box->line_style);
-  renderer->ops->set_dashlength(renderer, box->dashlength);
-  renderer->ops->set_linejoin(renderer, LINEJOIN_MITER);
+  renderer_ops->set_linewidth(renderer, box->border_width);
+  renderer_ops->set_linestyle(renderer, box->line_style);
+  renderer_ops->set_dashlength(renderer, box->dashlength);
+  renderer_ops->set_linejoin(renderer, LINEJOIN_MITER);
 
   if (box->corner_radius > 0) {
     Point start, end, center;
@@ -508,37 +378,37 @@ box_draw(Box *box, Renderer *renderer)
     start.x = center.x = elem->corner.x+radius;
     end.x = lr_corner.x-radius;
     start.y = end.y = elem->corner.y;
-    renderer->ops->draw_line(renderer, &start, &end, &box->border_color);
+    renderer_ops->draw_line(renderer, &start, &end, &box->border_color);
     start.y = end.y = lr_corner.y;
-    renderer->ops->draw_line(renderer, &start, &end, &box->border_color);
+    renderer_ops->draw_line(renderer, &start, &end, &box->border_color);
 
     center.y = elem->corner.y+radius;
-    renderer->ops->draw_arc(renderer, &center, 
+    renderer_ops->draw_arc(renderer, &center, 
 			    2.0*radius, 2.0*radius,
 			    90.0, 180.0, &box->border_color);
     center.x = end.x;
-    renderer->ops->draw_arc(renderer, &center, 
+    renderer_ops->draw_arc(renderer, &center, 
 			    2.0*radius, 2.0*radius,
 			    0.0, 90.0, &box->border_color);
 
     start.y = elem->corner.y+radius;
     start.x = end.x = elem->corner.x;
     end.y = center.y = lr_corner.y-radius;
-    renderer->ops->draw_line(renderer, &start, &end, &box->border_color);
+    renderer_ops->draw_line(renderer, &start, &end, &box->border_color);
     start.x = end.x = lr_corner.x;
-    renderer->ops->draw_line(renderer, &start, &end, &box->border_color);
+    renderer_ops->draw_line(renderer, &start, &end, &box->border_color);
 
     center.y = lr_corner.y-radius;
     center.x = elem->corner.x+radius;
-    renderer->ops->draw_arc(renderer, &center, 
+    renderer_ops->draw_arc(renderer, &center, 
 			    2.0*radius, 2.0*radius,
 			    180.0, 270.0, &box->border_color);
     center.x = lr_corner.x-radius;
-    renderer->ops->draw_arc(renderer, &center, 
+    renderer_ops->draw_arc(renderer, &center, 
 			    2.0*radius, 2.0*radius,
 			    270.0, 360.0, &box->border_color);
   } else {
-  renderer->ops->draw_rect(renderer, 
+  renderer_ops->draw_rect(renderer, 
 			   &elem->corner,
 			   &lr_corner, 
 			   &box->border_color);
@@ -569,8 +439,14 @@ box_update_data(Box *box, AnchorShape horiz, AnchorShape vert)
   height = box->text->height * box->text->numlines + box->padding*2 +
     box->border_width;
 
-  elem->width = width;
-  elem->height = height;
+  /*
+   *  If elem->width (e.g. the new requested dimensions of this object
+   *  from move_handle()) is smaller than the minimum width (i.e. the
+   *  width calculated from text-width, padding and border), then
+   *  set the width to the minimum.  Else, keep the width.
+   */
+  if (width > elem->width) elem->width = width;
+  if (height > elem->height) elem->height = height;
 
   /* move shape if necessary ... */
   switch (horiz) {
@@ -593,7 +469,18 @@ box_update_data(Box *box, AnchorShape horiz, AnchorShape vert)
   p = elem->corner;
   p.x += elem->width / 2.0;
   p.y += elem->height / 2.0 - box->text->height * box->text->numlines / 2 +
-    font_ascent(box->text->font, box->text->height);
+    box->text->ascent;
+  switch (box->text->alignment) {
+  case ALIGN_LEFT:
+    p.x -= (elem->width - box->padding*2 + box->border_width)/2;
+    break;
+  case ALIGN_RIGHT:
+    p.x += (elem->width - box->padding*2 + box->border_width)/2;
+    break;
+  case ALIGN_CENTER:
+    break;
+  }
+
   text_set_position(box->text, &p);
 
   radius = box->corner_radius;
@@ -602,38 +489,70 @@ box_update_data(Box *box, AnchorShape horiz, AnchorShape vert)
   radius *= (1-M_SQRT1_2);
   
   /* Update connections: */
-  box->connections[0].pos.x = elem->corner.x + radius;
-  box->connections[0].pos.y = elem->corner.y + radius;
-  box->connections[1].pos.x = elem->corner.x + elem->width / 4.0;
-  box->connections[1].pos.y = elem->corner.y;
-  box->connections[2].pos.x = elem->corner.x + elem->width / 2.0;
-  box->connections[2].pos.y = elem->corner.y;
-  box->connections[3].pos.x = elem->corner.x + elem->width * 3.0 / 4.0;
-  box->connections[3].pos.y = elem->corner.y;
-  box->connections[4].pos.x = elem->corner.x + elem->width - radius;
-  box->connections[4].pos.y = elem->corner.y + radius;
-  box->connections[5].pos.x = elem->corner.x;
-  box->connections[5].pos.y = elem->corner.y + elem->height / 4.0;
-  box->connections[6].pos.x = elem->corner.x + elem->width;
-  box->connections[6].pos.y = elem->corner.y + elem->height / 4.0;
-  box->connections[7].pos.x = elem->corner.x;
-  box->connections[7].pos.y = elem->corner.y + elem->height / 2.0;
-  box->connections[8].pos.x = elem->corner.x + elem->width;
-  box->connections[8].pos.y = elem->corner.y + elem->height / 2.0;
-  box->connections[9].pos.x = elem->corner.x;
-  box->connections[9].pos.y = elem->corner.y + elem->height * 3.0 / 4.0;
-  box->connections[10].pos.x = elem->corner.x + elem->width;
-  box->connections[10].pos.y = elem->corner.y + elem->height * 3.0 / 4.0;
-  box->connections[11].pos.x = elem->corner.x + radius;
-  box->connections[11].pos.y = elem->corner.y + elem->height - radius;
-  box->connections[12].pos.x = elem->corner.x + elem->width / 4.0;
-  box->connections[12].pos.y = elem->corner.y + elem->height;
-  box->connections[13].pos.x = elem->corner.x + elem->width / 2.0;
-  box->connections[13].pos.y = elem->corner.y + elem->height;
-  box->connections[14].pos.x = elem->corner.x + elem->width * 3.0 / 4.0;
-  box->connections[14].pos.y = elem->corner.y + elem->height;
-  box->connections[15].pos.x = elem->corner.x + elem->width - radius;
-  box->connections[15].pos.y = elem->corner.y + elem->height - radius;
+  connpoint_update(&box->connections[0],
+		    elem->corner.x + radius,
+		    elem->corner.y + radius,
+		    DIR_NORTHWEST);
+  connpoint_update(&box->connections[1],
+		    elem->corner.x + elem->width / 4.0,
+		    elem->corner.y,
+		    DIR_NORTH);
+  connpoint_update(&box->connections[2],
+		    elem->corner.x + elem->width / 2.0,
+		    elem->corner.y,
+		    DIR_NORTH);
+  connpoint_update(&box->connections[3],
+		    elem->corner.x + elem->width * 3.0 / 4.0,
+		    elem->corner.y,
+		    DIR_NORTH);
+  connpoint_update(&box->connections[4],
+		    elem->corner.x + elem->width - radius,
+		    elem->corner.y + radius,
+		    DIR_NORTHEAST);
+  connpoint_update(&box->connections[5],
+		    elem->corner.x,
+		    elem->corner.y + elem->height / 4.0,
+		    DIR_WEST);
+  connpoint_update(&box->connections[6],
+		    elem->corner.x + elem->width,
+		    elem->corner.y + elem->height / 4.0,
+		    DIR_EAST);
+  connpoint_update(&box->connections[7],
+		    elem->corner.x,
+		    elem->corner.y + elem->height / 2.0,
+		    DIR_WEST);
+  connpoint_update(&box->connections[8],
+		    elem->corner.x + elem->width,
+		    elem->corner.y + elem->height / 2.0,
+		    DIR_EAST);
+  connpoint_update(&box->connections[9],
+		    elem->corner.x,
+		    elem->corner.y + elem->height * 3.0 / 4.0,
+		    DIR_WEST);
+  connpoint_update(&box->connections[10],
+		    elem->corner.x + elem->width,
+		    elem->corner.y + elem->height * 3.0 / 4.0,
+		    DIR_EAST);
+  connpoint_update(&box->connections[11],
+		    elem->corner.x + radius,
+		    elem->corner.y + elem->height - radius,
+		    DIR_SOUTHWEST);
+  connpoint_update(&box->connections[12],
+		    elem->corner.x + elem->width / 4.0,
+		    elem->corner.y + elem->height,
+		    DIR_SOUTH);
+  connpoint_update(&box->connections[13],
+		    elem->corner.x + elem->width / 2.0,
+		    elem->corner.y + elem->height,
+		    DIR_SOUTH);
+  connpoint_update(&box->connections[14],
+		    elem->corner.x + elem->width * 3.0 / 4.0,
+		    elem->corner.y + elem->height,
+		    DIR_SOUTH);
+  connpoint_update(&box->connections[15],
+		    elem->corner.x + elem->width - radius,
+		    elem->corner.y + elem->height - radius,
+		    DIR_SOUTHEAST);
 
   extra->border_trans = box->border_width / 2.0;
   element_update_boundingbox(elem);
@@ -666,7 +585,7 @@ box_create(Point *startpoint,
   Object *obj;
   Point p;
   int i;
-  DiaFont *font;
+  DiaFont *font = NULL;
   real font_height;
 
   init_default_values();
@@ -681,7 +600,7 @@ box_create(Point *startpoint,
 
   elem->corner = *startpoint;
   elem->width = DEFAULT_WIDTH;
-  elem->height = DEFAULT_WIDTH;
+  elem->height = DEFAULT_HEIGHT;
 
   box->border_width =  attributes_get_default_linewidth();
   box->border_color = attributes_get_foreground();
@@ -697,9 +616,10 @@ box_create(Point *startpoint,
   p.x += elem->width / 2.0;
   p.y += elem->height / 2.0 + font_height / 2;
   box->text = new_text("", font, font_height, &p, &box->border_color,
-		       ALIGN_CENTER);
+                       ALIGN_CENTER);
   text_get_attributes(box->text,&box->attrs);
-
+  dia_font_unref(font);
+  
   element_init(elem, 8, 16);
 
   for (i=0;i<16;i++) {
@@ -769,7 +689,7 @@ box_load(ObjectNode obj_node, int version, const char *filename)
   int i;
   AttributeNode attr;
 
-  box = g_new(Box, 1);
+  box = g_malloc0(sizeof(Box));
   elem = &box->element;
   obj = &elem->object;
   
