@@ -21,14 +21,13 @@
 #endif
 
 #include <assert.h>
-#include <gtk/gtk.h>
 #include <math.h>
 
 #include "intl.h"
 #include "object.h"
 #include "connection.h"
 #include "connectionpoint.h"
-#include "render.h"
+#include "diarenderer.h"
 #include "attributes.h"
 #include "diamenu.h"
 
@@ -39,7 +38,6 @@
 #define DEFAULT_NUMHANDLES 6
 #define HANDLE_BUS (HANDLE_CUSTOM1)
 
-typedef struct _BusPropertiesDialog BusPropertiesDialog;
 
 typedef struct _Bus {
   Connection connection;
@@ -49,15 +47,8 @@ typedef struct _Bus {
   Point *parallel_points;
   Point real_ends[2];
 
-  BusPropertiesDialog *properties_dialog;
 } Bus;
 
-struct _BusPropertiesDialog {
-  GtkWidget *dialog;
-
-  GtkSpinButton *num_handles_spinner;
-};
-
 enum change_type {
   TYPE_ADD_POINT,
   TYPE_REMOVE_POINT
@@ -75,12 +66,13 @@ struct PointChange {
   ConnectionPoint *connected_to; /* NULL if not connected */
 };
 
-static void bus_move_handle(Bus *bus, Handle *handle,
-			    Point *to, HandleMoveReason reason, ModifierKeys modifiers);
-static void bus_move(Bus *bus, Point *to);
+static ObjectChange* bus_move_handle(Bus *bus, Handle *handle,
+				     Point *to, ConnectionPoint *cp,
+				     HandleMoveReason reason, ModifierKeys modifiers);
+static ObjectChange* bus_move(Bus *bus, Point *to);
 static void bus_select(Bus *bus, Point *clicked_point,
-		       Renderer *interactive_renderer);
-static void bus_draw(Bus *bus, Renderer *renderer);
+		       DiaRenderer *interactive_renderer);
+static void bus_draw(Bus *bus, DiaRenderer *renderer);
 static Object *bus_create(Point *startpoint,
 			  void *user_data,
 			  Handle **handle1,
@@ -90,10 +82,12 @@ static void bus_update_data(Bus *bus);
 static void bus_destroy(Bus *bus);
 static Object *bus_copy(Bus *bus);
 
+static PropDescription *bus_describe_props(Bus *bus);
+static void bus_get_props(Bus *bus, GPtrArray *props);
+static void bus_set_props(Bus *bus, GPtrArray *props);
 static void bus_save(Bus *bus, ObjectNode obj_node, const char *filename);
 static Object *bus_load(ObjectNode obj_node, int version,
 			const char *filename);
-static GtkWidget *bus_get_properties(Bus *bus);
 static DiaMenu *bus_get_object_menu(Bus *bus, Point *clickedpoint);
 
 static ObjectChange *
@@ -106,7 +100,9 @@ static ObjectTypeOps bus_type_ops =
 {
   (CreateFunc) bus_create,
   (LoadFunc)   bus_load,
-  (SaveFunc)   bus_save
+  (SaveFunc)   bus_save,
+  (GetDefaultsFunc)   NULL,
+  (ApplyDefaultsFunc) NULL
 };
 
 ObjectType bus_type =
@@ -133,15 +129,45 @@ static ObjectOps bus_ops = {
   (CopyFunc)            bus_copy,
   (MoveFunc)            bus_move,
   (MoveHandleFunc)      bus_move_handle,
-  (GetPropertiesFunc)   bus_get_properties,
-  (ApplyPropertiesFunc) NULL,
-  (ObjectMenuFunc)      bus_get_object_menu
+  (GetPropertiesFunc)   object_create_props_dialog,
+  (ApplyPropertiesFunc) object_apply_props_from_dialog,
+  (ObjectMenuFunc)      bus_get_object_menu,
+  (DescribePropsFunc)   bus_describe_props,
+  (GetPropsFunc)        bus_get_props,
+  (SetPropsFunc)        bus_set_props,
 };
 
-static GtkWidget *
-bus_get_properties(Bus *bus)
+static PropDescription bus_props[] = {
+  OBJECT_COMMON_PROPERTIES,
+  PROP_DESC_END
+};
+
+static PropDescription *
+bus_describe_props(Bus *bus)
 {
-  return NULL;
+  if (bus_props[0].quark == 0)
+    prop_desc_list_calculate_quarks(bus_props);
+  return bus_props;
+}
+
+static PropOffset bus_offsets[] = {
+  OBJECT_COMMON_PROPERTIES_OFFSETS,
+  { NULL, 0, 0 }
+};
+
+static void
+bus_get_props(Bus *bus, GPtrArray *props)
+{
+  object_get_props_from_offsets(&bus->connection.object, bus_offsets,
+				props);
+}
+
+static void
+bus_set_props(Bus *bus, GPtrArray *props)
+{
+  object_set_props_from_offsets(&bus->connection.object, bus_offsets,
+				props);
+  bus_update_data(bus);
 }
 
 static real
@@ -165,14 +191,15 @@ bus_distance_from(Bus *bus, Point *point)
 
 static void
 bus_select(Bus *bus, Point *clicked_point,
-	    Renderer *interactive_renderer)
+	    DiaRenderer *interactive_renderer)
 {
   connection_update_handles(&bus->connection);
 }
 
-static void
+static ObjectChange*
 bus_move_handle(Bus *bus, Handle *handle,
-		Point *to, HandleMoveReason reason, ModifierKeys modifiers)
+		Point *to, ConnectionPoint *cp,
+		HandleMoveReason reason, ModifierKeys modifiers)
 {
   Connection *conn = &bus->connection;
   Point *endpoints;
@@ -216,7 +243,8 @@ bus_move_handle(Bus *bus, Handle *handle,
       perp[i] = point_dot(&vhatperp, &u);
     }
     
-    connection_move_handle(&bus->connection, handle->id, to, reason);
+    connection_move_handle(&bus->connection, handle->id, to, cp, 
+			   reason, modifiers);
 
     vhat = endpoints[1];
     point_sub(&vhat, &endpoints[0]);
@@ -243,9 +271,11 @@ bus_move_handle(Bus *bus, Handle *handle,
   }
 
   bus_update_data(bus);
+
+  return NULL;
 }
 
-static void
+static ObjectChange*
 bus_move(Bus *bus, Point *to)
 {
   Point delta;
@@ -268,11 +298,14 @@ bus_move(Bus *bus, Point *to)
   }
 
   bus_update_data(bus);
+
+  return NULL;
 }
 
 static void
-bus_draw(Bus *bus, Renderer *renderer)
+bus_draw(Bus *bus, DiaRenderer *renderer)
 {
+  DiaRendererClass *renderer_ops = DIA_RENDERER_GET_CLASS (renderer);
   Point *endpoints;
   int i;
   
@@ -281,16 +314,16 @@ bus_draw(Bus *bus, Renderer *renderer)
 
   endpoints = &bus->real_ends[0];
   
-  renderer->ops->set_linewidth(renderer, LINE_WIDTH);
-  renderer->ops->set_linestyle(renderer, LINESTYLE_SOLID);
-  renderer->ops->set_linecaps(renderer, LINECAPS_BUTT);
+  renderer_ops->set_linewidth(renderer, LINE_WIDTH);
+  renderer_ops->set_linestyle(renderer, LINESTYLE_SOLID);
+  renderer_ops->set_linecaps(renderer, LINECAPS_BUTT);
 
-  renderer->ops->draw_line(renderer,
+  renderer_ops->draw_line(renderer,
 			   &endpoints[0], &endpoints[1],
 			   &color_black);
 
   for (i=0;i<bus->num_handles;i++) {
-    renderer->ops->draw_line(renderer,
+    renderer_ops->draw_line(renderer,
 			     &bus->parallel_points[i],
 			     &bus->handles[i]->pos,
 			     &color_black);
@@ -341,8 +374,6 @@ bus_create(Point *startpoint,
     obj->handles[2+i] = bus->handles[i];
   }
 
-  bus->properties_dialog = NULL;
-
   extra->start_trans = 
     extra->end_trans = 
     extra->start_long =
@@ -358,10 +389,6 @@ static void
 bus_destroy(Bus *bus)
 {
   int i;
-  if (bus->properties_dialog != NULL) {
-    gtk_widget_destroy(bus->properties_dialog->dialog);
-    g_free(bus->properties_dialog);
-  }
   connection_destroy(&bus->connection);
   for (i=0;i<bus->num_handles;i++)
     g_free(bus->handles[i]);
@@ -401,8 +428,6 @@ bus_copy(Bus *bus)
   newbus->real_ends[0] = bus->real_ends[0];
   newbus->real_ends[1] = bus->real_ends[1];
 
-  newbus->properties_dialog = NULL;
-
   return &newbus->connection.object;
 }
 
@@ -626,8 +651,6 @@ bus_load(ObjectNode obj_node, int version, const char *filename)
   obj->type = &bus_type;
   obj->ops = &bus_ops;
 
-  bus->properties_dialog = NULL;
-  
   connection_load(conn, obj_node);
 
   attr = object_find_attribute(obj_node, "bus_handles");
