@@ -22,12 +22,12 @@
  */
 
 #include <assert.h>
-#include <gtk/gtk.h>
 #include <math.h>
 
 #include "config.h"
 #include "intl.h"
 #include "connection.h"
+#include "diarenderer.h"
 #include "network.h"
 
 #ifndef M_PI_2
@@ -36,11 +36,6 @@
 
 #include "pixmaps/wanlink.xpm"
 
-typedef struct _WanLinkPropertiesDialog 
-{
-    GtkWidget *dialog;
-} WanLinkPropertiesDialog;
-
 #define WANLINK_POLY_LEN 6
 
 
@@ -49,7 +44,6 @@ typedef struct _WanLink {
 
     real width;
     Point poly[WANLINK_POLY_LEN];
-    WanLinkPropertiesDialog *properties_dialog;
 } WanLink;
 
 static Object *wanlink_create(Point *startpoint,
@@ -57,18 +51,20 @@ static Object *wanlink_create(Point *startpoint,
 			      Handle **handle1,
 			      Handle **handle2);
 static void wanlink_destroy(WanLink *wanlink);
-static void wanlink_draw (WanLink *wanlink, Renderer *renderer);
+static void wanlink_draw (WanLink *wanlink, DiaRenderer *renderer);
 static real wanlink_distance_from(WanLink *wanlink, Point *point);
 static void wanlink_select(WanLink *wanlink, Point *clicked_point,
-			 Renderer *interactive_renderer);
+			 DiaRenderer *interactive_renderer);
 static Object *wanlink_copy(WanLink *wanlink);
-static void wanlink_move(WanLink *wanlink, Point *to);
-static void wanlink_move_handle(WanLink *wanlink, Handle *handle,
-			      Point *to, HandleMoveReason reason, 
+static ObjectChange* wanlink_move(WanLink *wanlink, Point *to);
+static ObjectChange* wanlink_move_handle(WanLink *wanlink, Handle *handle,
+					 Point *to, ConnectionPoint *cp,
+					 HandleMoveReason reason, 
 			      ModifierKeys modifiers);
-static GtkWidget *wanlink_get_properties(WanLink *wanlink);
-
 
+static PropDescription *wanlink_describe_props(WanLink *wanlink);
+static void wanlink_get_props(WanLink *wanlink, GPtrArray *props);
+static void wanlink_set_props(WanLink *wanlink, GPtrArray *props);
 static void wanlink_save(WanLink *wanlink, ObjectNode obj_node,
 			 const char *filename);
 static Object *wanlink_load(ObjectNode obj_node, int version,
@@ -79,7 +75,9 @@ static ObjectTypeOps wanlink_type_ops =
 {
   (CreateFunc) wanlink_create,
   (LoadFunc)   wanlink_load,
-  (SaveFunc)   wanlink_save
+  (SaveFunc)   wanlink_save,
+  (GetDefaultsFunc)   NULL,
+  (ApplyDefaultsFunc) NULL
 };
 
 static ObjectOps wanlink_ops =
@@ -91,9 +89,13 @@ static ObjectOps wanlink_ops =
   (CopyFunc)            wanlink_copy,
   (MoveFunc)            wanlink_move,
   (MoveHandleFunc)      wanlink_move_handle,
-  (GetPropertiesFunc)   wanlink_get_properties,
-  (ApplyPropertiesFunc) NULL,
-  (ObjectMenuFunc)      NULL
+  (GetPropertiesFunc)   object_create_props_dialog,
+  (ApplyPropertiesFunc) object_apply_props_from_dialog,
+  (ObjectMenuFunc)      NULL,
+  (DescribePropsFunc)   wanlink_describe_props,
+  (GetPropsFunc)        wanlink_get_props,
+  (SetPropsFunc)        wanlink_set_props,
+
 };
 
 ObjectType wanlink_type =
@@ -105,6 +107,39 @@ ObjectType wanlink_type =
   &wanlink_type_ops      /* ops */
 };
 
+static PropDescription wanlink_props[] = {
+  OBJECT_COMMON_PROPERTIES,
+  PROP_DESC_END
+};
+
+static PropDescription *
+wanlink_describe_props(WanLink *wanlink)
+{
+  if (wanlink_props[0].quark == 0)
+    prop_desc_list_calculate_quarks(wanlink_props);
+  return wanlink_props;
+}
+
+static PropOffset wanlink_offsets[] = {
+  OBJECT_COMMON_PROPERTIES_OFFSETS,
+  { NULL, 0, 0 }
+};
+
+static void
+wanlink_get_props(WanLink *wanlink, GPtrArray *props)
+{
+  object_get_props_from_offsets(&wanlink->connection.object, wanlink_offsets,
+				props);
+}
+
+static void
+wanlink_set_props(WanLink *wanlink, GPtrArray *props)
+{
+  object_set_props_from_offsets(&wanlink->connection.object, wanlink_offsets,
+				props);
+  wanlink_update_data(wanlink);
+}
+
 #define FLASH_LINE NETWORK_GENERAL_LINEWIDTH
 #define FLASH_WIDTH 1.0
 #define FLASH_HEIGHT 11.0
@@ -112,12 +147,6 @@ ObjectType wanlink_type =
 
 #define FLASH_BOTTOM (FLASH_HEIGHT)
 
-static GtkWidget *
-wanlink_get_properties(WanLink *wanlink)
-{
-  return NULL;
-}
-
 
 static Object *
 wanlink_create(Point *startpoint,
@@ -150,7 +179,6 @@ wanlink_create(Point *startpoint,
       wanlink->poly[i] = defaultpoly;
   
   wanlink->width = FLASH_WIDTH;
-  wanlink->properties_dialog = NULL;
   
   wanlink_update_data(wanlink);
 
@@ -164,28 +192,25 @@ wanlink_create(Point *startpoint,
 static void
 wanlink_destroy(WanLink *wanlink)
 {
-  if (wanlink->properties_dialog != NULL) {
-    gtk_widget_destroy(wanlink->properties_dialog->dialog);
-    g_free(wanlink->properties_dialog);
-  }
   connection_destroy(&wanlink->connection);
 }
 
 
 static void 
-wanlink_draw (WanLink *wanlink, Renderer *renderer)
+wanlink_draw (WanLink *wanlink, DiaRenderer *renderer)
 {
+    DiaRendererClass *renderer_ops = DIA_RENDERER_GET_CLASS (renderer);
         
     assert (wanlink != NULL);
     assert (renderer != NULL);
     
-    renderer->ops->set_linewidth(renderer, FLASH_LINE);
-    renderer->ops->set_linejoin(renderer, LINEJOIN_MITER);
-    renderer->ops->set_linestyle(renderer, LINESTYLE_SOLID);
+    renderer_ops->set_linewidth(renderer, FLASH_LINE);
+    renderer_ops->set_linejoin(renderer, LINEJOIN_MITER);
+    renderer_ops->set_linestyle(renderer, LINESTYLE_SOLID);
     
     
-    renderer->ops->fill_polygon(renderer, wanlink->poly,  WANLINK_POLY_LEN, &color_black);
-    renderer->ops->draw_polygon(renderer, wanlink->poly,  WANLINK_POLY_LEN, &color_black);
+    renderer_ops->fill_polygon(renderer, wanlink->poly,  WANLINK_POLY_LEN, &color_black);
+    renderer_ops->draw_polygon(renderer, wanlink->poly,  WANLINK_POLY_LEN, &color_black);
 }
 
 static real
@@ -203,7 +228,7 @@ wanlink_distance_from(WanLink *wanlink, Point *point)
 
 static void
 wanlink_select(WanLink *wanlink, Point *clicked_point,
-	    Renderer *interactive_renderer)
+	    DiaRenderer *interactive_renderer)
 {
   connection_update_handles(&wanlink->connection);
 }
@@ -224,12 +249,11 @@ wanlink_copy(WanLink *wanlink)
   connection_copy(conn, newconn);
 
   newwanlink->width = wanlink->width;
-  newwanlink->properties_dialog = NULL;
 
   return (Object *)newwanlink;
 }
 
-static void
+static ObjectChange*
 wanlink_move(WanLink *wanlink, Point *to)
 {
   Point delta;
@@ -245,15 +269,21 @@ wanlink_move(WanLink *wanlink, Point *to)
   }
 
   wanlink_update_data(wanlink);
+
+  return NULL;
 }
 
-static void
+static ObjectChange*
 wanlink_move_handle(WanLink *wanlink, Handle *handle,
-		Point *to, HandleMoveReason reason, ModifierKeys modifiers)
+		    Point *to, ConnectionPoint *cp,
+		    HandleMoveReason reason, ModifierKeys modifiers)
 {
-  connection_move_handle(&wanlink->connection, handle->id, to, reason);
+  connection_move_handle(&wanlink->connection, handle->id, to, cp,
+			 reason, modifiers);
   
   wanlink_update_data(wanlink);
+
+  return NULL;
 }
 
 static void
@@ -285,8 +315,6 @@ wanlink_load(ObjectNode obj_node, int version, const char *filename)
     obj->type = &wanlink_type;
     obj->ops = &wanlink_ops;
 
-    wanlink->properties_dialog = NULL;
-
     connection_load(conn, obj_node);
     connection_init(conn, 2, 0);
     
