@@ -21,14 +21,13 @@
 #endif
 
 #include <assert.h>
-#include <gtk/gtk.h>
 #include <math.h>
 
 #include "intl.h"
 #include "object.h"
 #include "element.h"
 #include "connectionpoint.h"
-#include "render.h"
+#include "diarenderer.h"
 #include "attributes.h"
 #include "widgets.h"
 #include "message.h"
@@ -40,9 +39,13 @@
 #define DEFAULT_HEIGHT 1.0
 #define DEFAULT_BORDER 0.25
 
+typedef enum {
+  FREE_ASPECT,
+  FIXED_ASPECT,
+  SQUARE_ASPECT
+} AspectType;
+
 typedef struct _Box Box;
-typedef struct _BoxProperties BoxProperties;
-typedef struct _BoxDefaultsDialog BoxDefaultsDialog;
 
 struct _Box {
   Element element;
@@ -56,37 +59,24 @@ struct _Box {
   LineStyle line_style;
   real dashlength;
   real corner_radius;
+  AspectType aspect;
 };
 
-struct _BoxProperties {
-  Color *fg_color;
-  Color *bg_color;
+static struct _BoxProperties {
   gboolean show_background;
-  real border_width;
-  LineStyle line_style;
-  real dashlength;
   real corner_radius;
-};
-
-struct _BoxDefaultsDialog {
-  GtkWidget *vbox;
-
-  GtkToggleButton *show_background;
-  GtkSpinButton *corner_radius;
-};
-
-
-static BoxDefaultsDialog *box_defaults_dialog;
-static BoxProperties default_properties;
+  AspectType aspect;
+} default_properties = { TRUE, 0.0 };
 
 static real box_distance_from(Box *box, Point *point);
 static void box_select(Box *box, Point *clicked_point,
-		       Renderer *interactive_renderer);
-static void box_move_handle(Box *box, Handle *handle,
-			    Point *to, HandleMoveReason reason, 
+		       DiaRenderer *interactive_renderer);
+static ObjectChange* box_move_handle(Box *box, Handle *handle,
+			    Point *to, ConnectionPoint *cp,
+				     HandleMoveReason reason, 
 			    ModifierKeys modifiers);
-static void box_move(Box *box, Point *to);
-static void box_draw(Box *box, Renderer *renderer);
+static ObjectChange* box_move(Box *box, Point *to);
+static void box_draw(Box *box, DiaRenderer *renderer);
 static void box_update_data(Box *box);
 static Object *box_create(Point *startpoint,
 			  void *user_data,
@@ -101,16 +91,15 @@ static void box_set_props(Box *box, GPtrArray *props);
 
 static void box_save(Box *box, ObjectNode obj_node, const char *filename);
 static Object *box_load(ObjectNode obj_node, int version, const char *filename);
-static GtkWidget *box_get_defaults(void);
-static void box_apply_defaults(void);
+static DiaMenu *box_get_object_menu(Box *box, Point *clickedpoint);
 
 static ObjectTypeOps box_type_ops =
 {
   (CreateFunc) box_create,
   (LoadFunc)   box_load,
   (SaveFunc)   box_save,
-  (GetDefaultsFunc)   box_get_defaults,
-  (ApplyDefaultsFunc) box_apply_defaults
+  (GetDefaultsFunc)   NULL,
+  (ApplyDefaultsFunc) NULL
 };
 
 ObjectType box_type =
@@ -134,7 +123,7 @@ static ObjectOps box_ops = {
   (MoveHandleFunc)      box_move_handle,
   (GetPropertiesFunc)   object_create_props_dialog,
   (ApplyPropertiesFunc) object_apply_props_from_dialog,
-  (ObjectMenuFunc)      NULL,
+  (ObjectMenuFunc)      box_get_object_menu,
   (DescribePropsFunc)   box_describe_props,
   (GetPropsFunc)        box_get_props,
   (SetPropsFunc)        box_set_props,
@@ -142,6 +131,12 @@ static ObjectOps box_ops = {
 
 static PropNumData corner_radius_data = { 0.0, 10.0, 0.1 };
 
+static PropEnumData prop_aspect_data[] = {
+  { N_("Free"), FREE_ASPECT },
+  { N_("Fixed"), FIXED_ASPECT },
+  { N_("Square"), SQUARE_ASPECT },
+  { NULL, 0 }
+};
 static PropDescription box_props[] = {
   ELEMENT_COMMON_PROPERTIES,
   PROP_STD_LINE_WIDTH,
@@ -151,6 +146,8 @@ static PropDescription box_props[] = {
   PROP_STD_LINE_STYLE,
   { "corner_radius", PROP_TYPE_REAL, PROP_FLAG_VISIBLE,
     N_("Corner radius"), NULL, &corner_radius_data },
+  { "aspect", PROP_TYPE_ENUM, PROP_FLAG_VISIBLE,
+    N_("Aspect ratio"), NULL, prop_aspect_data },
   PROP_DESC_END
 };
 
@@ -168,6 +165,7 @@ static PropOffset box_offsets[] = {
   { "line_colour", PROP_TYPE_COLOUR, offsetof(Box, border_color) },
   { "fill_colour", PROP_TYPE_COLOUR, offsetof(Box, inner_color) },
   { "show_background", PROP_TYPE_BOOL, offsetof(Box, show_background) },
+  { "aspect", PROP_TYPE_ENUM, offsetof(Box, aspect) },
   { "line_style", PROP_TYPE_LINESTYLE,
     offsetof(Box, line_style), offsetof(Box, dashlength) },
   { "corner_radius", PROP_TYPE_REAL, offsetof(Box, corner_radius) },
@@ -189,78 +187,6 @@ box_set_props(Box *box, GPtrArray *props)
   box_update_data(box);
 }
 
-static void
-box_apply_defaults()
-{
-  default_properties.corner_radius = gtk_spin_button_get_value_as_float(box_defaults_dialog->corner_radius);
-  default_properties.show_background = gtk_toggle_button_get_active(box_defaults_dialog->show_background);
-}
-
-static void
-init_default_values() {
-  static int defaults_initialized = 0;
-
-  if (!defaults_initialized) {
-    default_properties.show_background = 1;
-    defaults_initialized = 1;
-  }
-}
-
-static GtkWidget *
-box_get_defaults()
-{
-  GtkWidget *vbox;
-  GtkWidget *hbox;
-  GtkWidget *label;
-  GtkWidget *checkbox;
-  GtkWidget *corner_radius;
-  GtkAdjustment *adj;
-
-  if (box_defaults_dialog == NULL) {
-  
-    init_default_values();
-
-    box_defaults_dialog = g_new(BoxDefaultsDialog, 1);
-
-    vbox = gtk_vbox_new(FALSE, 5);
-    box_defaults_dialog->vbox = vbox;
-
-    gtk_object_ref(GTK_OBJECT(vbox));
-    gtk_object_sink(GTK_OBJECT(vbox));
-
-    hbox = gtk_hbox_new(FALSE, 5);
-    checkbox = gtk_check_button_new_with_label(_("Draw background"));
-    box_defaults_dialog->show_background = GTK_TOGGLE_BUTTON( checkbox );
-    gtk_widget_show(checkbox);
-    gtk_widget_show(hbox);
-    gtk_box_pack_start (GTK_BOX (hbox), checkbox, TRUE, TRUE, 0);
-    gtk_box_pack_start (GTK_BOX (vbox), hbox, TRUE, TRUE, 0);
-
-    hbox = gtk_hbox_new(FALSE, 5);
-    label = gtk_label_new(_("Corner rounding:"));
-    gtk_box_pack_start (GTK_BOX (hbox), label, FALSE, TRUE, 0);
-    gtk_widget_show (label);
-    adj = (GtkAdjustment *) gtk_adjustment_new(0.1, 0.0, 10.0, 0.1, 1.0, 1.0);
-    corner_radius = gtk_spin_button_new(adj, 1.0, 2);
-    gtk_spin_button_set_wrap(GTK_SPIN_BUTTON(corner_radius), TRUE);
-    gtk_spin_button_set_numeric(GTK_SPIN_BUTTON(corner_radius), TRUE);
-    box_defaults_dialog->corner_radius = GTK_SPIN_BUTTON(corner_radius);
-    gtk_box_pack_start(GTK_BOX (hbox), corner_radius, TRUE, TRUE, 0);
-    gtk_widget_show (corner_radius);
-    gtk_widget_show(hbox);
-    gtk_box_pack_start (GTK_BOX(vbox), hbox, TRUE, TRUE, 0);
-    gtk_widget_show (vbox);
-    gtk_widget_show (vbox);
-  }
-
-  gtk_toggle_button_set_active(box_defaults_dialog->show_background, 
-			       default_properties.show_background);
-  gtk_spin_button_set_value(box_defaults_dialog->corner_radius, 
-			    default_properties.corner_radius);
-
-  return box_defaults_dialog->vbox;
-}
-
 static real
 box_distance_from(Box *box, Point *point)
 {
@@ -276,7 +202,7 @@ box_distance_from(Box *box, Point *point)
 
 static void
 box_select(Box *box, Point *clicked_point,
-	   Renderer *interactive_renderer)
+	   DiaRenderer *interactive_renderer)
 {
   real radius;
 
@@ -300,33 +226,80 @@ box_select(Box *box, Point *clicked_point,
   }
 }
 
-static void
+static ObjectChange*
 box_move_handle(Box *box, Handle *handle,
-		Point *to, HandleMoveReason reason, ModifierKeys modifiers)
+		Point *to, ConnectionPoint *cp,
+		HandleMoveReason reason, ModifierKeys modifiers)
 {
   assert(box!=NULL);
   assert(handle!=NULL);
   assert(to!=NULL);
 
-  element_move_handle(&box->element, handle->id, to, reason);
+  if (box->aspect != FREE_ASPECT){
+    double width, height;
+    double new_width, new_height;
+    double to_width, aspect_width;
+    Point corner = box->element.corner;
+    Point nw_to, se_to;
+
+    width = box->element.width;
+    height = box->element.height;
+    switch (handle->id) {
+    case HANDLE_RESIZE_N:
+    case HANDLE_RESIZE_S:
+      new_height = fabs(to->y - corner.y);
+      new_width = new_height / height * width;
+      break;
+    case HANDLE_RESIZE_W:
+    case HANDLE_RESIZE_E:
+      new_width = fabs(to->x - corner.x);
+      new_height = new_width / width * height;
+      break;
+    case HANDLE_RESIZE_NW:
+    case HANDLE_RESIZE_NE:
+    case HANDLE_RESIZE_SW:
+    case HANDLE_RESIZE_SE:
+      to_width = fabs(to->x - corner.x);
+      aspect_width = fabs(to->y - corner.y) / height * width;
+      new_width = to_width > aspect_width ? to_width : aspect_width;
+      new_height = new_width / width * height;
+      break;
+    default: 
+      new_width = width;
+      new_height = height;
+      break;
+    }
+	
+    se_to.x = corner.x + new_width;
+    se_to.y = corner.y + new_height;
+        
+    element_move_handle(&box->element, HANDLE_RESIZE_SE, &se_to, cp, reason, modifiers);
+  } else {
+    element_move_handle(&box->element, handle->id, to, cp, reason, modifiers);
+  }
 
   box_update_data(box);
+
+  return NULL;
 }
 
-static void
+static ObjectChange*
 box_move(Box *box, Point *to)
 {
   box->element.corner = *to;
   
   box_update_data(box);
+
+  return NULL;
 }
 
 static void
-box_draw(Box *box, Renderer *renderer)
+box_draw(Box *box, DiaRenderer *renderer)
 {
   Point lr_corner;
   Element *elem;
-  real radius;
+  DiaRendererClass *renderer_ops = DIA_RENDERER_GET_CLASS (renderer);
+
   
   assert(box != NULL);
   assert(renderer != NULL);
@@ -337,114 +310,36 @@ box_draw(Box *box, Renderer *renderer)
   lr_corner.y = elem->corner.y + elem->height;
 
   if (box->show_background) {
-    renderer->ops->set_fillstyle(renderer, FILLSTYLE_SOLID);
+    renderer_ops->set_fillstyle(renderer, FILLSTYLE_SOLID);
   
     /* Problem:  How do we make the fill with rounded corners? */
     if (box->corner_radius > 0) {
-      /* Not including until people have had a chance to change any renderers*/
-#ifdef NEW_RENDER_OPS
-      renderer->ops->fill_rounded_rect(renderer,
+      renderer_ops->fill_rounded_rect(renderer,
 				       &elem->corner,
 				       &lr_corner,
 				       &box->inner_color,
 				       box->corner_radius);
-#else
-	Point start, end, center;
-
-	radius = box->corner_radius;
-	radius = MIN(radius, elem->width/2);
-	radius = MIN(radius, elem->height/2);
-	start.x = center.x = elem->corner.x+radius;
-	end.x = lr_corner.x-radius;
-	start.y = elem->corner.y;
-	end.y = lr_corner.y;
-	renderer->ops->fill_rect(renderer, &start, &end, &box->inner_color);
-
-	center.y = elem->corner.y+radius;
-	renderer->ops->fill_arc(renderer, &center, 
-	2.0*radius, 2.0*radius,
-	90.0, 180.0, &box->inner_color);
-	center.x = end.x;
-	renderer->ops->fill_arc(renderer, &center, 
-	2.0*radius, 2.0*radius,
-	0.0, 90.0, &box->inner_color);
-
-	start.x = elem->corner.x;
-	start.y = elem->corner.y+radius;
-	end.x = lr_corner.x;
-	end.y = center.y = lr_corner.y-radius;
-	renderer->ops->fill_rect(renderer, &start, &end, &box->inner_color);
-
-	center.y = lr_corner.y-radius;
-	center.x = elem->corner.x+radius;
-	renderer->ops->fill_arc(renderer, &center, 
-	2.0*radius, 2.0*radius,
-	180.0, 270.0, &box->inner_color);
-	center.x = lr_corner.x-radius;
-	renderer->ops->fill_arc(renderer, &center, 
-	2.0*radius, 2.0*radius,
-	270.0, 360.0, &box->inner_color);
-#endif
     } else {
-      renderer->ops->fill_rect(renderer, 
+      renderer_ops->fill_rect(renderer, 
 			       &elem->corner,
 			       &lr_corner, 
 			       &box->inner_color);
     }
   }
 
-  renderer->ops->set_linewidth(renderer, box->border_width);
-  renderer->ops->set_linestyle(renderer, box->line_style);
-  renderer->ops->set_dashlength(renderer, box->dashlength);
-  renderer->ops->set_linejoin(renderer, LINEJOIN_MITER);
+  renderer_ops->set_linewidth(renderer, box->border_width);
+  renderer_ops->set_linestyle(renderer, box->line_style);
+  renderer_ops->set_dashlength(renderer, box->dashlength);
+  renderer_ops->set_linejoin(renderer, LINEJOIN_MITER);
 
   if (box->corner_radius > 0) {
-    renderer->ops->draw_rounded_rect(renderer, 
+    renderer_ops->draw_rounded_rect(renderer, 
 			     &elem->corner,
 			     &lr_corner, 
 			     &box->border_color,
 			     box->corner_radius);
-    /*
-      Point start, end, center;
-
-      radius = box->corner_radius;
-      radius = MIN(radius, elem->width/2);
-      radius = MIN(radius, elem->height/2);
-      start.x = center.x = elem->corner.x+radius;
-      end.x = lr_corner.x-radius;
-      start.y = end.y = elem->corner.y;
-      renderer->ops->draw_line(renderer, &start, &end, &box->border_color);
-      start.y = end.y = lr_corner.y;
-      renderer->ops->draw_line(renderer, &start, &end, &box->border_color);
-
-      center.y = elem->corner.y+radius;
-      renderer->ops->draw_arc(renderer, &center, 
-      2.0*radius, 2.0*radius,
-      90.0, 180.0, &box->border_color);
-      center.x = end.x;
-      renderer->ops->draw_arc(renderer, &center, 
-      2.0*radius, 2.0*radius,
-      0.0, 90.0, &box->border_color);
-
-      start.y = elem->corner.y+radius;
-      start.x = end.x = elem->corner.x;
-      end.y = center.y = lr_corner.y-radius;
-      renderer->ops->draw_line(renderer, &start, &end, &box->border_color);
-      start.x = end.x = lr_corner.x;
-      renderer->ops->draw_line(renderer, &start, &end, &box->border_color);
-
-      center.y = lr_corner.y-radius;
-      center.x = elem->corner.x+radius;
-      renderer->ops->draw_arc(renderer, &center, 
-      2.0*radius, 2.0*radius,
-      180.0, 270.0, &box->border_color);
-      center.x = lr_corner.x-radius;
-      renderer->ops->draw_arc(renderer, &center, 
-      2.0*radius, 2.0*radius,
-      270.0, 360.0, &box->border_color);
-    */
   } else {
-    renderer->ops->draw_rect(renderer, 
+    renderer_ops->draw_rect(renderer, 
 			     &elem->corner,
 			     &lr_corner, 
 			     &box->border_color);
@@ -460,6 +355,11 @@ box_update_data(Box *box)
   Object *obj = &elem->object;
   real radius;
   
+  if (box->aspect == SQUARE_ASPECT){
+    float size = elem->height < elem->width ? elem->height : elem->width;
+    elem->height = elem->width = size;
+  }
+
   radius = box->corner_radius;
   radius = MIN(radius, elem->width/2);
   radius = MIN(radius, elem->height/2);
@@ -483,6 +383,15 @@ box_update_data(Box *box)
   box->connections[7].pos.x = elem->corner.x + elem->width - radius;
   box->connections[7].pos.y = elem->corner.y + elem->height - radius;
 
+  box->connections[0].directions = DIR_NORTH|DIR_WEST;
+  box->connections[1].directions = DIR_NORTH;
+  box->connections[2].directions = DIR_NORTH|DIR_EAST;
+  box->connections[3].directions = DIR_WEST;
+  box->connections[4].directions = DIR_EAST;
+  box->connections[5].directions = DIR_SOUTH|DIR_WEST;
+  box->connections[6].directions = DIR_SOUTH;
+  box->connections[7].directions = DIR_SOUTH|DIR_EAST;
+
   extra->border_trans = box->border_width / 2.0;
   element_update_boundingbox(elem);
   
@@ -514,8 +423,6 @@ box_create(Point *startpoint,
   Object *obj;
   int i;
 
-  init_default_values();
-
   box = g_malloc0(sizeof(Box));
   elem = &box->element;
   obj = &elem->object;
@@ -524,17 +431,20 @@ box_create(Point *startpoint,
 
   obj->ops = &box_ops;
 
+
   elem->corner = *startpoint;
   elem->width = DEFAULT_WIDTH;
-  elem->height = DEFAULT_WIDTH;
+  elem->height = DEFAULT_HEIGHT;
 
   box->border_width =  attributes_get_default_linewidth();
   box->border_color = attributes_get_foreground();
   box->inner_color = attributes_get_background();
-  box->show_background = default_properties.show_background;
   attributes_get_default_line_style(&box->line_style, &box->dashlength);
+  /* For non-default objects, this is overridden by the default */
+  box->show_background = default_properties.show_background;
   box->corner_radius = default_properties.corner_radius;
-  
+  box->aspect = default_properties.aspect;
+
   element_init(elem, 8, 8);
 
   for (i=0;i<8;i++) {
@@ -579,6 +489,7 @@ box_copy(Box *box)
   newbox->line_style = box->line_style;
   newbox->dashlength = box->dashlength;
   newbox->corner_radius = box->corner_radius;
+  newbox->aspect = box->aspect;
   
   for (i=0;i<8;i++) {
     newobj->connections[i] = &newbox->connections[i];
@@ -622,6 +533,10 @@ box_save(Box *box, ObjectNode obj_node, const char *filename)
   if (box->corner_radius > 0.0)
     data_add_real(new_attribute(obj_node, "corner_radius"),
 		  box->corner_radius);
+
+  if (box->aspect != FREE_ASPECT)
+    data_add_enum(new_attribute(obj_node, "aspect"),
+		  box->aspect);
 }
 
 static Object *
@@ -677,6 +592,11 @@ box_load(ObjectNode obj_node, int version, const char *filename)
   if (attr != NULL)
     box->corner_radius =  data_real( attribute_first_data(attr) );
 
+  box->aspect = FREE_ASPECT;
+  attr = object_find_attribute(obj_node, "aspect");
+  if (attr != NULL)
+    box->aspect = data_enum(attribute_first_data(attr));
+
   element_init(elem, 8, 8);
 
   for (i=0;i<8;i++) {
@@ -689,3 +609,102 @@ box_load(ObjectNode obj_node, int version, const char *filename)
 
   return &box->element.object;
 }
+
+
+struct AspectChange {
+  ObjectChange obj_change;
+  AspectType old_type, new_type;
+  /* The points before this got applied.  Afterwards, all points can be
+   * calculated.
+   */
+  Point topleft;
+  real width, height;
+};
+
+static void
+aspect_change_free(struct AspectChange *change)
+{
+}
+
+static void
+aspect_change_apply(struct AspectChange *change, Object *obj)
+{
+  Box *box = (Box*)obj;
+
+  box->aspect = change->new_type;
+  box_update_data(box);
+}
+
+static void
+aspect_change_revert(struct AspectChange *change, Object *obj)
+{
+  Box *box = (Box*)obj;
+
+  box->aspect = change->old_type;
+  box->element.corner = change->topleft;
+  box->element.width = change->width;
+  box->element.height = change->height;
+  box_update_data(box);
+}
+
+static ObjectChange *
+aspect_create_change(Box *box, AspectType aspect)
+{
+  struct AspectChange *change;
+
+  change = g_new(struct AspectChange, 1);
+
+  change->obj_change.apply = (ObjectChangeApplyFunc) aspect_change_apply;
+  change->obj_change.revert = (ObjectChangeRevertFunc) aspect_change_revert;
+  change->obj_change.free = (ObjectChangeFreeFunc) aspect_change_free;
+
+  change->old_type = box->aspect;
+  change->new_type = aspect;
+  change->topleft = box->element.corner;
+  change->width = box->element.width;
+  change->height = box->element.height;
+
+  return (ObjectChange *)change;
+}
+
+
+static ObjectChange *
+box_set_aspect_callback (Object* obj, Point* clicked, gpointer data)
+{
+  ObjectChange *change;
+
+  change = aspect_create_change((Box*)obj, (AspectType)data);
+  change->apply(change, obj);
+
+  return change;
+}
+
+static DiaMenuItem box_menu_items[] = {
+  { N_("Free aspect"), box_set_aspect_callback, (void*)FREE_ASPECT, 
+    DIAMENU_ACTIVE|DIAMENU_TOGGLE },
+  { N_("Fixed aspect"), box_set_aspect_callback, (void*)FIXED_ASPECT, 
+    DIAMENU_ACTIVE|DIAMENU_TOGGLE },
+  { N_("Square"), box_set_aspect_callback, (void*)SQUARE_ASPECT, 
+    DIAMENU_ACTIVE|DIAMENU_TOGGLE},
+};
+
+static DiaMenu box_menu = {
+  "Box",
+  sizeof(box_menu_items)/sizeof(DiaMenuItem),
+  box_menu_items,
+  NULL
+};
+
+static DiaMenu *
+box_get_object_menu(Box *box, Point *clickedpoint)
+{
+  /* Set entries sensitive/selected etc here */
+  box_menu_items[0].active = DIAMENU_ACTIVE|DIAMENU_TOGGLE;
+  box_menu_items[1].active = DIAMENU_ACTIVE|DIAMENU_TOGGLE;
+  box_menu_items[2].active = DIAMENU_ACTIVE|DIAMENU_TOGGLE;
+
+  box_menu_items[box->aspect].active = 
+    DIAMENU_ACTIVE|DIAMENU_TOGGLE|DIAMENU_TOGGLE_ON;
+  
+  return &box_menu;
+}
