@@ -1,5 +1,6 @@
 /* Dia -- an diagram creation/manipulation program
  * Copyright (C) 1998 Alexander Larsson
+ * Copyright (C) 2002 David Hoover
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -21,14 +22,13 @@
 #endif
 
 #include <assert.h>
-#include <gtk/gtk.h>
 #include <math.h>
 
 #include "intl.h"
 #include "object.h"
 #include "connection.h"
 #include "connectionpoint.h"
-#include "render.h"
+#include "diarenderer.h"
 #include "attributes.h"
 #include "widgets.h"
 #include "arrows.h"
@@ -39,6 +39,8 @@
 
 #define DEFAULT_WIDTH 0.25
 
+typedef struct _LineProperties LineProperties;
+
 typedef struct _Line {
   Connection connection;
 
@@ -49,15 +51,27 @@ typedef struct _Line {
   LineStyle line_style;  
   Arrow start_arrow, end_arrow;
   real dashlength;
+  real absolute_start_gap, absolute_end_gap;
+  real fractional_start_gap, fractional_end_gap;
+  gboolean object_edge_start, object_edge_end;
 } Line;
 
-static void line_move_handle(Line *line, Handle *handle,
-			     Point *to, HandleMoveReason reason, 
+struct _LineProperties {
+  real absolute_start_gap, absolute_end_gap;
+  real fractional_start_gap, fractional_end_gap;
+  gboolean object_edge_start, object_edge_end;
+};
+
+static LineProperties default_properties;
+
+static ObjectChange* line_move_handle(Line *line, Handle *handle,
+				      Point *to, ConnectionPoint *cp,
+				      HandleMoveReason reason, 
 			     ModifierKeys modifiers);
-static void line_move(Line *line, Point *to);
+static ObjectChange* line_move(Line *line, Point *to);
 static void line_select(Line *line, Point *clicked_point,
-			Renderer *interactive_renderer);
-static void line_draw(Line *line, Renderer *renderer);
+			DiaRenderer *interactive_renderer);
+static void line_draw(Line *line, DiaRenderer *renderer);
 static Object *line_create(Point *startpoint,
 			   void *user_data,
 			   Handle **handle1,
@@ -75,13 +89,15 @@ static void line_save(Line *line, ObjectNode obj_node, const char *filename);
 static Object *line_load(ObjectNode obj_node, int version, const char *filename);
 static DiaMenu *line_get_object_menu(Line *line, Point *clickedpoint);
 
+void calculate_gap_endpoints(Line *line, Point *gap_endpoints);
+
 static ObjectTypeOps line_type_ops =
 {
   (CreateFunc) line_create,
   (LoadFunc)   line_load,
   (SaveFunc)   line_save,
-  (GetDefaultsFunc)   NULL /*line_get_defaults*/,
-  (ApplyDefaultsFunc) NULL /*line_apply_defaults*/
+  (GetDefaultsFunc)   NULL,
+  (ApplyDefaultsFunc) NULL
 };
 
 ObjectType line_type =
@@ -110,17 +126,37 @@ static ObjectOps line_ops = {
   (SetPropsFunc)        line_set_props
 };
 
+static PropNumData gap_range = { -G_MAXFLOAT, G_MAXFLOAT, 0.1};
+
 static PropDescription line_props[] = {
   OBJECT_COMMON_PROPERTIES,
   PROP_STD_LINE_WIDTH,
   PROP_STD_LINE_COLOUR,
   PROP_STD_LINE_STYLE,
+  PROP_FRAME_BEGIN("arrows",PROP_FLAG_STANDARD,N_("Arrows")),
   PROP_STD_START_ARROW,
   PROP_STD_END_ARROW,
+  PROP_FRAME_END("arrows",PROP_FLAG_STANDARD),
   { "start_point", PROP_TYPE_POINT, 0,
     N_("Start point"), NULL },
   { "end_point", PROP_TYPE_POINT, 0,
     N_("End point"), NULL },
+  /*
+  PROP_FRAME_BEGIN("gaps",0,N_("Line gaps")),
+  { "absolute_start_gap", PROP_TYPE_REAL, 0,
+    N_("Absolute start gap"), NULL, &gap_range },
+  { "absolute_end_gap", PROP_TYPE_REAL, 0,
+    N_("Absolute end gap"), NULL, &gap_range },
+  { "fractional_start_gap", PROP_TYPE_REAL, 0,
+    N_("Fractional start gap"), NULL, &gap_range },
+  { "fractional_end_gap", PROP_TYPE_REAL, 0,
+    N_("Fractional end gap"), NULL, &gap_range },
+  { "object_edge_start", PROP_TYPE_BOOL, 0,
+    N_("Start at object edge"), },
+  { "object_edge_end", PROP_TYPE_BOOL, 0,
+    N_("End at object edge"), },
+  PROP_FRAME_END("gaps",0),
+  */
   PROP_DESC_END
 };
 
@@ -142,6 +178,14 @@ static PropOffset line_offsets[] = {
   { "end_arrow", PROP_TYPE_ARROW, offsetof(Line, end_arrow) },
   { "start_point", PROP_TYPE_POINT, offsetof(Connection, endpoints[0]) },
   { "end_point", PROP_TYPE_POINT, offsetof(Connection, endpoints[1]) },
+  /*
+  { "absolute_start_gap", PROP_TYPE_REAL, offsetof(Line, absolute_start_gap) },
+  { "absolute_end_gap", PROP_TYPE_REAL, offsetof(Line, absolute_end_gap) },
+  { "fractional_start_gap", PROP_TYPE_REAL, offsetof(Line, fractional_start_gap) },
+  { "fractional_end_gap", PROP_TYPE_REAL, offsetof(Line, fractional_end_gap) },
+  { "object_edge_start", PROP_TYPE_BOOL, offsetof(Line, object_edge_start) },
+  { "object_edge_end", PROP_TYPE_BOOL, offsetof(Line, object_edge_end) },
+  */
   { NULL, 0, 0 }
 };
 
@@ -160,6 +204,21 @@ line_set_props(Line *line, GPtrArray *props)
   line_update_data(line);
 }
 
+static void
+line_init_defaults() {
+  static int defaults_initialized = 0;
+
+  if (!defaults_initialized) {
+    default_properties.absolute_start_gap = 0.0;
+    default_properties.absolute_end_gap = 0.0;
+    default_properties.fractional_start_gap = 0;
+    default_properties.fractional_end_gap = 0;
+    default_properties.object_edge_start = FALSE;
+    default_properties.object_edge_end = FALSE;
+    defaults_initialized = 1;
+  }
+}
+
 static ObjectChange *
 line_add_connpoint_callback(Object *obj, Point *clicked, gpointer data) 
 {
@@ -205,37 +264,151 @@ line_get_object_menu(Line *line, Point *clickedpoint)
   return &object_menu;
 }
 
+/*
+  About start_gap, end_gap, start_fractional_gap, end_fractional_gap
+  
+  Place positive reals (try 1.0, for instance) to create gaps on line
+  end/start.  If fractional_gap is false, these gaps are of fixed
+  length.  If fractional_gap is true, these gaps are a fraction of
+  total line length.  I don't know if fractional gaps are really
+  useful, but since it doesn't complicate the code, why not?
+  If a gap are negative, the line extends past the handle.
+  
+  Gaps as implemented for now do not use any object intersection
+  code.  It would be nice to have gap code that combined a fixed gap
+  length which could be optionally added to the object intersection
+  gap.
+*/
+Point
+calculate_object_edge(Point *objmid, Point *end, Object *obj) 
+{
+#define MAXITER 25
+#ifdef TRACE_DIST
+  Point trace[MAXITER];
+  real disttrace[MAXITER];
+#endif
+  Point mid1, mid2, mid3;
+  real dist;
+  int i = 0;
+
+  mid1 = *objmid;
+  mid2.x = (objmid->x+end->x)/2;
+  mid2.y = (objmid->y+end->y)/2;
+  mid3 = *end;
+
+  /* If the other end is inside the object */
+  dist = obj->ops->distance_from(obj, &mid3);
+  if (dist < 0.001) return mid1;
+
+
+  do {
+    dist = obj->ops->distance_from(obj, &mid2);
+    if (dist < 0.0000001) {
+      mid1 = mid2;
+    } else {
+      mid3 = mid2;
+    }
+    mid2.x = (mid1.x + mid3.x)/2;
+    mid2.y = (mid1.y + mid3.y)/2;
+#ifdef TRACE_DIST
+    trace[i] = mid2;
+    disttrace[i] = dist;
+#endif
+    i++;
+  } while (i < MAXITER && (dist < 0.0000001 || dist > 0.001));
+  
+#ifdef TRACE_DIST
+  if (i == MAXITER) {
+    for (i = 0; i < MAXITER; i++) {
+      printf("%d: %f, %f: %f\n", i, trace[i].x, trace[i].y, disttrace[i]);
+    }
+    printf("i = %d, dist = %f\n", i, dist);
+  }
+#endif
+
+  return mid2;
+}
+
+void
+calculate_gap_endpoints(Line *line, Point *gap_endpoints)
+{
+  Point endpoints[2];
+  real line_length;
+
+  endpoints[0] = line->connection.endpoints[0];
+  endpoints[1] = line->connection.endpoints[1];
+
+  if (line->object_edge_start &&
+      line->connection.endpoint_handles[0].connected_to != NULL) {
+      endpoints[0] = calculate_object_edge(&line->connection.endpoints[0],
+					   &line->connection.endpoints[1],
+					   line->connection.endpoint_handles[0].connected_to->object);
+  }
+  if (line->object_edge_end &&
+      line->connection.endpoint_handles[1].connected_to != NULL) {
+      endpoints[1] = calculate_object_edge(&line->connection.endpoints[1],
+					   &line->connection.endpoints[0],
+					   line->connection.endpoint_handles[1].connected_to->object);
+  }
+
+  line_length = distance_point_point(&endpoints[0],&endpoints[1]);
+
+  point_convex(&gap_endpoints[0],&endpoints[1],&endpoints[0],
+              line->absolute_start_gap/line_length +
+              line->fractional_start_gap
+              );
+
+  point_convex(&gap_endpoints[1],&endpoints[0],&endpoints[1],
+              line->absolute_end_gap/line_length +
+              line->fractional_end_gap
+              );
+}
+
 static real
 line_distance_from(Line *line, Point *point)
 {
   Point *endpoints;
 
   endpoints = &line->connection.endpoints[0]; 
-  return distance_line_point( &endpoints[0], &endpoints[1],
-			      line->line_width, point);
+
+  if (line->object_edge_start || line->object_edge_end ||
+      line->absolute_start_gap || line->absolute_end_gap ||
+      line->fractional_start_gap || line->fractional_end_gap) {
+    Point gap_endpoints[2];  /* Visible endpoints of line */
+
+    calculate_gap_endpoints(line, gap_endpoints);
+    return distance_line_point( &gap_endpoints[0], &gap_endpoints[1],
+                               line->line_width, point);
+  } else {
+    return distance_line_point( &endpoints[0], &endpoints[1],
+                               line->line_width, point);
+  }
 }
 
 static void
 line_select(Line *line, Point *clicked_point,
-	    Renderer *interactive_renderer)
+	    DiaRenderer *interactive_renderer)
 {
   connection_update_handles(&line->connection);
 }
 
-static void
+static ObjectChange*
 line_move_handle(Line *line, Handle *handle,
-		 Point *to, HandleMoveReason reason, ModifierKeys modifiers)
+		 Point *to, ConnectionPoint *cp,
+		 HandleMoveReason reason, ModifierKeys modifiers)
 {
   assert(line!=NULL);
   assert(handle!=NULL);
   assert(to!=NULL);
 
-  connection_move_handle(&line->connection, handle->id, to, reason);
+  connection_move_handle(&line->connection, handle->id, to, cp, reason, modifiers);
 
   line_update_data(line);
+
+  return NULL;
 }
 
-static void
+static ObjectChange*
 line_move(Line *line, Point *to)
 {
   Point start_to_end;
@@ -248,40 +421,45 @@ line_move(Line *line, Point *to)
   point_add(&endpoints[1], &start_to_end);
 
   line_update_data(line);
+
+  return NULL;
 }
 
 static void
-line_draw(Line *line, Renderer *renderer)
+line_draw(Line *line, DiaRenderer *renderer)
 {
-  Point *endpoints;
+  Point gap_endpoints[2];
+
+  DiaRendererClass *renderer_ops = DIA_RENDERER_GET_CLASS (renderer);
   
   assert(line != NULL);
   assert(renderer != NULL);
 
-  endpoints = &line->connection.endpoints[0];
+  renderer_ops->set_linewidth(renderer, line->line_width);
+  renderer_ops->set_linestyle(renderer, line->line_style);
+  renderer_ops->set_dashlength(renderer, line->dashlength);
+  renderer_ops->set_linecaps(renderer, LINECAPS_BUTT);
 
-  renderer->ops->set_linewidth(renderer, line->line_width);
-  renderer->ops->set_linestyle(renderer, line->line_style);
-  renderer->ops->set_dashlength(renderer, line->dashlength);
-  renderer->ops->set_linecaps(renderer, LINECAPS_BUTT);
+  if (line->object_edge_start || line->object_edge_end ||
+      line->absolute_start_gap || line->absolute_end_gap ||
+      line->fractional_start_gap || line->fractional_end_gap) {
+    calculate_gap_endpoints(line, gap_endpoints);
 
-  renderer->ops->draw_line(renderer,
-			   &endpoints[0], &endpoints[1],
-			   &line->line_color);
+    renderer_ops->draw_line_with_arrows(renderer,
+					&gap_endpoints[0], &gap_endpoints[1],
+					line->line_width,
+					&line->line_color,
+					&line->start_arrow,
+					&line->end_arrow);
+  } else {
+        renderer_ops->draw_line_with_arrows(renderer,
+					    &line->connection.endpoints[0], 
+					    &line->connection.endpoints[1],
+					    line->line_width,
+					    &line->line_color,
+					    &line->start_arrow,
+					    &line->end_arrow);
 
-  if (line->start_arrow.type != ARROW_NONE) {
-    arrow_draw(renderer, line->start_arrow.type,
-	       &endpoints[0], &endpoints[1],
-	       line->start_arrow.length, line->start_arrow.width, 
-	       line->line_width,
-	       &line->line_color, &color_white);
-  }
-  if (line->end_arrow.type != ARROW_NONE) {
-    arrow_draw(renderer, line->end_arrow.type,
-	       &endpoints[1], &endpoints[0],
-	       line->end_arrow.length, line->end_arrow.width, 
-	       line->line_width,
-	       &line->line_color, &color_white);
   }
 }
 
@@ -296,13 +474,19 @@ line_create(Point *startpoint,
   Object *obj;
   Point defaultlen = { 1.0, 1.0 };
 
-  /*line_init_defaults();*/
+  line_init_defaults();
 
   line = g_malloc0(sizeof(Line));
 
   line->line_width = attributes_get_default_linewidth();
   line->line_color = attributes_get_foreground();
-  
+  line->absolute_start_gap = default_properties.absolute_start_gap;
+  line->absolute_end_gap = default_properties.absolute_end_gap;
+  line->fractional_start_gap = default_properties.fractional_start_gap;
+  line->fractional_end_gap = default_properties.fractional_end_gap;
+  line->object_edge_start = default_properties.object_edge_start;
+  line->object_edge_end = default_properties.object_edge_end;
+    
   conn = &line->connection;
   conn->endpoints[0] = *startpoint;
   conn->endpoints[1] = *startpoint;
@@ -357,6 +541,12 @@ line_copy(Line *line)
   newline->dashlength = line->dashlength;
   newline->start_arrow = line->start_arrow;
   newline->end_arrow = line->end_arrow;
+  newline->absolute_start_gap = line->absolute_start_gap;
+  newline->absolute_end_gap = line->absolute_end_gap;
+  newline->fractional_start_gap = line->fractional_start_gap;
+  newline->fractional_end_gap = line->fractional_end_gap;
+  newline->object_edge_start = line->object_edge_start;
+  newline->object_edge_end = line->object_edge_end;
 
   line_update_data(line);
 
@@ -369,6 +559,7 @@ line_update_data(Line *line)
   Connection *conn = &line->connection;
   Object *obj = &conn->object;
   LineBBExtras *extra = &conn->extra_spacing;
+  Point start, end;
 
   extra->start_trans = (line->line_width / 2.0);
   extra->end_trans   = (line->line_width / 2.0);
@@ -378,12 +569,27 @@ line_update_data(Line *line)
     extra->start_trans = MAX(extra->start_trans,line->start_arrow.width);
   if (line->end_arrow.type != ARROW_NONE) 
     extra->end_trans = MAX(extra->end_trans,line->end_arrow.width);
-  connection_update_boundingbox(conn);
+
+  if (line->object_edge_start || line->object_edge_end ||
+      line->absolute_start_gap || line->absolute_end_gap ||
+      line->fractional_start_gap || line->fractional_end_gap) {
+    Point gap_endpoints[2];
+
+    calculate_gap_endpoints(line, gap_endpoints);
+    line_bbox(&gap_endpoints[0],&gap_endpoints[1],
+	      &conn->extra_spacing,&conn->object.bounding_box);
+    start = gap_endpoints[0];
+    end = gap_endpoints[1];
+  } else {
+    connection_update_boundingbox(conn);
+    start = conn->endpoints[0];
+    end = conn->endpoints[1];
+  }
 
   obj->position = conn->endpoints[0];
 
   connpointline_update(line->cpl);
-  connpointline_putonaline(line->cpl,&conn->endpoints[0],&conn->endpoints[1]);
+  connpointline_putonaline(line->cpl,&start, &end);
   
   connection_update_handles(conn);
 }
@@ -426,6 +632,25 @@ line_save(Line *line, ObjectNode obj_node, const char *filename)
 		  line->end_arrow.width);
   }
  
+  if (line->absolute_start_gap)
+    data_add_real(new_attribute(obj_node, "absolute_start_gap"),
+                 line->absolute_start_gap);
+  if (line->absolute_end_gap)
+    data_add_real(new_attribute(obj_node, "absolute_end_gap"),
+                 line->absolute_end_gap);
+  if (line->fractional_start_gap)
+    data_add_real(new_attribute(obj_node, "fractional_start_gap"),
+                 line->fractional_start_gap);
+  if (line->fractional_end_gap)
+    data_add_real(new_attribute(obj_node, "fractional_end_gap"),
+                 line->fractional_end_gap);
+  if (line->object_edge_start)
+    data_add_boolean(new_attribute(obj_node, "object_edge_start"),
+		     line->object_edge_start);
+  if (line->object_edge_end)
+    data_add_boolean(new_attribute(obj_node, "object_edge_end"),
+		     line->object_edge_end);
+
   if (line->line_style != LINESTYLE_SOLID && line->dashlength != DEFAULT_LINESTYLE_DASHLEN)
     data_add_real(new_attribute(obj_node, "dashlength"),
 		  line->dashlength);
@@ -465,8 +690,8 @@ line_load(ObjectNode obj_node, int version, const char *filename)
     line->line_style = data_enum(attribute_first_data(attr));
 
   line->start_arrow.type = ARROW_NONE;
-  line->start_arrow.length = 0.8;
-  line->start_arrow.width = 0.8;
+  line->start_arrow.length = DEFAULT_ARROW_LENGTH;
+  line->start_arrow.width = DEFAULT_ARROW_WIDTH;
   attr = object_find_attribute(obj_node, "start_arrow");
   if (attr != NULL)
     line->start_arrow.type = data_enum(attribute_first_data(attr));
@@ -478,8 +703,8 @@ line_load(ObjectNode obj_node, int version, const char *filename)
     line->start_arrow.width = data_real(attribute_first_data(attr));
 
   line->end_arrow.type = ARROW_NONE;
-  line->end_arrow.length = 0.8;
-  line->end_arrow.width = 0.8;
+  line->end_arrow.length = DEFAULT_ARROW_LENGTH;
+  line->end_arrow.width = DEFAULT_ARROW_WIDTH;
   attr = object_find_attribute(obj_node, "end_arrow");
   if (attr != NULL)
     line->end_arrow.type = data_enum(attribute_first_data(attr));
@@ -490,6 +715,31 @@ line_load(ObjectNode obj_node, int version, const char *filename)
   if (attr != NULL)
     line->end_arrow.width = data_real(attribute_first_data(attr));
 
+  line->absolute_start_gap = 0.0;
+  attr = object_find_attribute(obj_node, "absolute_start_gap");
+  if (attr != NULL)
+    line->absolute_start_gap =  data_real( attribute_first_data(attr) );
+  line->absolute_end_gap = 0.0;
+  attr = object_find_attribute(obj_node, "absolute_end_gap");
+  if (attr != NULL)
+    line->absolute_end_gap =  data_real( attribute_first_data(attr) );
+  line->fractional_start_gap = 0;
+  attr = object_find_attribute(obj_node, "fractional_start_gap");
+  if (attr != NULL)
+    line->fractional_start_gap =  data_real( attribute_first_data(attr) );
+  line->fractional_end_gap = 0;
+  attr = object_find_attribute(obj_node, "fractional_end_gap");
+  if (attr != NULL)
+    line->fractional_end_gap =  data_real( attribute_first_data(attr) );
+  line->object_edge_start = FALSE;
+  attr = object_find_attribute(obj_node, "object_edge_start");
+  if (attr != NULL)
+    line->object_edge_start = data_boolean( attribute_first_data(attr) );
+  line->object_edge_end = FALSE;
+  attr = object_find_attribute(obj_node, "object_edge_end");
+  if (attr != NULL)
+    line->object_edge_end = data_boolean( attribute_first_data(attr) );
+
   line->dashlength = DEFAULT_LINESTYLE_DASHLEN;
   attr = object_find_attribute(obj_node, "dashlength");
   if (attr != NULL)
