@@ -21,14 +21,13 @@
 #endif
 
 #include <assert.h>
-#include <gtk/gtk.h>
 #include <math.h>
 
 #include "intl.h"
 #include "object.h"
 #include "poly_conn.h"
 #include "connectionpoint.h"
-#include "render.h"
+#include "diarenderer.h"
 #include "attributes.h"
 #include "widgets.h"
 #include "diamenu.h"
@@ -52,12 +51,13 @@ typedef struct _Polyline {
 } Polyline;
 
 
-static void polyline_move_handle(Polyline *polyline, Handle *handle,
-				   Point *to, HandleMoveReason reason, ModifierKeys modifiers);
-static void polyline_move(Polyline *polyline, Point *to);
+static ObjectChange* polyline_move_handle(Polyline *polyline, Handle *handle,
+					  Point *to, ConnectionPoint *cp,
+					  HandleMoveReason reason, ModifierKeys modifiers);
+static ObjectChange* polyline_move(Polyline *polyline, Point *to);
 static void polyline_select(Polyline *polyline, Point *clicked_point,
-			      Renderer *interactive_renderer);
-static void polyline_draw(Polyline *polyline, Renderer *renderer);
+			      DiaRenderer *interactive_renderer);
+static void polyline_draw(Polyline *polyline, DiaRenderer *renderer);
 static Object *polyline_create(Point *startpoint,
 				 void *user_data,
 				 Handle **handle1,
@@ -92,7 +92,9 @@ static ObjectType polyline_type =
   0,                         /* version */
   (char **) polyline_xpm,      /* pixmap */
   
-  &polyline_type_ops       /* ops */
+  &polyline_type_ops,       /* ops */
+  NULL, /* pixmap_file */
+  0 /* default_user_data */
 };
 
 ObjectType *_polyline_type = (ObjectType *) &polyline_type;
@@ -115,7 +117,7 @@ static ObjectOps polyline_ops = {
 };
 
 static PropDescription polyline_props[] = {
-  OBJECT_COMMON_PROPERTIES,
+  POLYCONN_COMMON_PROPERTIES,
   PROP_STD_LINE_WIDTH,
   PROP_STD_LINE_COLOUR,
   PROP_STD_LINE_STYLE,
@@ -133,7 +135,7 @@ polyline_describe_props(Polyline *polyline)
 }
 
 static PropOffset polyline_offsets[] = {
-  OBJECT_COMMON_PROPERTIES_OFFSETS,
+  POLYCONN_COMMON_PROPERTIES_OFFSETS,
   { "line_width", PROP_TYPE_REAL, offsetof(Polyline, line_width) },
   { "line_colour", PROP_TYPE_COLOUR, offsetof(Polyline, line_color) },
   { "line_style", PROP_TYPE_LINESTYLE,
@@ -176,34 +178,40 @@ static int polyline_closest_segment(Polyline *polyline, Point *point) {
 
 static void
 polyline_select(Polyline *polyline, Point *clicked_point,
-		  Renderer *interactive_renderer)
+		  DiaRenderer *interactive_renderer)
 {
   polyconn_update_data(&polyline->poly);
 }
 
-static void
+static ObjectChange*
 polyline_move_handle(Polyline *polyline, Handle *handle,
-		       Point *to, HandleMoveReason reason, ModifierKeys modifiers)
+		     Point *to, ConnectionPoint *cp,
+		     HandleMoveReason reason, ModifierKeys modifiers)
 {
   assert(polyline!=NULL);
   assert(handle!=NULL);
   assert(to!=NULL);
 
-  polyconn_move_handle(&polyline->poly, handle, to, reason);
+  polyconn_move_handle(&polyline->poly, handle, to, cp, reason, modifiers);
   polyline_update_data(polyline);
+
+  return NULL;
 }
 
 
-static void
+static ObjectChange*
 polyline_move(Polyline *polyline, Point *to)
 {
   polyconn_move(&polyline->poly, to);
   polyline_update_data(polyline);
+
+  return NULL;
 }
 
 static void
-polyline_draw(Polyline *polyline, Renderer *renderer)
+polyline_draw(Polyline *polyline, DiaRenderer *renderer)
 {
+  DiaRendererClass *renderer_ops = DIA_RENDERER_GET_CLASS (renderer);
   PolyConn *poly = &polyline->poly;
   Point *points;
   int n;
@@ -211,28 +219,18 @@ polyline_draw(Polyline *polyline, Renderer *renderer)
   points = &poly->points[0];
   n = poly->numpoints;
 
-  renderer->ops->set_linewidth(renderer, polyline->line_width);
-  renderer->ops->set_linestyle(renderer, polyline->line_style);
-  renderer->ops->set_dashlength(renderer, polyline->dashlength);
-  renderer->ops->set_linejoin(renderer, LINEJOIN_MITER);
-  renderer->ops->set_linecaps(renderer, LINECAPS_BUTT);
+  renderer_ops->set_linewidth(renderer, polyline->line_width);
+  renderer_ops->set_linestyle(renderer, polyline->line_style);
+  renderer_ops->set_dashlength(renderer, polyline->dashlength);
+  renderer_ops->set_linejoin(renderer, LINEJOIN_MITER);
+  renderer_ops->set_linecaps(renderer, LINECAPS_BUTT);
 
-  renderer->ops->draw_polyline(renderer, points, n, &polyline->line_color);
-
-  if (polyline->start_arrow.type != ARROW_NONE) {
-    arrow_draw(renderer, polyline->start_arrow.type,
-	       &points[0], &points[1],
-	       polyline->start_arrow.length, polyline->start_arrow.width,
-	       polyline->line_width,
-	       &polyline->line_color, &color_white);
-  }
-  if (polyline->end_arrow.type != ARROW_NONE) {
-    arrow_draw(renderer, polyline->end_arrow.type,
-	       &points[n-1], &points[n-2],
-	       polyline->end_arrow.length, polyline->end_arrow.width,
-	       polyline->line_width,
-	       &polyline->line_color, &color_white);
-  }
+  renderer_ops->draw_polyline_with_arrows(renderer,
+					   points, n,
+					   polyline->line_width,
+					   &polyline->line_color,
+					   &polyline->start_arrow,
+					   &polyline->end_arrow);
 }
 
 /** user_data is a struct polyline_create_data, containing an array of 
@@ -270,7 +268,7 @@ polyline_create(Point *startpoint,
     *handle1 = poly->object.handles[0];
     *handle2 = poly->object.handles[1];
   } else {
-    PolylineCreateData *pcd = (PolylineCreateData *)user_data;
+    MultipointCreateData *pcd = (MultipointCreateData *)user_data;
 
     polyconn_init(poly, pcd->num_points);
 
@@ -431,8 +429,8 @@ polyline_load(ObjectNode obj_node, int version, const char *filename)
     polyline->dashlength = data_real(attribute_first_data(attr));
 
   polyline->start_arrow.type = ARROW_NONE;
-  polyline->start_arrow.length = 0.8;
-  polyline->start_arrow.width = 0.8;
+  polyline->start_arrow.length = DEFAULT_ARROW_LENGTH;
+  polyline->start_arrow.width = DEFAULT_ARROW_WIDTH;
   attr = object_find_attribute(obj_node, "start_arrow");
   if (attr != NULL)
     polyline->start_arrow.type = data_enum(attribute_first_data(attr));
@@ -444,8 +442,8 @@ polyline_load(ObjectNode obj_node, int version, const char *filename)
     polyline->start_arrow.width = data_real(attribute_first_data(attr));
 
   polyline->end_arrow.type = ARROW_NONE;
-  polyline->end_arrow.length = 0.8;
-  polyline->end_arrow.width = 0.8;
+  polyline->end_arrow.length = DEFAULT_ARROW_LENGTH;
+  polyline->end_arrow.width = DEFAULT_ARROW_WIDTH;
   attr = object_find_attribute(obj_node, "end_arrow");
   if (attr != NULL)
     polyline->end_arrow.type = data_enum(attribute_first_data(attr));
